CREATE OR REPLACE VIEW public.view_global_final_report
AS SELECT DISTINCT semesters.year AS semesteryear,
                   semesters.num AS semesternum,
                   study_plans.externalid AS planexternalid,
                   subjects.externalid AS subjectexternalid,
                   record_books.externalid AS recordbookexternalid,
                   gd.examtype,
                   ( SELECT sum(rating_table.rate) AS sum
                     FROM rating_table
                              JOIN submodules ON rating_table.submoduleid = submodules.id
                              JOIN modules ON submodules.moduleid = modules.id
                     WHERE modules.disciplineid = gd.id AND modules.type = 'regular'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                     GROUP BY modules.disciplineid, rating_table.recordbookid) AS semesterrate,
                   ( SELECT
                         CASE
                             WHEN rt.rate IS NOT NULL THEN rt.rate
                             WHEN rt.rate IS NULL THEN
                                 CASE epo.type
                                     WHEN 'pass'::absence_pass THEN '-2'::integer
                                     WHEN 'absence'::absence_pass THEN '-1'::integer
                                     ELSE NULL::integer
                                     END
                             ELSE NULL::integer
                             END AS rate
                     FROM view_roadmap vr
                              LEFT JOIN rating_table rt ON vr.submoduleid = rt.submoduleid AND rt.recordbookid = record_books.id
                              LEFT JOIN exam_period_options epo ON epo.submoduleid = vr.submoduleid AND epo.recordbookid = record_books.id
                     WHERE vr.disciplineid = gd.id AND vr.moduletype = 'exam'::regular_exam_bonus_extra
                     ORDER BY vr.moduletype, vr.moduleordernum, vr.submoduleordernum
                     LIMIT 1) AS examrate,
                   ( SELECT
                         CASE
                             WHEN rt.rate IS NOT NULL THEN rt.rate
                             WHEN rt.rate IS NULL THEN
                                 CASE epo.type
                                     WHEN 'pass'::absence_pass THEN 0
                                     WHEN 'absence'::absence_pass THEN '-1'::integer
                                     ELSE NULL::integer
                                     END
                             ELSE NULL::integer
                             END AS rate
                     FROM view_roadmap vr
                              LEFT JOIN rating_table rt ON vr.submoduleid = rt.submoduleid AND rt.recordbookid = record_books.id
                              LEFT JOIN exam_period_options epo ON epo.submoduleid = vr.submoduleid AND epo.recordbookid = record_books.id
                     WHERE vr.disciplineid = gd.id AND vr.moduletype = 'exam'::regular_exam_bonus_extra
                     ORDER BY vr.moduletype, vr.moduleordernum, vr.submoduleordernum
                         OFFSET 1
                     LIMIT 1) AS exam2rate,
                   ( SELECT
                         CASE
                             WHEN rt.rate IS NOT NULL THEN rt.rate
                             WHEN rt.rate IS NULL THEN
                                 CASE epo.type
                                     WHEN 'pass'::absence_pass THEN 0
                                     WHEN 'absence'::absence_pass THEN '-1'::integer
                                     ELSE NULL::integer
                                     END
                             ELSE NULL::integer
                             END AS rate
                     FROM view_roadmap vr
                              LEFT JOIN rating_table rt ON vr.submoduleid = rt.submoduleid AND rt.recordbookid = record_books.id
                              LEFT JOIN exam_period_options epo ON epo.submoduleid = vr.submoduleid AND epo.recordbookid = record_books.id
                     WHERE vr.disciplineid = gd.id AND vr.moduletype = 'exam'::regular_exam_bonus_extra
                     ORDER BY vr.moduletype, vr.moduleordernum, vr.submoduleordernum
                         OFFSET 2
                     LIMIT 1) AS exam3rate,
                   ( SELECT rating_table.rate
                     FROM rating_table
                              JOIN submodules ON rating_table.submoduleid = submodules.id
                              JOIN modules ON submodules.moduleid = modules.id
                     WHERE modules.disciplineid = gd.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                     ORDER BY submodules.ordernum
                     LIMIT 1) AS extrarate,
                   ( SELECT rating_table.rate
                     FROM rating_table
                              JOIN submodules ON rating_table.submoduleid = submodules.id
                              JOIN modules ON submodules.moduleid = modules.id
                     WHERE modules.disciplineid = gd.id AND modules.type = 'extra'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                     ORDER BY submodules.ordernum
                         OFFSET 1
                     LIMIT 1) AS extra2rate,
                   ( SELECT rating_table.rate
                     FROM rating_table
                              JOIN submodules ON rating_table.submoduleid = submodules.id
                              JOIN modules ON submodules.moduleid = modules.id
                     WHERE modules.disciplineid = gd.id AND modules.type = 'bonus'::regular_exam_bonus_extra AND rating_table.recordbookid = record_books.id
                     LIMIT 1) AS bonusrate
   FROM global_disciplines
            JOIN semesters on global_disciplines.semesterid = semesters.id
            JOIN disciplines gd on gd.id = global_disciplines.disciplineid
            JOIN disciplines_students on gd.id = disciplines_students.disciplineid and disciplines_students.type = 'attach'::attach_detach
            JOIN subjects on subjects.id = gd.subjectid
            JOIN disciplines ld on ld.subjectid = gd.subjectid and ld.semesterid = gd.semesterid and ld.isglobal = TRUE and ld.isinactive = true
            JOIN disciplines_study_plans on disciplines_study_plans.disciplineid = ld.id
            JOIN study_plans ON study_plans.id = disciplines_study_plans.studyplanid
            JOIN view_disciplines_recordbooks ON view_disciplines_recordbooks.disciplineid = ld.id AND 'attach'::attach_detach = COALESCE(view_disciplines_recordbooks.type, 'attach'::attach_detach)
            JOIN record_books ON record_books.id = view_disciplines_recordbooks.recordbookid and record_books.id = disciplines_students.recordbookid
            JOIN record_books_plans ON record_books.id = record_books_plans.recordbookid AND record_books_plans.studyplanid = study_plans.id;

DROP FUNCTION IF EXISTS public.finalreport_getbysemesteryeardisciplinegroup(pyear integer, pnum integer, pdisciplineexternalid character varying, pgroupid integer);
CREATE OR REPLACE FUNCTION public.finalreport_getbysemesteryeardisciplinegroup(pyear integer, pnum integer, pdisciplineexternalid character varying, pgroupid integer)
    RETURNS TABLE("SemesterYear" integer, "SemesterNum" integer, "PlanExternalID" integer, "SubjectExternalID" character varying, "RecordBookExternalID" character varying, "ExamType" exam_credit_grading_credit, "SemesterRate" bigint, "ExamRate" integer, "Exam2Rate" integer, "Exam3Rate" integer, "ExtraRate" integer, "Extra2Rate" integer, "BonusRate" integer)
    LANGUAGE plpgsql
AS $function$
declare vSemesterId integer default -1;
BEGIN

    select semesters.id into vSemesterId
    from semesters
    where semesters.year = pyear and semesters.num = pnum;

    return query
        select DisciplineGrades.SemesterYear as "SemesterYear",
               DisciplineGrades.SemesterNum as "SemesterNum",
               DisciplineGrades.PlanExternalID as "PlanExternalDI",
               DisciplineGrades.SubjectExternalID as "SubjectExternalID",
               DisciplineGrades.RecordBookExternalID as "RecordBookExternalID",
               DisciplineGrades.ExamType as "ExamType",
               DisciplineGrades.SemesterRate as "SemesterRate",
               DisciplineGrades.ExamRate as "ExamRate",
               DisciplineGrades.Exam2Rate as "Exam2Rate",
               DisciplineGrades.Exam3Rate as "Exam3Rate",
               DisciplineGrades.ExtraRate as "ExtraRate",
               DisciplineGrades.Extra2Rate as "Extra2Rate",
               DisciplineGrades.BonusRate as "BonusRate"
        from
            (SELECT *
             FROM view_final_report
             WHERE view_final_report.SemesterYear = pyear and
                     view_final_report.SemesterNum = pnum and
                     view_final_report.SubjectExternalID = pdisciplineexternalid and
                 (view_final_report.semesterrate IS NOT NULL or
                  view_final_report.examrate IS NOT NULL or
                  view_final_report.exam2rate IS NOT NULL or
                  view_final_report.exam3rate IS NOT NULL or
                  view_final_report.extrarate IS NOT NULL or
                  view_final_report.extra2rate IS NOT null or
                  view_final_report.bonusrate IS NOT NULL)
                UNION
            SELECT *
             FROM view_global_final_report
             WHERE view_global_final_report.SemesterYear = pyear and
                     view_global_final_report.SemesterNum = pnum and
                     view_global_final_report.SubjectExternalID = pdisciplineexternalid and
                 (view_global_final_report.semesterrate IS NOT NULL or
                  view_global_final_report.examrate IS NOT NULL or
                  view_global_final_report.exam2rate IS NOT NULL or
                  view_global_final_report.exam3rate IS NOT NULL or
                  view_global_final_report.extrarate IS NOT NULL or
                  view_global_final_report.extra2rate IS NOT null or
                  view_global_final_report.bonusrate IS NOT NULL)
                ) as DisciplineGrades
                join record_books on record_books.externalid = DisciplineGrades.recordbookexternalid
                join students_groups on students_groups.recordbookid = record_books.id
        where students_groups.semesterid = vSemesterId and students_groups.groupid = pgroupid;
END
$function$;


drop function if exists public.discipline_isglobal(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.discipline_isglobal(pdisciplineid integer)
    RETURNS bool
    LANGUAGE plpgsql
AS $function$
DECLARE vIsGlobal bool DEFAULT FALSE;
begin
    select disciplines.isglobal into vIsGlobal
    from disciplines
    where disciplines.id = pdisciplineid;
    return vIsGlobal;
end;
$function$;
