DROP FUNCTION IF EXISTS public.getdisciplinesforgroup(pgroupid integer, psemesterid integer);
DROP FUNCTION IF EXISTS public.getdisciplinesforgroup(pgroupid integer, psemesterid integer, pshowhidden boolean);
CREATE OR REPLACE FUNCTION public.getdisciplinesforgroup(pgroupid integer, psemesterid integer, pshowhidden boolean default False)
    RETURNS TABLE("ID" integer, "SubjectName" character varying, "Subtype" scientific_disciplinary_coursework, "Type" exam_credit_grading_credit, "CompoundDiscID" integer, "CompoundDiscName" character varying, "CurRate" integer, "MaxRate" integer)
    LANGUAGE plpgsql
AS $function$
BEGIN
    return query
        SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
                        view_disciplines.SubjectName as "SubjectName",
                        view_disciplines.Subtype as "Subtype",
                        view_disciplines.ExamType  AS "Type",
                        view_disciplines.CompoundDiscID as "CompoundDiscID",
                        view_disciplines.CompoundDiscName as "CompoundDiscName",
                        view_disciplines.CurRate as "CurRate",
                        view_disciplines.MaxRate as "MaxRate"
        FROM students_groups
                 LEFT JOIN view_disciplines_recordbooks ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID
                 INNER JOIN view_disciplines ON view_disciplines_recordbooks.DisciplineID = view_disciplines.DisciplineID
        WHERE   students_groups.GroupID = pGroupID AND
                students_groups.SemesterID = pSemesterID AND
                view_disciplines.SemesterID = pSemesterID AND
                (NOT (view_disciplines.IsInactive) OR pshowhidden) AND
                (NOT (view_disciplines.IsGlobal) OR pshowhidden) AND
                students_groups.State <= 'outlet' AND
          -- view_disciplines_recordbooks.Type may be attach or NULL
                COALESCE(view_disciplines_recordbooks.Type, 'attach') = 'attach'
        ORDER BY view_disciplines.ExamType DESC,
                 view_disciplines.SubjectName ASC;
end;
$function$;


drop function if exists public.getdisciplinesforexport(pgroupid integer, psemesterid integer);
CREATE OR REPLACE FUNCTION public.getdisciplinesforexport(pgroupid integer, psemesterid integer)
    RETURNS TABLE("ID" integer, "SubjectName" character varying, "DisciplineExternalID" character varying,
                  "PlanExternalID" integer)
    LANGUAGE plpgsql
AS $function$
BEGIN
    return query
        select D."ID" as "ID",
               D."SubjectName" as "SubjectName",
               subjects.externalid as "DisciplineExternalID",
               study_plans.externalid as "PlanExternalID"
        from public.getdisciplinesforgroup(pgroupid, psemesterid, True) as D
                 join disciplines on disciplines.id = D."ID"
                 join subjects on disciplines.subjectid = subjects.id
                 join disciplines_study_plans on disciplines_study_plans.disciplineid = D."ID"
                 join students_groups on public.students_groups.groupid = pgroupid
                 join record_books_plans on record_books_plans.recordbookid = public.students_groups.recordbookid
                 join study_plans on disciplines_study_plans.studyplanid = study_plans.id and record_books_plans.studyplanid = study_plans.id;

end;
$function$;