DROP function if exists public.subject_getbyexternalid(psubjectexternalid character varying);
DROP function if exists public.subject_getbyexternalid(psubjectexternalid character varying, pfacultyid integer);
CREATE OR REPLACE FUNCTION public.subject_getbyexternalid(psubjectexternalid character varying, pfacultyid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vID INT DEFAULT -1;
begin
  --	select public.subject_getbyexternalid(
  --	:psubjectexternalid 	-- put the psubjectexternalid parameter value instead of 'psubjectexternalid' (varchar)
  --);
  SELECT subjects.ID INTO vID
  FROM subjects
    join subjects_faculties
      on subjects.id = subjects_faculties.subjectid
  WHERE subjects.ExternalID = pSubjectExternalID and
        subjects_faculties.facultyid = pfacultyid
  LIMIT 1;

  if vID is null then return -1;
  else  RETURN vID;
  end if;
END
$function$;


DROP FUNCTION IF EXISTS public.deletemodulebonus(pteacherid integer, pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.deletemodulebonus(pteacherid integer, pdisciplineid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vBonusModuleID INT DEFAULT -1;
begin
  --	select public.deletemodulebonus(
  --	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
  --	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
  --);

  select * from GetBonusModule(pDisciplineID) into vBonusModuleID;
  CASE WHEN  NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
             InternalIsMapLocked(pDisciplineID)
    THEN
      RETURN -1;
  else null;
  END CASE;

  CASE WHEN vBonusModuleID <= 0 THEN
    RETURN -1;
  else null;
  END CASE;

  DELETE FROM submodules
  WHERE vBonusModuleID = submodules.ModuleID;

  DELETE FROM modules
  WHERE vBonusModuleID = modules.ID;
  --LIMIT 1;

  RETURN 0;
END
$function$;

DROP FUNCTION IF EXISTS public.student_changeinfo(pid integer, pexternalid character varying, plastname character varying, pfirstname character varying, psecondname character varying);
CREATE OR REPLACE FUNCTION public.student_changeinfo(pid integer, pexternalid character varying, plastname character varying, pfirstname character varying, psecondname character varying)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vAccountID INT DEFAULT -1;
        vStudentID INT DEFAULT -1;
begin
  --	select public.student_changeinfo(
  --	:pid,	-- put the pid parameter value instead of 'pid' (int4)
  --	:pexternalid,	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
  --	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
  --	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
  --	:psecondname 	-- put the psecondname parameter value instead of 'psecondname' (varchar)
  --);
  SELECT students.AccountID INTO vAccountID
  FROM students
  WHERE students.ID = pID;

  UPDATE accounts
  SET ExternalID = COALESCE(pExternalID, ExternalID),
    LastName = COALESCE(pLastName, LastName),
    FirstName = COALESCE(pFirstName, FirstName),
    SecondName = COALESCE(pSecondName, SecondName)
  WHERE ID = vAccountID;

  RETURN 0;
  EXCEPTION
  when others then RETURN -1;
END
$function$;

alter type bachelor_master_specialist add VALUE 'postgraduate' after 'specialist';

-- TODO: что с этой функцией не так, кто ее использовал?
drop function if exists public.creategrade(pgradenum integer, pdegree bachelor_master_specialist);
CREATE OR REPLACE FUNCTION public.creategrade(pgradenum integer, pdegree bachelor_master_specialist)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
BEGIN
  --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
  IF pDegree IS NULL OR NOT (pDegree BETWEEN 'bachelor' AND 'postgraduate') THEN
    RETURN -1;
  END IF;

  -- INSERT INTO grades (Num, Degree) SELECT (pGradeNum, pDegree)
  --   ON DUPLICATE KEY UPDATE
  --     grades.ID = LAST_INSERT_ID(grades.ID);
  RETURN LAST_INSERT_ID();
END
$function$;

--
-- INSERT INTO public.grades (num,"degree") VALUES
--   (1,'postgraduate')
--   ,(2,'postgraduate')
--   ,(3,'postgraduate')
--   ,(4,'postgraduate')
-- ;

DROP FUNCTION IF EXISTS public.discipline_getstudentstemp(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.discipline_getstudentstemp(pdisciplineid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
begin
  --select grade.Discipline_GetStudentsTemp(
  ---	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
  --);
  CREATE TEMPORARY TABLE tDisciplineStudents AS (
    SELECT  record_books.StudentID,record_books.id,
      COALESCE(view_disciplines_recordbooks."type") AS "Type" ---?????
    FROM view_disciplines_recordbooks
      INNER JOIN record_books ON view_disciplines_recordbooks.RecordBookID = record_books.ID
    WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID
    GROUP BY record_books.StudentID,record_books.id,COALESCE(view_disciplines_recordbooks."type")
  );
  RETURN 0;
end;
$function$;

DROP FUNCTION IF EXISTS public.getstudentsforrating(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.getstudentsforrating(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "RecordBookID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "IsAttached" boolean)
LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1; vRes INT DEFAULT -1;
BEGIN
  vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

  DROP TABLE IF EXISTS tDisciplineStudents;
  vRes := Discipline_GetStudentsTemp(pDisciplineID);
  return query
  SELECT  view_students.StudentID 		AS "ID",
          view_students.RecordBookID		as "RecordBookID",
          view_students.LastName			as "LastName",
          view_students.FirstName			as "FirstName",
          view_students.SecondName		as "SecondName",
          view_students.GradeID			as "GradeID",
          view_students.GradeNum			as "GradeNum",
          view_students."degree"			as "Degree",
          view_students.GroupID			as "GroupID",
          view_students.GroupNum			as "GroupNum",
          ( tDisciplineStudents."Type" IS NOT NULL ) AS "IsAttached"
  FROM tDisciplineStudents
    INNER JOIN view_students ON   view_students.StudentID = tDisciplineStudents.StudentID AND
                                  view_students.SemesterID = vSemesterID
                                  and view_students.RecordBookID = tDisciplineStudents.id
  WHERE  tDisciplineStudents."Type" is null or ( tDisciplineStudents."Type" !='detach')
  ORDER BY    view_students.GradeID ASC,
    view_students.GroupNum ASC,
    view_students.LastName ASC,
    view_students.FirstName ASC;
END;
$function$;

DROP FUNCTION IF EXISTS public.get_lessons_for_discipline(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.get_lessons_for_discipline(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "LessonDate" date, "LessonType" integer)
LANGUAGE sql
AS $function$
SELECT lessons.id AS "ID",
       lessons.lesson_date AS "LessonDate",
       lessons.lesson_type AS "LessonType"
FROM lessons
WHERE lessons.discipline_id = pdisciplineid
      AND lessons.lesson_deleted = 0
order by lessons.lesson_date;
$function$;

-- -- to fix issue381 wrong year in lessons for semester 10:
-- update lessons set lesson_date = lesson_date + interval '1 year' where id in
-- (select lessons.id from lessons
--  join disciplines on lessons.discipline_id = disciplines.id
-- where disciplines.semesterid = 10);


CREATE SEQUENCE seq_logs_sync_1c
  START WITH 1
  INCREMENT BY 1
  NO MINVALUE
  NO MAXVALUE
  CACHE 1;


CREATE TABLE logs_sync_1c (
  id integer DEFAULT nextval('seq_logs_sync_1c'::regclass) NOT NULL,
  date timestamp(0) without time zone DEFAULT now() NOT NULL
);


ALTER TABLE ONLY logs_sync_1c
  ADD CONSTRAINT logs_sync_pk PRIMARY KEY (id);


CREATE OR REPLACE FUNCTION public.logs_sync_1c()
  RETURNS integer
LANGUAGE plpgsql
AS $function$
declare vID int default -1;
        vRowUpd int;
begin
  INSERT INTO logs_sync_1c DEFAULT VALUES returning id into vID;
  get diagnostics vRowUpd = ROW_COUNT;
  RETURN vRowUpd - 1;
end
$function$;

CREATE OR REPLACE FUNCTION public.getAllSyncDates()
  RETURNS TABLE("Date" timestamp)
LANGUAGE plpgsql
AS $function$
BEGIN
  return query
  SELECT  logs_sync_1c."date" as "Date"
  FROM logs_sync_1c
  ORDER BY  logs_sync_1c."date" DESC;
END
$function$;

DROP FUNCTION IF EXISTS public.changedisciplinehours(pteacherid integer, pdisciplineid integer, phours integer, ptype integer);
CREATE OR REPLACE FUNCTION public.changedisciplinehours(pteacherid integer, pdisciplineid integer, phours integer, ptype integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
declare vRow int;
BEGIN
  if NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) THEN
    RETURN -2;
  END if;

  CASE pType
    WHEN 0 THEN
    UPDATE disciplines
    set PracticeCount = pHours
    WHERE disciplines.ID = pDisciplineID;
    WHEN 1 THEN
    UPDATE disciplines
    SET LectureCount = pHours
    WHERE disciplines.ID = pDisciplineID;
    WHEN 2 THEN
    UPDATE disciplines
    SET LabCount = pHours
    WHERE disciplines.ID = pDisciplineID;
  else return -1;
  END CASE;
  get diagnostics vRow = ROW_COUNT;
  RETURN vRow -1;
END
$function$;

