 DROP TRIGGER IF EXISTS  tr_i_submodule ON submodules CASCADE; 

 DROP TRIGGER IF EXISTS  tr_u_submodule ON submodules CASCADE; 

 DROP TRIGGER IF EXISTS  tr_d_submodule ON submodules CASCADE; 

----------------------------------------------------------------


CREATE OR REPLACE FUNCTION public."DeleteModuleExam"(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
  DECLARE vExamModuleID INTEGER DEFAULT -1;
BEGIN
  -- check teacher rights
  IF  NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
      InternalIsMapLocked(pDisciplineID)
  THEN
    RETURN -1;
  END IF;

  -- get exam module ID
  SELECT modules.ID INTO vExamModuleID
  FROM modules
    WHERE   modules.Type = 'exam' AND
          modules.DisciplineID = pDisciplineID
  LIMIT 1;

  IF vExamModuleID <= 0 THEN
    RETURN -1;
  END IF;

  -- everything is checked then delete
  DELETE FROM submodules
    WHERE vExamModuleID = submodules.ModuleID;

  DELETE FROM modules
    WHERE vExamModuleID = modules.ID;
  RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public."GetStudents"(pfacultyid integer, pgradeid integer, pgroupid integer, psemesterid integer, pword1 character varying, pword2 character varying, pword3 character varying)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer)
 LANGUAGE plpgsql
AS $function$
  DECLARE FullName VARCHAR;
begin
	return query
  SELECT  view_students.StudentID AS "ID",
    view_students.LastName AS "LastName",
    view_students.FirstName AS "FirstName",
    view_students.SecondName AS "SecondName",
    view_students.AccountID AS "AccountID",
    view_students.GradeID AS "GradeID",
    view_students.GradeNum AS "GradeNum",
    view_students.Degree AS "Degree",
    view_students.GroupID AS "GroupID",
    view_students.GroupNum AS "GroupNum"
  FROM view_students
    WHERE
    CASE WHEN pSemesterID != 0 THEN view_students.SemesterID = pSemesterID ELSE
    TRUE
    END AND
      CASE WHEN pFacultyID != 0 THEN view_students.FacultyID = pFacultyID ELSE TRUE END AND
    CASE WHEN pGradeID != 0 THEN view_students.GradeID = pGradeID ELSE TRUE END AND
    CASE WHEN pGroupID != 0 THEN view_students.GroupID = pGroupID ELSE TRUE END AND
    CASE WHEN pWord1 != '' AND FullName != '' THEN
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord1, '%') AND 
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord2, '%') AND 
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord3, '%')
      ELSE TRUE END;
END
$function$;

CREATE OR REPLACE FUNCTION public."SearchStudents"(pgradeid integer, pgroupid integer, pfacultyid integer, pfullname character varying, pdisciplineid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" integer, "GroupID" integer, "GroupNum" integer)
 LANGUAGE plpgsql
AS $function$
  DECLARE vSemesterID INTEGER DEFAULT -1;
  DECLARE vRes INTEGER DEFAULT -1;
BEGIN

  DROP TABLE IF EXISTS tDisciplineStudents;
  SELECT Discipline_GetStudentsTemp(pDisciplineID) INTO vRes;

  SELECT  view_students.StudentID AS "ID",
    view_students.LastName AS "LastName",
    view_students.FirstName AS "FirstName",
    view_students.SecondName AS "SecondName",
    view_students.GradeID AS "GradeID",
    view_students.GradeNum AS "GradeNum",
    view_students.Degree AS "Degree",
    view_students.GroupID AS "GroupID",
    view_students.GroupNum AS "GroupNum",
    CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) AS "FullName"
  FROM view_students
    LEFT JOIN tDisciplineStudents ON tDisciplineStudents.StudentID = view_students.StudentID
  WHERE   view_students.SemesterID = vSemesterID AND
          view_students.FacultyID = pFacultyID AND
          view_students.GradeID = pGradeID AND
          (pGroupID = 0 OR view_students.GroupID = pGroupID) AND
          tDisciplineStudents.StudentID IS NULL AND
          CASE WHEN pFullName != '' THEN
            FullName LIKE CONCAT('%', pFullName, '%')
          ELSE TRUE END
  ORDER BY    view_students.GradeID ASC,
    view_students.GroupID ASC,
    view_students.LastName ASC,
    view_students.FirstName ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.account_createactivated(plastname character varying, pfirstname character varying, psecondname character varying, pexternalid character varying, pinila character varying, puserroleid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vID int;
begin
--	select public.account_createactivated(
--	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
--	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
--	:psecondname,	-- put the psecondname parameter value instead of 'psecondname' (varchar)
--	:pexternalid,	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--	:pinila,	-- put the pinila parameter value instead of 'pinila' (varchar)
--	:puserroleid 	-- put the puserroleid parameter value instead of 'puserroleid' (int4)
--);

        INSERT INTO accounts (ExternalID, INILA, Login , Password , EMail, LastName, FirstName, SecondName, UserRoleID)
        VALUES  ( pExternalID,pINILA,NULL, NULL, NULL, pLastName, pFirstName, pSecondName, pUserRoleID) returning id into vID;
        RETURN vID;
END;
$function$;

CREATE OR REPLACE FUNCTION public.account_getidfromexternalid(paccountexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
        DECLARE pID INT DEFAULT null;
begin
--	select public.account_getidfromexternalid(
--	:paccountexternalid 	-- put the paccountexternalid parameter value instead of 'paccountexternalid' (varchar)
--);
        SELECT accounts.ID INTO pID
        FROM accounts
        WHERE accounts.ExternalID = pAccountExternalID
        LIMIT 1;
        if pID isnull then return -1;
          else  RETURN pID;
         
         end if;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.account_getidfrominila(paccountinila character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
  DECLARE pID INT DEFAULT -1;
begin
--	select public.account_getidfrominila(
--	:paccountinila 	-- put the paccountinila parameter value instead of 'paccountinila' (varchar)
--);
        SELECT accounts.ID INTO pID
        FROM accounts
        WHERE accounts.INILA = pAccountINILA
        LIMIT 1;
         if pID is null then return -1;
          else RETURN pID;
         end if;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.activateaccount(pcode character varying, plogin character varying, pemail character varying, ppassword character varying, vaccountid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    -- check for matching with existing accounts (note: Login & E-Mail are unique)
    --EXEC SQL WHENEVER SQLWARNING SQLPRINT;
    --EXEC SQL WHENEVER SQLERROR STOP;
    
    IF CheckAccountExistence(pEMail, 'email') THEN
        RETURN -3;
    END IF;
    IF CheckAccountExistence(pLogin, 'login') THEN
        RETURN -4;
    END IF;

    -- activate account
    UPDATE accounts
        SET accounts.Login = pLogin,
            accounts.Password = SHA1(pPassword),
            accounts.EMail = pEMail,
            accounts.ActivationCode = NULL
        WHERE accounts.ActivationCode = pCode;
        vAccountID := accounts.ID; -- save accountID

    RETURN CASE WHEN ROW_COUNT() = 0 THEN  -2 ELSE  vAccountID END; -- account with this Code not found
END;

$function$;

CREATE OR REPLACE FUNCTION public.addmodule(pteacherid integer, pdisciplineid integer, pname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vOrderNum INT DEFAULT 0;
    DECLARE currID int DEFAULT 0;
BEGIN
--select public.addmodule(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pname 	-- put the pname parameter value instead of 'pname' (varchar)
--);
 
    if  (NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR InternalIsMapLocked(pDisciplineID) )THEN
        RETURN -2;
       end if;
  -- get free orderNum for module with type 'regular'
    SELECT coalesce(MAX(modules.OrderNum)+1,1) into  vOrderNum 
    FROM modules
       WHERE   modules.DisciplineID = pDisciplineID AND modules."type" = 'regular';

    INSERT INTO modules ("name", OrderNum, DisciplineID ) VALUES (pName, vOrderNum, pDisciplineID) returning  id into currID;
    RETURN currID;
--exception
--WHEN others  then return -1;
end
$function$;

CREATE OR REPLACE FUNCTION public.addmodulebonus(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vChecker INT DEFAULT -1; vModuleID INT DEFAULT -1;
begin
--	select public.addmodulebonus(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    IF  (NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
        InternalIsMapLocked(pDisciplineID))
    THEN
        RETURN -1;
    END IF;
    -- check existing of bonus module
    SELECT modules.ID INTO vChecker
        FROM modules
        WHERE   modules.DisciplineID = pDisciplineID AND
                modules."type" = 'bonus';
    IF vChecker > 0 THEN
        RETURN -2;
    END IF;

    INSERT INTO modules
        (Name, OrderNum, DisciplineID, Type)
        VALUES  ('Бонусные баллы' , 2141692 , pDisciplineID, 'bonus') RETURNING ID into  vModuleID;
    vChecker := AddSubmodule(pTeacherID, vModuleID, 10, '', NULL, 'LandmarkControl');
    RETURN 0; -- почему???
END
$function$;

CREATE OR REPLACE FUNCTION public.addmoduleexam(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vModule INT DEFAULT -1; vChecker INT DEFAULT -1;
    vIsExamExists BOOLEAN;
begin
--	select public.addmoduleexam(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    IF (NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
        InternalIsMapLocked(pDisciplineID))
    THEN
        RETURN -1;
    END IF;

    RETURN AddModuleExamUnsafe(pDisciplineID);
END 
$function$;

CREATE OR REPLACE FUNCTION public.addmoduleexamunsafe(pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vModule INT DEFAULT -1; vChecker INT DEFAULT -1;
    vIsExamExists BOOLEAN; vTemp int;
BEGIN
--select public.AddModuleExamUnsafe(
--	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
--);
    IF InternalIsMapLocked(pDisciplineID) THEN
        RETURN -1;
    END IF;

    vIsExamExists := EXISTS(
        SELECT * FROM modules
        WHERE modules.DisciplineID = pDisciplineID AND modules.Type = 'exam'
        LIMIT 1
    );
    IF vIsExamExists THEN
        RETURN -2;
    END IF;

    INSERT INTO modules (Name, OrderNum, DisciplineID, Type) VALUES  ('Экзамен' , 3141592 , pDisciplineID, 'exam') 
   				returning  ID into vModule;
    -- 3 attempt for pass exam -- разобраться почему так?
    SELECT AddSubmoduleUnsafe(vModule, 40, '', NULL, 'LandmarkControl') INTO vTemp;
    SELECT AddSubmoduleUnsafe(vModule, 40, '', NULL, 'LandmarkControl') INTO vTemp;
    SELECT AddSubmoduleUnsafe(vModule, 40, '', NULL, 'LandmarkControl') INTO vTemp;
    RETURN vModule;
END;
$function$;

CREATE OR REPLACE FUNCTION public.addmoduleextra(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vChecker INT DEFAULT -1; vModule INT DEFAULT -1; vType INT DEFAULT -1; vGap INT DEFAULT -1;
begin
--	select public.AddModuleExtra(
--	:pTeacherID,	-- put the pTeacherID parameter value instead of 'pTeacherID' (INT)
--	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
--);

    IF  NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID)
    THEN
        RETURN -1;
    END IF;
    -- try to find existing extra module
    SELECT modules.ID INTO vChecker
        FROM modules
        WHERE modules.DisciplineID = pDisciplineID AND modules.Type = 'extra'
        LIMIT 1;
    IF vChecker > 0 THEN
        RETURN -2;
    END IF;

    -- add extra module
    INSERT INTO modules
        (Name, OrderNum, DisciplineID, Type)
        VALUES ('Добор баллов' , 2900666 , pDisciplineID, 'extra');

	-- get discipline exam type
    SELECT  modules.ID, disciplines.ExamType INTO vModule, vType
        FROM modules
        INNER JOIN disciplines ON disciplines.ID = modules.DisciplineID
        WHERE   modules.DisciplineID = pDisciplineID AND
                modules.Type = 'extra'
        LIMIT 1;
    IF vModule <= 0 THEN
        RETURN -1;
    END IF;

    -- 1 extra attempt for exam and 2 for credit
    vGap := -1;
    IF vType = 1 THEN -- exam
        vGap := 7;
    END IF;
    IF vType = 2 OR vType = 3 THEN -- credit, grading_credit
        vGap := 29;
        vChecker := addsubmodule(pTeacherID, vModule, vGap, '', NULL, 'LandmarkControl');
    END IF;

    vChecker := AddSubmodule(pTeacherID, vModule, vGap, '', NULL, 'LandmarkControl');
    RETURN vModule;
--   exception 
--when others then return -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.addsubmodule(pteacherid integer, pmoduleid integer, pmaxrate integer, pname character varying, pdescription character varying, pcontroltype character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vIsLocked BOOLEAN DEFAULT TRUE;
   vID integer default null;
begin
--	select public.addsubmodule(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pmoduleid,	-- put the pmoduleid parameter value instead of 'pmoduleid' (int4)
--	:pmaxrate,	-- put the pmaxrate parameter value instead of 'pmaxrate' (int4)
--	:pname,	-- put the pname parameter value instead of 'pname' (varchar)
--	:pdescription,	-- put the pdescription parameter value instead of 'pdescription' (varchar)
--	:pcontroltype 	-- put the pcontroltype parameter value instead of 'pcontroltype' (varchar)
--);

    -- check author and discipline lock
    SELECT disciplines.AuthorID into vID
         FROM modules
        INNER JOIN disciplines ON  disciplines.ID = modules.DisciplineID
        WHERE modules.ID = pModuleID
        LIMIT 1;
       IF vID is null or vID !=pTeacherID  THEN
        RETURN -2;
    END IF;
    
 RETURN AddSubmoduleUnsafe(pModuleID, pMaxRate, pName, pDescription, pControlType);

END;
$function$;

CREATE OR REPLACE FUNCTION public.addsubmoduleunsafe(pmoduleid integer, pmaxrate integer, pname character varying, pdescription character varying, pcontroltype character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
--select public.addsubmoduleunsafe(
--	:pmoduleid,	-- put the pmoduleid parameter value instead of 'pmoduleid' (int4)
--	:pmaxrate,	-- put the pmaxrate parameter value instead of 'pmaxrate' (int4)
--	:pname,	-- put the pname parameter value instead of 'pname' (varchar)
--	:pdescription,	-- put the pdescription parameter value instead of 'pdescription' (varchar)
--	:pcontroltype 	-- put the pcontroltype parameter value instead of 'pcontroltype' (varchar)
--);
    DECLARE vOrderNum INT DEFAULT -1; vIsLocked INT DEFAULT -1; vDescription VARCHAR; recId int ;
BEGIN
    -- check discipline lock
    SELECT disciplines.IsLocked INTO vIsLocked
        FROM modules
        INNER JOIN disciplines ON disciplines.ID = modules.DisciplineID
        WHERE modules.ID = pModuleID
        LIMIT 1;
    IF  vIsLocked != 0 THEN
        RETURN -2;
    END IF;
    -- get free order
        SELECT COALESCE(MAX(submodules.OrderNum),0)+1 FROM submodules
        WHERE submodules.ModuleID = pModuleID LIMIT 1 into vOrderNum;
         if (pDescription = '')   THEN   vDescription = NULL ;
          ELSE  vDescription = pDescription; 
         end if ; -- была очень странная проверка (vDescription = '')
         if (pControlType ='') then pControlType = 'CurrentControl';
        end if;
        INSERT INTO submodules (ModuleID, MaxRate, OrderNum, "name", Description, "type") VALUES
        (pModuleID, pMaxRate, vOrderNum, pName, vDescription, cast ( pControlType as current_landmark_control)) returning id into recID;
    RETURN  recID;
   
 --exception 
-- when others then return -1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.bindgroup(pteacherid integer, pdisciplineid integer, pgroupid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vres int;
begin
--	select public.bindgroup(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pgroupid,	-- put the pgroupid parameter value instead of 'pgroupid' (int4)
--	:vres 	-- put the vres parameter value instead of 'vres' (int4)
--);

    -- todo: move to php layer
   IF (pTeacherID IS NOT NULL AND NOT InternalIsTeacherBound(pTeacherID, pDisciplineID))
       OR InternalIsMapLocked(pDisciplineID) THEN
        RETURN -1;
    END IF;
    -- already bound
   IF EXISTS(
        SELECT * FROM disciplines_groups
        WHERE disciplines_groups.DisciplineID = pDisciplineID AND disciplines_groups.GroupID = pGroupID
    ) THEN
        RETURN 1;
    END IF;

    -- bind whole group
    INSERT INTO disciplines_groups (DisciplineID, GroupID) SELECT ( pDisciplineID, pGroupID );
    SELECT LogBind(pDisciplineID, pTeacherID, pGroupID, 'attach', 'group') INTO vRes;
    RETURN RemoveStudentsAttach(pDisciplineID, pGroupID); -- delete students of this group which were bound to discipline before
  --  exception
--when others then RETURN -3;
END
$function$;

CREATE OR REPLACE FUNCTION public.bindstudent(pteacherid integer, pdisciplineid integer, pstudentid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vStudentGroupID INT DEFAULT -1; vSemesterID INT DEFAULT -1; vInGroup INT DEFAULT -1;
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
    --EXEC SQL WHENEVER SQLWARNING SQLPRINT;
    --EXEC SQL WHENEVER SQLERROR STOP;
    
    -- todo: move to php layer
    CASE WHEN NOT InternalIsTeacherBound(pTeacherID, pDisciplineID) THEN
        RETURN -1;
    END CASE;


    --SET vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');
    vStudentGroupID := GetStudentGroup(pStudentID, vSemesterID);

    -- check if student's group is bound yet
    vInGroup := EXISTS(
        SELECT * FROM disciplines_groups
            WHERE disciplines_groups.DisciplineID = pDisciplineID AND disciplines_groups.GroupID = vStudentGroupID
    );

    -- bind student
    IF vInGroup THEN -- student in group -> try to remove detached attribute
        DELETE FROM disciplines_students
            WHERE   disciplines_students.DisciplineID = pDisciplineID AND disciplines_students.StudentID = pStudentID;
    ELSE -- bind stand alone student ;(
        --INSERT INTO disciplines_students (DisciplineID, StudentID, Type) VALUES (pDisciplineID, pStudentID, 'attach')
           -- ON DUPLICATE KEY UPDATE
              --  disciplines_students.Type = 'attach';
    END IF;

    RETURN LogBind(pDisciplineID, pTeacherID, pStudentID, 'attach', 'student');
END
$function$;

CREATE OR REPLACE FUNCTION public.calculatemaxrateforextra(pdisciplineid integer, precordbookid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vExamType exam_credit_grading_credit DEFAULT NULL;
    vDiscID INT DEFAULT 0; vLim INT DEFAULT 0; vResult INT DEFAULT 0;
BEGIN
--select public.calculatemaxrateforextra(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pRecordBookID 	-- put the pRecordBookID parameter value instead of 'pRecordBookID' (int4)
--);
    -- get disc type
    --vExamType := GetDisciplineProperty(pDisciplineID, 'type');
    vExamType := GetDisciplineTypeExam(pDisciplineID);
    -- submodule doesn't exists
    IF vExamType IS NULL --OR vExamType <= 0 
    THEN
        RETURN -1;
    END IF;
    vLim := CASE WHEN vExamType = 'exam' THEN  38 ELSE  60 END;

    SELECT SUM(CASE WHEN view_roadmap.ModuleType = 'regular' THEN  rating_table.Rate ELSE  0 END) INTO vResult
        FROM view_roadmap
        LEFT JOIN rating_table
        --ON rating_table.StudentID = pStudentID AND
        ON rating_table.recordbookid = pRecordBookID AND
                                    rating_table.SubmoduleID = view_roadmap.SubmoduleID
        WHERE view_roadmap.DisciplineID = pDisciplineID
        LIMIT 1;
    RETURN vLim - vResult;
END;
$function$;

CREATE OR REPLACE FUNCTION public.changeaccountdata(puserid integer, pdata text, ptype login_email_password)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    --EXEC SQL WHENEVER SQLWARNING SQLPRINT;
    --EXEC SQL WHENEVER SQLERROR STOP;
    
    CASE pType
        WHEN 'login' THEN
            UPDATE accounts
                SET accounts.Login = pData
                WHERE accounts.ID = pUserID;

        WHEN 'email' THEN
            UPDATE accounts
                SET accounts.EMail = pData
                WHERE accounts.ID = pUserID;

        WHEN 'password' THEN
            UPDATE accounts
                SET accounts.Password = SHA1(pData)
                WHERE accounts.ID = pUserID;

    END CASE;

    RETURN 1;
END
$function$;

CREATE OR REPLACE FUNCTION public.changedisciplinecontrol(pteacherid integer, pdisciplineid integer, pexamtype exam_credit_grading_credit)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

 DECLARE vChecker INT DEFAULT -1; vExtraMax INT DEFAULT -1; vExtraID INT DEFAULT -1;
 DECLARE vOldExamType exam_credit_grading_credit;
-- DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
BEGIN
    -- todo: move to php
    if  InternalIsMapLocked(pDisciplineID) OR NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) THEN
        RETURN -1;
    END IF;

    -- get exam type and extra module ID
    SELECT disciplines.ExamType, modules.ID INTO vOldExamType, vExtraID
        FROM modules
        INNER JOIN disciplines ON disciplines.ID = modules.DisciplineID
        WHERE   modules.DisciplineID = pDisciplineID AND modules."type" = 'extra'
        LIMIT 1;
    IF vExtraID <= 0 THEN
        RETURN -1;
    END IF;

    -- check type changing: exam <-> credit/grading_credit
    IF NOT (vOldExamType = 'exam' OR pExamType != 'exam') THEN

        -- TODO: extract method addExtraModule
        IF pExamType = 'exam' THEN -- change to exam
            -- count discipline's current max rate
            vChecker := GetDisciplineProperty(pDisciplineID, 'maxrate');
            IF vChecker >= 61 THEN -- can't add exam module ( > 100 points)
                RETURN 1;
            END IF;
            vChecker := AddModuleExamUnsafe(pDisciplineID);

            -- delete extra submodules(only 1 extra for exam)
            DELETE FROM submodules
                WHERE submodules.OrderNum > 1 AND submodules.ModuleID = vExtraID;

            vExtraMax := 7;
        ELSE -- change to credit
            vExtraMax := 29;
            vChecker := DeleteModuleExam(pTeacherID, pDisciplineID);
            -- 2 extra submodules (1 already created for exam)
            vChecker := AddSubmodule(pTeacherID, vExtraID, vExtraMax, '', NULL, 'LandmarkControl');
        END IF;
    END IF;

    -- set new exam type
   
    UPDATE disciplines
        set disciplines.ExamType = pExamType
        WHERE disciplines.ID = pDisciplineID;
        --LIMIT 1;

    -- set max rate for extra
    UPDATE submodules
        SET submodules.MaxRate = vExtraMax
        WHERE submodules.ModuleID = vExtraID;
    RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public.changedisciplinehours(pteacherid integer, pdisciplineid integer, phours integer, ptype integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    CASE WHEN NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) THEN
        RETURN -1;
    END CASE;

    CASE pType
    --WHEN 0 THEN
        --UPDATE disciplines
           -- disciplines.PracticeCount := pHours
            --WHERE disciplines.ID = pDisciplineID
            --LIMIT 1;
    WHEN 1 THEN
        UPDATE disciplines
            SET disciplines.LectureCount = pHours
            WHERE disciplines.ID = pDisciplineID;
            --LIMIT 1;
    WHEN 2 THEN
        UPDATE disciplines
            SET disciplines.LabCount = pHours
            WHERE disciplines.ID = pDisciplineID;
            --LIMIT 1;
    END CASE;
    RETURN ROW_COUNT()-1;
END
$function$;

CREATE OR REPLACE FUNCTION public.changedisciplinesubject(pteacherid integer, pdisciplineid integer, psubjectid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    -- TODO: move to php
    CASE WHEN  NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
        InternalIsMapLocked(pDisciplineID)
    THEN
        RETURN -1;
    END CASE;

    RETURN ChangeDisciplineSubjectUnsafe(pDisciplineID, pSubjectID);
END
$function$;

CREATE OR REPLACE FUNCTION public.changedisciplinesubjectunsafe(pdisciplineid integer, psubjectid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
    --EXEC SQL WHENEVER SQLWARNING SQLPRINT;
    --EXEC SQL WHENEVER SQLERROR STOP;
    
    UPDATE disciplines
        SET disciplines.SubjectID = pSubjectID
        WHERE disciplines.ID = pDisciplineID;
        --LIMIT 1;
    RETURN ROW_COUNT()-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.changemodulename(pteacherid integer, pmoduleid integer, pname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vDisciplineID INT DEFAULT -1;
    vAlreadySameName BOOL DEFAULT FALSE;
    rowUpd int;
BEGIN
--select public.changesubmoduledescription(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psubmoduleid,	-- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--	:pdescription 	-- put the pdescription parameter value instead of 'pdescription' (varchar)
--);

     SELECT DisciplineID FROM modules WHERE ID = pModuleID LIMIT 1 into vDisciplineID;
    IF InternalIsMapLocked(vDisciplineID) THEN
        RETURN -1;
    END IF;

    -- Bodging, strange row_count() behaviour in some cases.
    -- i.e. update 1 module row. In case it's new and old names are same, row_count value may be 0.
    vAlreadySameName = EXISTS(
        SELECT * FROM modules
        WHERE modules.ID = pModuleID AND modules."type"='regular' AND modules."name" = pName
        LIMIT 1
    );
    IF vAlreadySameName THEN
        return 0;
    END IF;

    UPDATE modules
        SET "name" = pName
        WHERE modules.ID = pModuleID AND modules."type"='regular';
        --LIMIT 1;
        get diagnostics rowUpd = ROW_COUNT;
    RETURN rowUpd-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.changepersonaldataforteacher(teacherid integer, teacherlast character varying, teacherfirst character varying, teachersecond character varying, jobpositionid integer, departmentid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
  
	DECLARE checker INT;
BEGIN
	checker := -1;

	SELECT teachers.ID
	INTO checker
	FROM teachers
	INNER JOIN job_positions ON JobPositionID = job_positions.ID
	INNER JOIN departments ON DepartmentID = departments.ID
	WHERE TeacherID = teachers.ID
	LIMIT 1;
	IF 	TeacherLast IS NULL OR
		TeacherSecond IS NULL OR
		TeacherSecond IS NULL OR  
		checker <= 0 
	THEN 
		RETURN -1;
	END IF;

	UPDATE teachers
	SET 	teachers.LastName = TeacherLast,
			teachers.FirstName = TeacherFirst,
			teachers.SecondName = TeacherSecond,
			teachers.JobPositionID = JobPositionID,
			teachers.DepartmentID = DepartmentID
	WHERE TeacherID = teachers.ID;
	--LIMIT 1;
	RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public.changesubmoduledescription(pteacherid integer, psubmoduleid integer, pdescription character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vIsLocked INT DEFAULT -1;
	declare test int default 0;
BEGIN
--select public.changesubmoduledescription(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psubmoduleid,	-- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--	:pdescription 	-- put the pdescription parameter value instead of 'pdescription' (varchar)
--);

    SELECT disciplines.IsLocked INTO vIsLocked
        FROM view_roadmap
        INNER JOIN disciplines ON disciplines.ID = view_roadmap.DisciplineID
        WHERE   view_roadmap.SubmoduleID = pSubmoduleID AND disciplines.AuthorID = pTeacherID
        LIMIT 1;
    IF  vIsLocked != 0 THEN
        RETURN -1;
    END IF;

    UPDATE submodules
        SET submodules.Description  = pDescription
        WHERE submodules.ID = pSubmoduleID;	    
        --LIMIT 1;
    RETURN 0;
END;

$function$;

CREATE OR REPLACE FUNCTION public.changesubmodulemaxandcontrol(pteacherid integer, psubmoduleid integer, pmaxrate integer, pcontroltype character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vChecker INT DEFAULT -1; 
   			vDisciplineID INT DEFAULT -1; 
   			vIsLocked INT DEFAULT -1; 
   			vNewDiscMaxRate INT DEFAULT -1; 
   			vCurRate INT DEFAULT -1;
   		    rowUpd INT;
BEGIN
--select public.changesubmodulemaxandcontrol(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psubmoduleid,	-- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--	:pmaxrate,	-- put the pmaxrate parameter value instead of 'pmaxrate' (int4)
--	:pcontroltype 	-- put the pcontroltype parameter value instead of 'pcontroltype' (varchar)
--);
    -- check that discipline and submodule exists and doesn't locked
    SELECT  disciplines.IsLocked,
            disciplines.MaxRate - submodules.MaxRate + pMaxRate
            INTO vIsLocked, vNewDiscMaxRate
        FROM submodules
        INNER JOIN modules        ON  submodules.ModuleID = modules.ID
        INNER JOIN disciplines    ON  disciplines.ID = modules.DisciplineID
        WHERE submodules.ID = pSubmoduleID AND
              disciplines.AuthorID = pTeacherID
        LIMIT 1;
    IF  vIsLocked != 0 OR
        vNewDiscMaxRate > 100
    THEN
        RETURN -1;
    END IF;

    UPDATE submodules
        SET     submodules.MaxRate = pMaxRate,
                submodules."type" = pControlType
        WHERE submodules.ID = pSubmoduleID;
        --LIMIT 1;
    get diagnostics rowUpd = ROW_COUNT;
    RETURN rowUpd-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.changesubmodulename(pteacherid integer, psubmoduleid integer, pname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vIsLocked INT DEFAULT -1;
BEGIN
--select public.changesubmodulename(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psubmoduleid,	-- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--	:pname 	-- put the pname parameter value instead of 'pname' (varchar)
--);

    SELECT disciplines.IsLocked INTO vIsLocked
        FROM view_roadmap
        INNER JOIN disciplines ON  disciplines.ID = view_roadmap.DisciplineID
        WHERE  view_roadmap.SubmoduleID = pSubmoduleID AND disciplines.AuthorID = pTeacherID
        LIMIT 1;
    IF  vIsLocked != 0 THEN
        RETURN -1;
    END IF;

    UPDATE submodules
        SET submodules."name" = pName
        WHERE submodules.ID = pSubmoduleID;
        --LIMIT 1;
    RETURN 0;
END;
$function$;

CREATE OR REPLACE FUNCTION public.changeteacherinfo(pexternalid character varying, pinila character varying, pteacherid integer, plastname character varying, pfirstname character varying, psecondname character varying, pjobpositionid integer, pdepartmentid integer, pstatus boolean)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vRows integer;
 DECLARE vAccountID INT DEFAULT -1;
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
       SELECT accounts.ID
        INTO vAccountID
        FROM accounts
        INNER JOIN teachers ON accounts.ID = teachers.AccountID
        WHERE teachers.ID = pTeacherID; 
    -- set new info
    UPDATE accounts
    SET accounts.ExternalID = pExternalID,
        accounts.INILA = pINILA,
        accounts.LastName = pLastName,
        accounts.FirstName = pFirstName,
        accounts.SecondName = pSecondName
    WHERE accounts.ID = vAccountID;
   
    UPDATE teachers
        SET teachers.LastName = pLastName,
            teachers.FirstName = pFirstName,
            teachers.SecondName = pSecondName,
            teachers.JobPositionID = pJobPositionID,
            teachers.DepartmentID = pDepartmentID
        WHERE teachers.ID = pTeacherID;
        --LIMIT 1;
        get diagnostics vRows = ROW_COUNT;
    RETURN vRows-1;
  -- EXCEPTION 
  -- when others then RETURN -1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.changeteacherinfo(pteacherid integer, plastname character varying, pfirstname character varying, psecondname character varying, pjobpositionid integer, pdepartmentid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vRows integer;
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
    
    -- set new info
    UPDATE teachers
        SET teachers.LastName = pLastName,
            teachers.FirstName = pFirstName,
            teachers.SecondName = pSecondName,
            teachers.JobPositionID = pJobPositionID,
            teachers.DepartmentID = pDepartmentID
        WHERE teachers.ID = pTeacherID;
        --LIMIT 1;
        get diagnostics vRows = ROW_COUNT;
    RETURN vRows-1;
  -- EXCEPTION 
  -- when others then RETURN -1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.checkaccountexistence(pdata text, ptype login_email_code)
 RETURNS boolean
 LANGUAGE plpgsql
AS $function$
begin
--	select public.checkaccountexistence(
--	:pdata,	-- put the pdata parameter value instead of 'pdata' (text)
--	:ptype 	-- put the ptype parameter value instead of 'ptype' (login_email_code)
--);

RETURN EXISTS(
    SELECT * FROM accounts
        WHERE CASE pType
            WHEN 'login' THEN pData = accounts.Login
            WHEN 'email' THEN pData = accounts.EMail
            WHEN 'code' THEN pData = accounts.ActivationCode
            ELSE FALSE
        END
    LIMIT 1
);
end
$function$;

CREATE OR REPLACE FUNCTION public.controlstudentgroup(precordbookid integer, pgroupid integer, pstate common_outlet_expulsion_leave, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
DECLARE vChecker INT DEFAULT 0;
begin
--	select public.controlstudentgroup(
--	:precordbookid,	-- put the precordbookid parameter value instead of 'precordbookid' (int4)
--	:pgroupid,	-- put the pgroupid parameter value instead of 'pgroupid' (int4)
--	:pstate,	-- put the pstate parameter value instead of 'pstate' (common_outlet_expulsion_leave)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
    INSERT INTO students_groups (RecordBookID, GroupID, SemesterID, State)
        VALUES (pRecordBookID, pGroupID, pSemesterID, pState)
        ON conflict on CONSTRAINT students_groups_semesterid_recordbookid_key
          do update set
            students_groups.GroupID = pGroupID,
            students_groups.State = pState,
            students_groups.Date = current_date;
    get diagnostics vChecker=ROW_COUNT;
    RETURN vchecker-1;
   --EXCEPTION 
   -- when others then RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.createaccount(pLastName character varying, pFirstName character varying, pSecondName character varying, pcode character varying, puserroleid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare insID int;
begin
--	select public.createaccount(
--	:pcode,	-- put the pcode parameter value instead of 'pcode' (varchar)
--	:puserroleid 	-- put the puserroleid parameter value instead of 'puserroleid' (int4)
--);

    INSERT INTO accounts (Login , Password , EMail, UserRoleID, ActivationCode, FirstName, SecondName, LastName)
        VALUES  ( NULL, NULL, NULL, pUserRoleID, pCode, pFirstName, pSecondName, pLastName) returning ID into insID;
    RETURN insID;
END;
$function$;

CREATE OR REPLACE FUNCTION public.createauthtoken(paccountid integer, pdescription character varying, prightmask integer)
 RETURNS character varying
 LANGUAGE plpgsql
AS $function$

    DECLARE vTries int DEFAULT 13; -- number of tries to generate unique token
    vCreated boolean DEFAULT FALSE;
 --   vSeed int DEFAULT FLOOR(4294967296 * RAND(CURRENT_TIMESTAMP ^ LAST_INSERT_ID() ^ (pAccountID << 10)));
    vToken char(40);
BEGIN
--select public.createauthtoken(
--	:paccountid,	-- put the paccountid parameter value instead of 'paccountid' (int4)
--	:pdescription,	-- put the pdescription parameter value instead of 'pdescription' (varchar)
--	:prightmask 	-- put the prightmask parameter value instead of 'prightmask' (int4)
--);
    SELECT setseed(FLOOR(4294967296 * (CURRENT_TIMESTAMP # (pAccountID << 10)))); -- ???????
    WHILE (NOT vCreated AND vTries > 0) loop
    begin
    
        vToken := SHA1(RANDOM());
        vTries := vTries - 1;
        vCreated := TRUE;  
        INSERT INTO auth_tokens(Token, AccountID, Description, Mask) VALUES (vToken, pAccountID, pDescription, pRightMask);
   		--EXCEPTION 
   		--	when others 
   		--	then  vCreated := FALSE; 
   	end;
   END loop; 

    RETURN IF(vCreated, vToken, NULL);
END
$function$;

CREATE OR REPLACE FUNCTION public.creategrade(pgradenum integer, pdegree bachelor_master_specialist)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
    IF pDegree IS NULL OR NOT (pDegree BETWEEN 'bachelor' AND 'specialist') THEN
        RETURN -1;
    END IF;

   -- INSERT INTO grades (Num, Degree) SELECT (pGradeNum, pDegree)
     --   ON DUPLICATE KEY UPDATE
       --     grades.ID = LAST_INSERT_ID(grades.ID);
    RETURN LAST_INSERT_ID();
END
$function$;

CREATE OR REPLACE FUNCTION public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
 DECLARE vGroupID INT DEFAULT null; 
         vSpecId INT DEFAULT null; 
         vGroupYear INT DEFAULT null; 
         vIsSpecMatch BOOL DEFAULT null;
   
BEGIN
--select public.creategroup(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pgroupnum,	-- put the pgroupnum parameter value instead of 'pgroupnum' (int4)
--	:pspecname,	-- put the pspecname parameter value instead of 'pspecname' (varchar)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:pyear 	-- put the pyear parameter value instead of 'pyear' (int4)
--);    
-- create specialization vSpecId - UNIQUE (pSpecName,pFacultyID)
    select specializations.id into vSpecId
           from specializations
           where specializations.facultyid=pFacultyID and specializations."name" like pSpecName;
    if (vSpecId is null) then
         INSERT INTO specializations (Name, Abbr, FacultyID) 
         VALUES  (pSpecName, NULL, pFacultyID)
         returning id into vSpecId;
    end if;


-- create group  vGroupID - UNIQUE (pGradeID, pGroupNum, pFacultyID)
    select study_groups.id into vGroupID 
           from study_groups
           where study_groups.gradeid=pgradeID 
           		 and study_groups.groupnum= pGroupNum
           		 and study_groups.facultyid=pFacultyID;
    if ( vGroupID  is null) then
        INSERT INTO study_groups (GradeID, GroupNum, FacultyID) 
               VALUES (pGradeID, pGroupNum, pFacultyID)
               returning id into  vGroupID ;
    end if;
        

    SELECT groups_years.groupid, groups_years.specializationid = vSpecId
        INTO vGroupYear, vIsSpecMatch
        FROM groups_years
        WHERE groups_years.groupid = vGroupID AND groups_years."year" = pYear
        LIMIT 1;

        --RAISE notice 'vgroupyear = (%) visspecmatch= (%)', vGroupYear, vIsSpecMatch;
        
    IF  (vGroupYear is null) THEN
        INSERT INTO groups_years (GroupID, Year, SpecializationID)
        VALUES (vGroupID, pYear, vSpecId);
    ELSEIF NOT vIsSpecMatch THEN
        RETURN -1;
    END IF;

    RETURN vGroupID;
   -- EXCEPTION 
    --	when others then 
   	--		RETURN -1;
END
$function$
;

CREATE OR REPLACE FUNCTION public.createoraddstudent(pexternalid character varying, plastname character varying, pfirstname character varying, psecondname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
        DECLARE vAccountID INT DEFAULT -1; 
                vStudentID INT DEFAULT -1;
begin
--	select public.createoraddstudent(
--	:pexternalid,	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
--	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
--	:psecondname 	-- put the psecondname parameter value instead of 'psecondname' (varchar)
--);
        vAccountID = CreateAccount(pLastName, pFirstName, pSecondName, null, GetUserRole('student'));

        IF pExternalID is not null THEN
            UPDATE accounts
                SET ExternalID = pExternalID
                WHERE ID = vAccountID;
        END IF;

       --create student
        INSERT INTO students (AccountID)
        VALUES  (vAccountID) returning ID into vStudentID;
        RETURN vStudentID;
     --  EXCEPTION 
      --  when others then RETURN -1;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.createrecoverytoken(paccountoremail character varying, ptoken character varying)
 RETURNS character varying
 LANGUAGE plpgsql
AS $function$
    DECLARE vAccountID INT DEFAULT null;
    DECLARE vUserFullName TEXT; 
BEGIN
--select public.createrecoverytoken(
--	:paccountoremail,	-- put the paccountoremail parameter value instead of 'paccountoremail' (varchar)
--	:ptoken 	-- put the ptoken parameter value instead of 'ptoken' (varchar)
--);
	-- get account ID
    SELECT accounts.ID INTO vAccountID
        FROM accounts
        WHERE accounts.EMail = pAccountOrEMail OR
              accounts.Login = pAccountOrEMail
        LIMIT 1;
    IF vAccountID is null THEN
        RETURN '';
    END IF;

    vUserFullName := GetUserFullNameByAccountID(vAccountID);
    IF vUserFullName IS NULL OR vUserFullName = '' THEN
        RETURN '';
    END IF;

    -- transform all unused recovery tokens into used
    UPDATE recovery_tokens
    SET recovery_tokens.isUsed = 1
    WHERE   recovery_tokens.isUsed = 0 AND
            recovery_tokens.AccountID = vAccountID;

    -- handle catch constraints violations
    INSERT INTO recovery_tokens
        ( AccountID, Token )
        VALUES  (vAccountID, pToken);
    RETURN vUserFullName;
  -- EXCEPTION 
  -- when others then  RETURN -2; -- тип ???
END
$function$;

CREATE OR REPLACE FUNCTION public.createrequest(paccountid integer, ptitle character varying, pdescription text, pimage boolean)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare ROWID integer; hi integer default 0;
begin
--	select public.createrequest(
--	:paccountid,	-- put the paccountid parameter value instead of 'paccountid' (int4)
--	:ptitle,	-- put the ptitle parameter value instead of 'ptitle' (varchar)
--	:pdescription,	-- put the pdescription parameter value instead of 'pdescription' (text)
--	:pimage 	-- put the pimage parameter value instead of 'pimage' (bool)
--);

    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
    
    if pImage then hi=1; --несовпадение типов boolean и integer
    end if;
    
    INSERT INTO requests (AccountID, Title, Description, Status, HasImage)
         --  VALUES  (pAccountID, pTitle, pDescription, 'opened', pImage) 
         VALUES  (pAccountID, pTitle, pDescription, 'opened', hi) 
           returning id into rowid;
    RETURN rowid;
  -- exception
  -- when others then return -1;
   
END
$function$;

CREATE OR REPLACE FUNCTION public.createstudentex(plastname character varying, pfirstname character varying, psecondname character varying, pgradenum integer, pgroupnum integer, pdegree bachelor_master_specialist, pspecname character varying, pfacultyid integer, pactivationcode character varying, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

DECLARE  vGradeID INT DEFAULT -1;  vGroupID INT DEFAULT -1; vYear INT DEFAULT -1;
begin
--	select public.createstudentex(
--	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
--	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
--	:psecondname,	-- put the psecondname parameter value instead of 'psecondname' (varchar)
--	:pgradenum,	-- put the pgradenum parameter value instead of 'pgradenum' (int4)
--	:pgroupnum,	-- put the pgroupnum parameter value instead of 'pgroupnum' (int4)
--	:pdegree,	-- put the pdegree parameter value instead of 'pdegree' (bachelor_master_specialist)
--	:pspecname,	-- put the pspecname parameter value instead of 'pspecname' (varchar)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:pactivationcode,	-- put the pactivationcode parameter value instead of 'pactivationcode' (varchar)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
    vYear := COALESCE((SELECT Year FROM semesters WHERE semesters.ID = pSemesterID LIMIT 1), -1);
    vGradeID := CreateGrade(pGradeNum, pDegree);
    vGroupID := CreateGroup(vGradeID, pGroupNum, pSpecName, pFacultyID, vYear);
    RETURN CreateStudent(pLastName, pFirstName, pSecondName, vGroupID, pActivationCode, pSemesterID);
END
$function$;

CREATE OR REPLACE FUNCTION public.deleteauthtoken(ptoken character)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vCount int;
begin
--	select public.deleteauthtoken(
--	:ptoken 	-- put the ptoken parameter value instead of 'ptoken' (bpchar)
--);

    DELETE FROM auth_tokens
        WHERE auth_tokens.Token = pToken;
     get diagnostics vCount=ROW_COUNT;
    RETURN vCount-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.deletemodule(pteacherid integer, pmoduleid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vDisciplineID INT DEFAULT -1;
    curs cursor for select  id from modules   WHERE   modules.DisciplineID = vDisciplineID AND
                modules."type" = 'regular'
        ORDER BY modules.OrderNum ASC;
       counter int;
BEGIN
--select public.deletemodule(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pmoduleid 	-- put the pmoduleid parameter value instead of 'pmoduleid' (int4)
--);

    -- get discipline ID
        SELECT disciplines.ID FROM modules
        INNER JOIN disciplines    ON  modules.DisciplineID = disciplines.ID AND disciplines.AuthorID = pTeacherID
        WHERE modules.ID = pModuleID LIMIT 1
into vDisciplineID;

    -- check rights
    if NOT InternalIsTeacherAuthor(pTeacherID, vDisciplineID) OR
        InternalIsMapLocked(vDisciplineID)
    THEN
        RETURN -1;
    END if;

    DELETE FROM submodules
        WHERE submodules.ModuleID = pModuleID;
    DELETE FROM modules
        WHERE modules.ID = pModuleID;

    -- restore continuous ordering
    --нужно сделать через курсор
    set counter = 0;
   for rec in curs loop
     counter = counter + 1;
     update modules set modules.OrderNum  = counter WHERE CURRENT of curs;
   end loop;
    /*update modules
        SET modules.OrderNum = (counter = counter + 1)
        WHERE   modules.DisciplineID = vDisciplineID AND
                modules.Type = 'regular';
    select modules
        ORDER BY modules.OrderNum ASC;
*/
    RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public.deletemodulebonus(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vBonusModuleID INT DEFAULT -1;
begin
--	select public.deletemodulebonus(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);

    CASE WHEN  NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
        InternalIsMapLocked(pDisciplineID)
    THEN
        RETURN -1;
	else null;	
    END CASE;

    CASE WHEN GetBonusModule(pDisciplineID) <= 0 THEN
        RETURN -1;
	else null;	
    END CASE;

    DELETE FROM submodules
        WHERE vBonusModuleID = submodules.ModuleID;

    DELETE FROM modules
        WHERE vBonusModuleID = modules.ID;
        --LIMIT 1;

    RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public.deletemoduleexam(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
  DECLARE vExamModuleID INTEGER DEFAULT -1;
BEGIN
  -- check teacher rights
  IF  NOT InternalIsTeacherAuthor(pTeacherID, pDisciplineID) OR
      InternalIsMapLocked(pDisciplineID)
  THEN
    RETURN -1;
  END IF;

  -- get exam module ID
  SELECT modules.ID INTO vExamModuleID
  FROM modules
    WHERE   modules.Type = 'exam' AND
          modules.DisciplineID = pDisciplineID
  LIMIT 1;

  IF vExamModuleID <= 0 THEN
    RETURN -1;
  END IF;

  -- everything is checked then delete
  DELETE FROM submodules
    WHERE vExamModuleID = submodules.ModuleID;

  DELETE FROM modules
    WHERE vExamModuleID = modules.ID;
  RETURN 0;
END
$function$;

CREATE OR REPLACE FUNCTION public.deletesubject(psubjectid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vSubjectUsage BOOLEAN DEFAULT FALSE;
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1;
    --EXEC SQL WHENEVER SQLWARNING SQLPRINT;
    --EXEC SQL WHENEVER SQLERROR STOP;
    
    vSubjectUsage := EXISTS(
        SELECT * FROM disciplines
        WHERE disciplines.SubjectID = pSubjectID LIMIT 1
    );
    IF vSubjectUsage THEN
        RETURN -1; -- Удаляемый предмет используется в disciplines.
    END IF;

    DELETE FROM subjects_faculties
        WHERE subjects_faculties.SubjectID = pSubjectID;
    DELETE FROM subjects
        WHERE subjects.ID = pSubjectID;
        --LIMIT 1;

    RETURN 0; -- Успешно удалено;
END;
$function$;

CREATE OR REPLACE FUNCTION public.deletesubmodule(pteacherid integer, psubmoduleid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vIsLocked INT DEFAULT -1; vModuleID INT DEFAULT -1;
            counter INT;
            curs cursor for 
              select ID from submodules 
                        where submodules.ModuleID = vModuleID
             			ORDER BY submodules.OrderNum ASC;
BEGIN
--select public.deletesubmodule(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psubmoduleid 	-- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--);
    SELECT modules.ID, disciplines.IsLocked
        INTO vModuleID, vIsLocked
        FROM submodules
        INNER JOIN modules ON modules.ID = submodules.ModuleID
        INNER JOIN disciplines ON modules.DisciplineID = disciplines.ID
        WHERE   disciplines.AuthorID = pTeacherID AND
                submodules.ID = pSubmoduleID
        LIMIT 1;
    IF  vIsLocked != 0 THEN
        RETURN -1;
    END IF;

    -- handler will catch constraint violation
    DELETE FROM submodules
        WHERE submodules.ID = pSubmoduleID;
        --LIMIT 1;

    -- restore continuous ordering
    counter = 0;
     for rec in curs loop
     counter = counter + 1;
     update sumodules set submodules.OrderNum  = counter WHERE CURRENT of curs;
   end loop;
   /* UPDATE submodules
        SET submodules.OrderNum = (counter = counter + 1)
        WHERE   submodules.ModuleID = vModuleID;
        
    select submodules
        ORDER BY submodules.OrderNum ASC;
*/
    RETURN 0;
  --  exception 
   -- when others then return -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.department_create(pname character varying, pfacultyid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
declare vID int;
begin
--	select public.department_create(
--	:pname,	-- put the pname parameter value instead of 'pname' (varchar)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--);

	INSERT INTO departments (Name, FacultyID) VALUES (pName, pFacultyID)
      ON conflict on constraint departments_name_facultyid_key do
       update set departments.facultyid=pFacultyID returning id into vID;
 
    RETURN vID; -- может достаточно return 0; ???
--EXCEPTION 
--	when others then RETURN -1;
end
$function$;

CREATE OR REPLACE FUNCTION public.department_create_with_externalid(pdepartmentexternalid character varying, pdepartmentname text, pfacultyexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
  DECLARE vInnerID INT default null; vFacultyInnerID INT DEFAULT null;
begin
--	select public.department_create_with_externalid(
--	:pdepartmentexternalid,	-- put the pdepartmentexternalid parameter value instead of 'pdepartmentexternalid' (varchar)
--	:pdepartmentname,	-- put the pdepartmentname parameter value instead of 'pdepartmentname' (text)
--	:pfacultyexternalid 	-- put the pfacultyexternalid parameter value instead of 'pfacultyexternalid' (varchar)
--);
  vFacultyInnerID = Faculty_GetIdByExternalID(pFacultyExternalID);
  IF vFacultyInnerID IS NULL THEN
    RETURN -2;
  END IF;

 SELECT ID into  vInnerID
           FROM departments
           WHERE departments.ExternalID LIKE pDepartmentExternalID OR departments.Name LIKE pDepartmentName
           LIMIT 1;

  IF vInnerID IS NULL THEN
    INSERT INTO departments (ExternalID, Name, FacultyID) 
           VALUES (pDepartmentExternalID, pDepartmentName, vFacultyInnerID) returning id into vInnerID;
    RETURN vInnerID;
  else
     SELECT Department_Update(vInnerID, pDepartmentExternalID, pDepartmentName, vFacultyInnerID) into vInnerID;
    RETURN vInnerID;
  END IF;
-- EXCEPTION 
 -- when others then RETURN -1;
END
 $function$;

CREATE OR REPLACE FUNCTION public.department_search(pname character varying, pfacultyid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
--	select public.department_search(
--	:pname,	-- put the pname parameter value instead of 'pname' (varchar)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--);
RETURN COALESCE((
    SELECT departments.ID FROM departments
    WHERE   departments.Name = pName AND departments.FacultyID = pFacultyID LIMIT 1
), -1);
END
$function$;

CREATE OR REPLACE FUNCTION public.department_update(pdepartmentid integer, pdepartmentexternalid text, pdepartmentname text, pfacultyid text)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
begin
--	select public.department_update(
--	:pdepartmentid,	-- put the pdepartmentid parameter value instead of 'pdepartmentid' (int4)
--	:pdepartmentexternalid,	-- put the pdepartmentexternalid parameter value instead of 'pdepartmentexternalid' (text)
--	:pdepartmentname,	-- put the pdepartmentname parameter value instead of 'pdepartmentname' (text)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (text)
--);
	
  IF pDepartmentExternalID IS NOT NULL THEN
    UPDATE departments SET ExternalID = pDepartmentExternalID WHERE ID = pDepartmentID;
  END IF;

  IF pDepartmentName IS NOT NULL THEN
    UPDATE departments SET Name = pDepartmentName WHERE ID = pDepartmentID;
  END IF;

  IF pFacultyID IS NOT NULL THEN
    UPDATE departments SET FacultyID = pFacultyID WHERE ID = pDepartmentID;
  END IF;

  RETURN pDepartmentID;
 -- EXCEPTION  when others then RETURN -1;
end
 $function$;

CREATE OR REPLACE FUNCTION public.departments_get(pfacultyid integer)
 RETURNS TABLE("ID" integer, "Name" character varying, "FacultyID" integer, "ExternalID" character varying)
 LANGUAGE plpgsql
AS $function$ 
begin
--	select public.departments_get(
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--);
  if (pFacultyID is null) or (pFacultyID = 0) THEN
  return query 
  SELECT departments.id as "ID",
    departments."name" as "Name",
    departments.facultyid as "FacultyID",
    departments.externalid as "ExternalID"
    
    FROM departments
    ORDER BY departments.Name ASC;
  else
  return query
    SELECT  departments.id as "ID",
    departments."name" as "Name",
    departments.facultyid as "FacultyID",
    departments.externalid as "ExternalID"
    
    FROM departments
    WHERE departments.FacultyID = pFacultyID
    ORDER BY departments.Name ASC;
  END if;
END;
$function$;

CREATE OR REPLACE FUNCTION public.detachallstudents(pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
begin
--	select public.detachallstudents(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);

    DELETE FROM disciplines_groups
        WHERE disciplines_groups.DisciplineID = pDisciplineID;
    DELETE FROM disciplines_students
        WHERE disciplines_students.DisciplineID = pDisciplineID;
    RETURN 0;
END;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_bindteacher(pdisciplineid integer, pbindingteacherid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
BEGIN
--select public.discipline_bindteacher(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pbindingteacherid 	-- put the pbindingteacherid parameter value instead of 'pbindingteacherid' (int4)
--);

--try to insert BindingTeacher in access list
    INSERT INTO disciplines_teachers (DisciplineID, TeacherID) VALUES (pDisciplineID, pBindingTeacherID)
     on conflict on constraint disciplines_teachers_disciplineid_teacherid_key --just stub
               do nothing; 
    RETURN 0;
  -- EXCEPTION 
   -- when others then RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.discipline_clear(pauthorteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
BEGIN
    if NOT InternalIsTeacherAuthor(pAuthorTeacherID, pDisciplineID) THEN
        RETURN -1;
    END if;
 

    -- clear logs
    DELETE FROM logs_rating
        WHERE logs_rating.SubmoduleID IN
            (SELECT view_roadmap.SubmoduleID
                FROM view_roadmap
                WHERE view_roadmap.DisciplineID = pDisciplineID);

    -- clear rating
    DELETE FROM rating_table
        WHERE rating_table.SubmoduleID IN
            (SELECT view_roadmap.SubmoduleID
             FROM view_roadmap
             WHERE view_roadmap.DisciplineID = pDisciplineID);
			 
	-- clear exam_period_options
	DELETE FROM exam_period_options
        WHERE exam_period_options.SubmoduleID IN
            (SELECT view_roadmap.SubmoduleID
             FROM view_roadmap
             WHERE view_roadmap.DisciplineID = pDisciplineID);

    -- unlock discipline
    UPDATE disciplines
        SET disciplines.IsLocked = 0
        WHERE disciplines.ID = pDisciplineID;
        --LIMIT 1;

    RETURN ROW_COUNT()-1;
END
$function$;

CREATE OR REPLACE FUNCTION public.discipline_create(pteacherid integer, pgradeid integer, psubjectid integer, pexamtype exam_credit_grading_credit, plecturecount integer, ppracticecount integer, plabcount integer, pfacultyid integer, psemesterid integer, psubtype scientific_disciplinary_coursework)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vDisciplineID INT DEFAULT -1;
    		vTemp int;
begin
--	select public.Discipline_Create(
--	:pTeacherID,	-- put the pTeacherID parameter value instead of 'pTeacherID' (INT)
--	:pGradeID,	-- put the pGradeID parameter value instead of 'pGradeID' (INT)
--	:pSubjectID,	-- put the pSubjectID parameter value instead of 'pSubjectID' (INT)
--	:pExamType,	-- put the pExamType parameter value instead of 'pExamType' (ENUM(15))
--	:pLectureCount,	-- put the pLectureCount parameter value instead of 'pLectureCount' (INT)
--	:pPracticeCount,	-- put the pPracticeCount parameter value instead of 'pPracticeCount' (INT)
--	:pLabCount,	-- put the pLabCount parameter value instead of 'pLabCount' (INT)
--	:pFacultyID,	-- put the pFacultyID parameter value instead of 'pFacultyID' (INT)
--	:pSemesterID,	-- put the pSemesterID parameter value instead of 'pSemesterID' (INT)
--	:pSubtype 	-- put the pSubtype parameter value instead of 'pSubtype' (ENUM(24))
--);

-- todo: make more flexible creation of coursework
-- I mean, while creating scientific coursework
--    we don't have the SubjectID field, but we must.
--    346 is used as a default id for scientific courseworks.
--    This constant is duplicated in Model_Helper_CourseWorkBuilder

    IF pSubtype IS NOT NULL THEN
        pSubjectID := 346;
    END IF;
    -- create discipline
    INSERT INTO disciplines
        ( AuthorID, GradeID, SubjectID, ExamType, LectureCount, PracticeCount,
          LabCount, SemesterID, FacultyID, Subtype) 
          VALUES
        (pTeacherID, pGradeID, pSubjectID, pExamType, pLectureCount, pPracticeCount,
         pLabCount, pSemesterID, pFacultyID, pSubtype)
        returning ID into  vDisciplineID;
    
    SELECT Discipline_BindTeacher(vDisciplineID, pTeacherID) INTO vTemp;
    -- add exam and extra modules
    IF pExamType = 'exam' THEN
        SELECT AddModuleExamUnsafe(vDisciplineID) INTO vTemp;
    END IF;
    SELECT AddModuleExtra(pTeacherID, vDisciplineID) INTO vTemp;

    RETURN vDisciplineID;
--EXCEPTION 
 --  when others then RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.discipline_delegate(pdisciplineid integer, pnewauthorid integer, vtemp integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
 declare vtemp int;
BEGIN  
--	select public.discipline_delegate(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pnewauthorid,	-- put the pnewauthorid parameter value instead of 'pnewauthorid' (int4)
--	:vtemp 	-- put the vtemp parameter value instead of 'vtemp' (int4)
--);

    SELECT Discipline_BindTeacher(pDisciplineID, pNewAuthorID) INTO vTemp;
    UPDATE disciplines
        SET disciplines.AuthorID = pNewAuthorID
        WHERE disciplines.ID = pDisciplineID;
    get diagnostics vtemp = ROW_COUNT;
    RETURN vtemp-1;
  -- EXCEPTION 
  -- when others then RETURN -1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_delete(pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
begin
--	select public.discipline_delete(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);

    -- delete roadmap
    DELETE FROM submodules
        WHERE submodules.moduleID in (
            select modules.id FROM modules
            WHERE modules.DisciplineID = pDisciplineID
        );

    DELETE FROM modules
        WHERE modules.DisciplineID = pDisciplineID;

    -- detach all entities from discipline
    DELETE FROM disciplines_teachers
        WHERE disciplines_teachers.DisciplineID = pDisciplineID;
    DELETE FROM disciplines_students
        WHERE disciplines_students.DisciplineID = pDisciplineID;
    DELETE FROM disciplines_groups
        WHERE disciplines_groups.DisciplineID = pDisciplineID;
    DELETE FROM logs_binds_groups
        WHERE logs_binds_groups.DisciplineID = pDisciplineID;
    DELETE FROM logs_binds_students
        WHERE logs_binds_students.DisciplineID = pDisciplineID;

    -- delete discipline
    DELETE FROM disciplines
        WHERE disciplines.ID = pDisciplineID;
        --LIMIT 1;

    RETURN 0;
END;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_getidfromexternalid(pdisciplineexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
begin
--	select public.discipline_getidfromexternalid(
--	:pdisciplineexternalid 	-- put the pdisciplineexternalid parameter value instead of 'pdisciplineexternalid' (varchar)
--);
	return 
	coalesce(
	  (SELECT disciplines.ID    FROM disciplines
        WHERE disciplines.ExternalID = pDisciplineExternalID
        LIMIT 1),
	-1);
	
end
 $function$;

CREATE OR REPLACE FUNCTION public.discipline_getinfo(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "AuthorID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "Type" exam_credit_grading_credit, "Lectures" integer, "Practice" integer, "Labs" integer, "SemesterID" integer, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "FacultyID" integer, "FacultyName" character varying, "IsLocked" integer, "Milestone" integer, "Subtype" scientific_disciplinary_coursework, "CompoundDiscID" integer, "IsMapCreated" boolean, "IsBonus" boolean, "semesterNum" integer, "semesterYear" integer)
 LANGUAGE plpgsql
AS $function$
    DECLARE vIsBonus BOOLEAN;
begin
--	select * from public.discipline_getinfo(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    vIsBonus := EXISTS(
        SELECT * FROM modules
            WHERE   modules.DisciplineID = pDisciplineID AND
                    modules.Type = 'bonus'
            LIMIT 1
    );
	   return query
    SELECT
        view_disciplines.DisciplineID AS "ID",
        view_disciplines.AuthorID as "AuthorID",
        view_disciplines.GradeID as "GradeID",
        view_disciplines.GradeNum as "GradeNum",
        view_disciplines."degree" as "Degree",
        view_disciplines.ExamType       AS "Type",
        view_disciplines.LectureCount   AS "Lectures",
        view_disciplines.PracticeCount  AS "Practice",
        view_disciplines.LabCount       AS "Labs",
        view_disciplines.SemesterID as "SemesterID",
        view_disciplines.SubjectID as "SubjectID",
        view_disciplines.SubjectName as "SubjectName",
        view_disciplines.SubjectAbbr as "SubjectAbbr",
        view_disciplines.FacultyID as "FacultyID",
        view_disciplines.FacultyName as "FacultyName",
        view_disciplines.IsLocked as "IsLocked",
        view_disciplines.Milestone as "Milestone",
        view_disciplines.Subtype as "Subtype",
        view_disciplines.CompoundDiscID as "CompoundDiscID",
        (view_disciplines.MaxRate = 100) AS "IsMapCreated",
        vIsBonus AS "IsBonus",
        semesters.Num AS "semesterNum", -- TODO: Camelize
        semesters.Year AS "semesterYear"
    FROM view_disciplines
    INNER JOIN semesters ON semesters.ID = view_disciplines.SemesterID
    WHERE view_disciplines.DisciplineID = pDisciplineID
    LIMIT 1;
END ;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_getinfobysubmodule(psubmoduleid integer)
 RETURNS TABLE("ID" integer, "AuthorID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "Type" exam_credit_grading_credit, "Lectures" integer, "Practice" integer, "Labs" integer, "SemesterID" integer, "SubjectID" integer, "SubjectName" character varying, "SubjectAbbr" character varying, "FacultyID" integer, "FacultyName" character varying, "IsLocked" integer, "Milestone" integer, "Subtype" scientific_disciplinary_coursework, "CompoundDiscID" integer, "IsMapCreated" boolean, "IsBonus" boolean, "semesterNum" integer, "semesterYear" integer)
 LANGUAGE plpgsql
AS $function$

    DECLARE vDisciplineID INT DEFAULT -1;
begin
--	select * from public.discipline_getinfobysubmodule(
--	:psubmoduleid 	-- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--);
    SELECT  modules.DisciplineID
        INTO vDisciplineID
        FROM submodules
        INNER JOIN modules ON  submodules.ModuleID = modules.ID
        WHERE submodules.ID = pSubmoduleID
        LIMIT 1;

   return query select * from Discipline_GetInfo(vDisciplineID); --CamelCase сохранится?
END;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_getmodules(pdisciplineid integer, ptype exam_rate_all)
 RETURNS TABLE("ID" integer, "Name" character varying, "Type" regular_exam_bonus_extra)
 LANGUAGE plpgsql
AS $function$
begin
--	select public.discipline_getmodules(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:ptype 	-- put the ptype parameter value instead of 'ptype' (exam_rate_all)
--);
    return query
    SELECT  modules.ID as "ID",
            modules."name" as "Name",
            modules."type" as "Type"
        FROM modules
        WHERE   modules.DisciplineID = pDisciplineID AND
            CASE pType
                WHEN 'exam' THEN modules."type" = 'exam' OR modules."type" = 'extra'
                WHEN 'rate' THEN modules."type" != 'exam'
                ELSE TRUE
            END
        ORDER BY
            /*CASE pType
                WHEN 'exam' THEN InternalOrderModuleTypesForSession(modules."type")
                WHEN 'rate' THEN modules."type" # 1 -- XOR 1, 3, 2, 4 ASC
                ELSE TRUE
            END ASC,*/
            modules.OrderNum ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_getstudents(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "RecordBookID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "AttachType" attach_detach)
 LANGUAGE plpgsql
AS $function$ 
    DECLARE vSemesterID int DEFAULT -1;
BEGIN
 --select  * from grade.Discipline_GetStudents(
--	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
--);
    vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');
  return query
    SELECT  view_students.StudentID AS "ID",
            view_students.RecordBookID as "RecordBookID",
            view_students.LastName as "LastName",
            view_students.FirstName as "FirstName",
            view_students.SecondName as "SecondName",
            view_students.GradeID as "GradeID",
            view_students.GradeNum as "GradeNum",
            view_students."degree" as "Degree",
            view_students.GroupID as "GroupID",
            view_students.GroupNum as "GroupNum",
            view_disciplines_recordbooks.Type AS "AttachType"
        FROM view_disciplines_recordbooks
        INNER JOIN view_students ON  view_disciplines_recordbooks.RecordBookID = view_students.RecordBookID
        WHERE view_students.SemesterID = vSemesterID AND view_disciplines_recordbooks.DisciplineID = pDisciplineID
        ORDER BY    view_disciplines_recordbooks.Type = 'attach' ASC,
                    view_students.GradeID ASC,
                    view_students.GroupNum ASC,
                    view_students.LastName ASC,
                    view_students.FirstName ASC;
END
 $function$;

CREATE OR REPLACE FUNCTION public.discipline_getstudentstemp(pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
begin
--select grade.Discipline_GetStudentsTemp(
---	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
--);	
CREATE TEMPORARY TABLE tDisciplineStudents AS (
        SELECT  record_books.StudentID,
            COALESCE(view_disciplines_recordbooks."type") AS "Type" ---?????
        FROM view_disciplines_recordbooks
            INNER JOIN record_books ON view_disciplines_recordbooks.RecordBookID = record_books.ID
        WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID
        GROUP BY record_books.StudentID,COALESCE(view_disciplines_recordbooks."type")
    );
    RETURN 0;
 end;
 $function$;

CREATE OR REPLACE FUNCTION public.discipline_getsubmodules(pdisciplineid integer, ptype exam_rate_all)
 RETURNS TABLE("ModuleID" integer, "ID" integer, "Name" character varying, "Rate" integer, "Type" current_landmark_control)
 LANGUAGE plpgsql
AS $function$

begin
--	select * from public.discipline_getsubmodules(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:ptype 	-- put the ptype parameter value instead of 'ptype' (exam_rate_all)
--);
return query
    SELECT  view_roadmap.ModuleID as "ModuleID",
            view_roadmap.SubmoduleID    AS "ID",
            view_roadmap.SubmoduleName  AS "Name",
            view_roadmap.SubmoduleRate  AS "Rate",
            view_roadmap.SubmoduleType  AS "Type"
        FROM view_roadmap
        WHERE   view_roadmap.DisciplineID = pDisciplineID AND
            CASE pType
                WHEN 'exam' THEN view_roadmap.ModuleType = 'exam' OR view_roadmap.ModuleType = 'extra'
                WHEN 'rate' THEN view_roadmap.ModuleType != 'exam' OR view_roadmap.SubmoduleOrderNum = 1
                ELSE TRUE
            END
        ORDER BY
            view_roadmap.ModuleOrderNum ASC,
            view_roadmap.SubmoduleOrderNum ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_setexamperiodoption(precordbookid integer, psubmoduleid integer, ptype absence_pass DEFAULT NULL::absence_pass)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
 DECLARE vType absence_pass DEFAULT NULL;
BEGIN
--select public.discipline_setexamperiodoption(
--    :precordbookid,    -- put the precordbookid parameter value instead of 'precordbookid' (int4)
--    :psubmoduleid,    -- put the psubmoduleid parameter value instead of 'psubmoduleid' (int4)
--    :ptype     -- put the ptype parameter value instead of 'ptype' (absence_pass)
--);
    IF pType is not null THEN
         vType = pType;
    END IF;
 
    INSERT INTO exam_period_options (RecordBookID, SubmoduleID, Type) VALUES(pRecordBookID, pSubmoduleID, vType)
        ON  conflict ON CONSTRAINT  exam_period_options_recordbookid_submoduleid_key
          do update set  "type" = vType;
 
    RETURN 0;
 --EXCEPTION 
  --        when others 
  --            then RETURN -1;
end
$function$;

CREATE OR REPLACE FUNCTION public.discipline_setgradeunsafe(pdisciplineid integer, pgradeid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
declare vRow int;
begin
--	select public.Discipline_SetGradeUnsafe(
--	:pDisciplineID,	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
--	:pGradeID 	-- put the pGradeID parameter value instead of 'pGradeID' (INT)
--);

    -- set grade
    UPDATE disciplines
        SET disciplines.GradeID = pGradeID
        WHERE disciplines.ID = pDisciplineID ;
     get diagnostics vRow = ROW_COUNT;
       RETURN vRow-1;
 --  EXCEPTION 
 --  when others then RETURN -1;
END
 $function$;

CREATE OR REPLACE FUNCTION public.discipline_setrate(pteacherid integer, precordbookid integer, psubmoduleid integer, prate integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
    DECLARE vDisciplineID INT DEFAULT -1;
               vMaxRate INT DEFAULT -1;
               vModuleType regular_exam_bonus_extra;
    DECLARE vIsOver int DEFAULT 0; 
               vIsLocked int DEFAULT 0; 
               vIsUsed int2 DEFAULT 0;
               vIsUpdated boolean default false;
begin
--    select Discipline_SetRate(
--    :pTeacherID,    -- put the pTeacherID parameter value instead of 'pTeacherID' (INT)
--    :pRecordBookID,    -- put the pRecordBookID parameter value instead of 'pRecordBookID' (INT)
--    :pSubmoduleID,    -- put the pSubmoduleID parameter value instead of 'pSubmoduleID' (INT)
--    :pRate     -- put the pRate parameter value instead of 'pRate' (INT)
--);
 
      -- 0) check rights здесь еще нельзя проверять, т.к. нет vDisciplineID
      -- нужно переставить очередность
    /*  IF NOT InternalIsStudentAttached(pRecordBookID, vDisciplineID) OR
       NOT InternalIsTeacherBound(pTeacherID, vDisciplineID) THEN
        RETURN 1;
      END IF;*/

        IF pRate < 0 THEN
        INSERT INTO logs_rating (RecordBookID, SubmoduleID, TeacherID, Rate, "action" ) VALUES
            (pRecordBookID, pSubmoduleID, pTeacherID, pRate, 'delete');
    
        -- TODO: extract method log rate
        DELETE FROM rating_table
            WHERE   rating_table.RecordBookID = pRecordBookID AND
                    rating_table.SubmoduleID = pSubmoduleID
           ;
        RETURN 0;
        END IF;

    vIsOver = 0;
   SELECT  disciplines.ID,
            disciplines.IsLocked,
            disciplines.Milestone,
            submodules.IsUsed,
            submodules.maxRate,
            modules."type"
        INTO vDisciplineID, vIsLocked, vIsOver, vIsUsed, vMaxRate, vModuleType
        FROM submodules
        INNER JOIN modules                ON  submodules.ModuleID = modules.ID
        INNER JOIN disciplines            ON  modules.DisciplineID = disciplines.ID
        WHERE   submodules.ID = pSubmoduleID
        LIMIT 1;
 
    -- correct max rate for extra module
    IF vModuleType = 'extra' THEN -- 4 - extra
         vMaxRate = CalculateMaxRateForExtra(vDisciplineID, pRecordBookID);
    END IF;

      -- 1) check rights ПОЧЕМУ ТАК ПОЗДНО ???? см п.0
    IF NOT InternalIsStudentAttached(pRecordBookID, vDisciplineID) OR
       NOT InternalIsTeacherBound(pTeacherID, vDisciplineID) THEN
        RETURN 1;
    END IF;

    --2) check, you can't rate regular and bonus after milestone
    IF (vIsOver=1) AND (vModuleType = 'regular' OR vModuleType = 'bonus') THEN --1 - regular, 3 - bonus
        RETURN 2;
    END IF;

    --3) check, max rate exceeding
    IF  pRate > vMaxRate THEN
        RETURN 3;
    END IF;

     -- add rate, or update old
  visUpdated = exists (select * from rating_table where RecordBookID=pRecordBookID and  SubmoduleID= pSubmoduleID);

 INSERT INTO rating_table (RecordBookID, TeacherID, SubmoduleID, Rate, "date")
        VALUES  (pRecordBookID, pTeacherID, pSubmoduleID, pRate, current_date )
        on conflict on constraint rating_table_pkey do
         update set
            TeacherID  = pTeacherID, --(@isUpdated := pTeacherID),
            Rate       = pRate,
            "date"       = current_date;
        -- returning true into visUpdated ; ---??????? если не получится, разделить insert и update
          --log rate

    INSERT INTO logs_rating (RecordBookID, SubmoduleID, TeacherID, Rate, "action" )
        VALUES (pRecordBookID, pSubmoduleID, pTeacherID, pRate, cast (iif_sql(visUpdated, 'change', 'add') as add_change_delete) ); -- может нужно cast(   as add_change_delete) ???
   -- lock discipline for structure editing
  
    IF  vIsLocked = 0 THEN
        UPDATE disciplines
            SET IsLocked = 1
            WHERE disciplines.ID = vDisciplineID
            ;
    END IF;

--add submodule to max rate counting (see triggers) 
    IF vIsUsed=0 THEN
        UPDATE submodules
            SET submodules.IsUsed = 1
            WHERE submodules.ID = pSubmoduleID
           ;
    END IF;
 
    RETURN 0;
--EXCEPTION 
--when others then RETURN -1;
end
 $function$
 ;

CREATE OR REPLACE FUNCTION public.discipline_unbindteacher(pdisciplineid integer, pbindingteacher integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vRow int;
begin
--	select public.discipline_unbindteacher(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pbindingteacher 	-- put the pbindingteacher parameter value instead of 'pbindingteacher' (int4)
--);

    DELETE FROM disciplines_teachers
        WHERE   disciplines_teachers.DisciplineID = pDisciplineID AND disciplines_teachers.TeacherID = pBindingTeacher;
        --LIMIT 1;
        get diagnostics vRow=ROW_COUNT;
    RETURN vRow-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.faculty_create(pfacultyexternalid text, pfacultyname text, pfacultyabbr text)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
  DECLARE vInnerID integer DEFAULT -1;
begin
--	select public.faculty_create(
--	:pfacultyexternalid,	-- put the pfacultyexternalid parameter value instead of 'pfacultyexternalid' (text)
--	:pfacultyname,	-- put the pfacultyname parameter value instead of 'pfacultyname' (text)
--	:pfacultyabbr 	-- put the pfacultyabbr parameter value instead of 'pfacultyabbr' (text)
--);
  SELECT faculties.id  into vInnerID
            FROM faculties
            WHERE faculties.externalid LIKE pFacultyExternalID OR faculties."name" LIKE pFacultyName
            LIMIT 1  ;

  IF vInnerID IS NULL THEN
    INSERT INTO faculties (ExternalID, "name", Abbr) VALUES (pFacultyExternalID, pFacultyName, pFacultyAbbr) 
       returning id into vInnerID;
    RETURN vInnnerID;
  ELSE
    RETURN (SELECT Faculty_Update(vInnerID, pFacultyExternalID, pFacultyName, pFacultyAbbr));
  END IF;
 --EXCEPTION 
 --when others then
 --RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.faculty_getidbyexternalid(pfacultyexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
        DECLARE pID INT DEFAULT -1;
begin
--	select public.faculty_getidfromexternalid(
--	:pFacultyExternalID	-- put the pFacultyExternalID parameter value instead of 'pFacultyExternalID' (varchar)
--);
        SELECT faculties.ID INTO pID
        FROM faculties
        WHERE faculties.ExternalID = pFacultyExternalID
        LIMIT 1; -- скорее всего не требуется
        RETURN pID;
    END;
$function$;

CREATE OR REPLACE FUNCTION public.faculty_update(pfacultyid integer, pfacultyexternalid text, pfacultyname text, pfacultyabbr text)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
begin
--	select public.faculty_update(
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:pfacultyexternalid,	-- put the pfacultyexternalid parameter value instead of 'pfacultyexternalid' (text)
--	:pfacultyname,	-- put the pfacultyname parameter value instead of 'pfacultyname' (text)
--	:pfacultyabbr 	-- put the pfacultyabbr parameter value instead of 'pfacultyabbr' (text)
--);
  IF pFacultyExternalID IS NOT NULL THEN
    UPDATE faculties SET ExternalID = pFacultyExternalID WHERE ID = pFacultyID;
  END IF;

  IF pFacultyName IS NOT NULL THEN
    UPDATE faculties SET Name = pFacultyName WHERE ID = pFacultyID;
  END IF;

  IF pFacultyAbbr IS NOT NULL THEN
    UPDATE faculties SET Abbr = pFacultyAbbr WHERE ID = pFacultyID;
  END IF;

  RETURN pFacultyID;
 -- EXCEPTION 
 -- when others then 
 --   RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.findgroup(pgradeid integer, pgroupnum integer, pfacultyid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
begin
--	select public.findgroup(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pgroupnum,	-- put the pgroupnum parameter value instead of 'pgroupnum' (int4)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--);

RETURN COALESCE((
    SELECT study_groups.ID FROM study_groups
    WHERE   study_groups.GradeID = pGradeID
            AND study_groups.GroupNum = pGroupNum
            AND study_groups.FacultyID = pFacultyID
    LIMIT 1
), -1);
END
$function$;

CREATE OR REPLACE FUNCTION public.getattestationdata(pdisciplineid integer, pgroupid integer)
 RETURNS TABLE("StudentID" integer, "Rate" integer, "Date" date, "OrderNum" integer, "Type" regular_exam_bonus_extra)
 LANGUAGE plpgsql
AS $function$

    DECLARE vSemesterID INT DEFAULT -1;
BEGIN
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

   return query
    SELECT    students.ID   AS "StudentID",
              rating_table.Rate As "Rate",
              rating_table."date"  As "Date",
              submodules.OrderNum As "OrderNum",
              modules."type" as "Type"
        FROM students
        INNER JOIN accounts ON students.AccountID = accounts.ID
        INNER JOIN students_groups ON students_groups.StudentID = students.ID AND
                                        students_groups.SemesterID = vSemesterID AND
                                        students_groups.State <= 'outlet'

        LEFT JOIN disciplines_groups ON   disciplines_groups.DisciplineID = pDisciplineID AND
                                            disciplines_groups.GroupID = students_groups.GroupID
        LEFT JOIN disciplines_students ON disciplines_students.DisciplineID = pDisciplineID AND
                                            disciplines_students.StudentID = students.ID

        LEFT JOIN modules ON  modules.DisciplineID = pDisciplineID AND
                                (modules.Type = 'exam' OR modules.Type = 'extra')
        LEFT JOIN submodules ON   submodules.ModuleID = modules.ID
        LEFT JOIN rating_table ON     rating_table.SubmoduleID = submodules.ID AND
                                        rating_table.StudentID = students.ID
        WHERE   students_groups.GroupID = pGroupID AND rating_table.Rate IS NOT NULL AND
                ((disciplines_students.StudentID IS NOT NULL AND disciplines_students.Type = 'attach')
                 OR (disciplines_groups.DisciplineID IS NOT NULL AND disciplines_students.Type IS NULL)
                )
        ORDER BY    CONCAT(accounts.LastName, ' ', accounts.FirstName, ' ', coalesce(accounts.SecondName,'')) ASC,
                    students.ID ASC,
                    modules.Type = 'exam' ASC,
                    submodules.OrderNum ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getdisciplineproperty(pdisciplineid integer, ptype grade_subject_author_semester_milestone_type_maxrate)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vRes INT DEFAULT -1;
begin
--	select public.getdisciplineproperty(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:ptype 	-- put the ptype parameter value instead of 'ptype' (grade_subject_author_semester_milestone_type_maxrate)
--);
    SELECT CASE pType
            WHEN 'semester' THEN disciplines.SemesterID
            WHEN 'author' THEN disciplines.AuthorID
            WHEN 'grade' THEN disciplines.GradeID
            WHEN 'subject' THEN disciplines.SubjectID
            WHEN 'milestone' THEN disciplines.Milestone
         --   WHEN 'type' THEN cast (disciplines.ExamType as integer) +0
            WHEN 'maxrate' THEN disciplines.MaxRate
            else -1
        END INTO vRes
        FROM disciplines
        WHERE disciplines.ID = pDisciplineID
        LIMIT 1;
     RETURN vRes;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getdisciplinesforgroup(pgroupid integer, psemesterid integer)
 RETURNS TABLE("ID" integer, "SubjectName" character varying, "Subtype" character varying, "Type" exam_credit_grading_credit, "CompoundDiscID" integer, "CompoundDiscName" character varying, "CurRate" integer, "MaxRate" integer)
 LANGUAGE plpgsql
AS $function$
BEGIN
 return query
     SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
        view_disciplines.SubjectName as "SubjectName",
        view_disciplines.Subtype as "Subtype",
        view_disciplines.ExamType  AS "Type",
        view_disciplines.CompoundDiscID as "CompoundDiscID",
        view_disciplines.CompoundDiscName as "CompoundDiscName",
        view_disciplines.CurRate as "CurRate",
        view_disciplines.MaxRate as "MaxRate"
    FROM students_groups
        LEFT JOIN view_disciplines_recordbooks ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID
        INNER JOIN view_disciplines ON view_disciplines_recordbooks.DisciplineID = view_disciplines.DisciplineID
        WHERE   students_groups.GroupID = pGroupID AND
            students_groups.SemesterID = pSemesterID AND
            view_disciplines.SemesterID = pSemesterID AND
            students_groups.State <= 'outlet' AND
           -- view_disciplines_recordbooks.Type may be attach or NULL
            COALESCE(view_disciplines_recordbooks.Type, 'attach') = 'attach'
        ORDER BY view_disciplines.ExamType DESC,
                 view_disciplines.SubjectName ASC;
end;
$function$;

CREATE OR REPLACE FUNCTION public.geteditrightsforteacher(pteacherid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vUserRole INT DEFAULT -1;
BEGIN
--select public.geteditrightsforteacher(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);

    IF InternalIsTeacherBound(pTeacherID, pDisciplineID) > 0 THEN
        RETURN 1;
    END IF;

    SELECT accounts.UserRoleID INTO vUserRole
        FROM teachers
        INNER JOIN accounts ON teachers.AccountID = accounts.ID
        WHERE teachers.ID = pTeacherID
        LIMIT 1;
    RETURN iif_sql(vUserRole = GetUserRole('dean'), 0, -1);
END;
$function$;

CREATE OR REPLACE FUNCTION public.getexamrateid(precordbook integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

begin
--	select public.getexamrateid(
--	:pstudentid,	-- put the pstudentid parameter value instead of 'pstudentid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);

RETURN COALESCE((
    SELECT view_roadmap.SubmoduleID FROM view_roadmap
    LEFT JOIN rating_table AS rt on rt.SubmoduleID = view_roadmap.SubmoduleID AND rt.RecordBookID = pRecordBook
        WHERE view_roadmap.DisciplineID = pDisciplineID AND view_roadmap.ModuleType = 'exam'
    ORDER BY (rt.Rate IS NOT NULL) DESC, view_roadmap.SubmoduleOrderNum DESC
    LIMIT 1
), -1);
END
$function$;

CREATE OR REPLACE FUNCTION public.getfullinfo(puserid integer, psemesterid integer)
 RETURNS TABLE("StudentID" integer, "GroupID" integer, "GroupNum" integer, "GroupName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "SpecCode" character varying, "SemesterID" integer, "TeacherID" integer, "DepID" integer, "DepName" character varying, "JobPositionName" character varying, "FacultyID" integer, "FacultyName" character varying, "FacultyAbbr" character varying, "ID" integer, "Login" character varying, "EMail" character varying, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "Type" student_teacher, "RoleID" integer, "Role" character varying, "RoleMark" integer, "IsEnabled" smallint, "Code" character varying)
 LANGUAGE plpgsql
AS $function$
DECLARE vAccountType student_teacher;
begin
 --select *from public.getfullinfo(
--	:puserid,	-- put the puserid parameter value instead of 'puserid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);   
    SELECT user_roles."type" INTO vAccountType
        FROM accounts
        INNER JOIN user_roles ON accounts.UserRoleID = user_roles.ID
        WHERE accounts.ID = pUserID
        LIMIT 1;

-- type 1: student
--      2: teacher
    IF vAccountType = 'student' then
     return query
        SELECT
                -- personal students info
                students.ID 						as "StudentID",
                               -- group info
				view_groups_reduced.groupid			as "GroupID",
				view_groups_reduced.groupnum		as "GroupNum",
				view_groups_reduced.groupname		as "GroupName",
				view_groups_reduced.gradeid			as "GradeID",
				view_groups_reduced.gradenum		as "GradeNum",
				view_groups_reduced."degree"		as "Degree",
				view_groups_reduced.specid			as "SpecID",
				view_groups_reduced.specname		as "SpecName",
				view_groups_reduced.specabbr		as "SpecAbbr",
				view_groups_reduced.speccode		as "SpecCode",
 				students_groups.SemesterID			as "SemesterID",
               	-- teacher info
                cast(null  as integer)				as "TeacherID",
                cast(null  as integer)				as "DepID",
                cast(null  as varchar)				as "DepName",
                cast(null  as varchar)				as "JobPositionName",

                --faculty info
                view_groups_reduced.facultyid		as "FacultyID",
                view_groups_reduced.facultyname		as "FacultyName",
                view_groups_reduced.facultyabbr		as "FacultyAbbr",
 				-- account info
                accounts.ID							as "ID",
                accounts.Login						as "Login",
                accounts.EMail						as "EMail",
                accounts.LastName					as "LastName",
                accounts.FirstName					as "FirstName",
                accounts.SecondName					as "SecondName",
                user_roles."type"					as "Type",
                accounts.UserRoleID 				AS "RoleID",
                user_roles.RoleName 				AS "Role",
                user_roles.Mark     				AS "RoleMark",
                accounts.IsEnabled					as "IsEnabled",
                accounts.ActivationCode 			AS "Code"
            FROM students
            INNER JOIN accounts ON accounts.ID = students.AccountID
            INNER JOIN user_roles ON user_roles.ID = accounts.UserRoleID
            LEFT JOIN record_books ON record_books.StudentID = students.ID
            LEFT JOIN students_groups ON  students_groups.RecordBookID = record_books.ID
            LEFT JOIN view_groups_reduced ON view_groups_reduced.GroupID = students_groups.GroupID
            WHERE   students.AccountID = pUserID and (pSemesterID is null or students_groups.SemesterID = pSemesterID )
             ORDER BY students_groups.SemesterID  DESC
            LIMIT 1;
    ELSE
        return query
    		SELECT  
                        -- personal students info не олучается поставить null????
                cast(null  as integer)			as "StudentID",
                               -- group info
				cast(null  as integer)			as "GroupID",
				cast(null  as integer)			as "GroupNum",
				cast(null  as varchar)			as "GroupName",
				cast(null  as integer)			as "GradeID",
				cast(null  as integer)			as "GradeNum",
				cast(null as bachelor_master_specialist)								as "Degree",
				cast(null  as integer)			as "SpecID",
				cast(null as varchar)			as "SpecName",
				cast(null as varchar)			as "SpecAbbr",
				cast(null as varchar)			as "SpecCode",
 				cast(null  as integer)			as "SemesterID",
 				        -- teacher info
                view_teachers.TeacherID				as "TeacherID",
                view_teachers.DepID					as "DepID",
                view_teachers.DepName				as "DepName",
                view_teachers.JobPositionName		as "JobPositionName",
 				                --faculty info
                view_teachers.FacultyID				as "FacultyID",
                view_teachers.FacultyName			as "FacultyName",
                view_teachers.FacultyAbbr			as "FacultyAbbr",
                
                -- account info
                accounts.ID							as "ID",
                accounts.Login						as "Login",
                accounts.EMail						as "EMail",
                accounts.LastName					as "LastName",
                accounts.FirstName					as "FirstName",
                accounts.SecondName					as "SecondName",
                user_roles."type"					as "Type",
                accounts.UserRoleID 				AS "RoleID",
                user_roles.RoleName 				AS "Role",
                user_roles.Mark     				AS "RoleMark",
                accounts.IsEnabled					as "IsEnabled",
               accounts.ActivationCode 			AS "Code"
            FROM view_teachers
            INNER JOIN accounts ON accounts.ID = view_teachers.AccountID
            INNER JOIN user_roles ON user_roles.ID = accounts.UserRoleID
            WHERE view_teachers.AccountID = pUserID
            LIMIT 1;
    END IF;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getgroupsfordisciplineall(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "GroupNum" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying)
 LANGUAGE plpgsql
AS $function$

    DECLARE vSemesterID INT DEFAULT -1; vYear INT DEFAULT -1;
BEGIN
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');
     SELECT "year"  into vYear from semesters where semesters.ID = vSemesterID;

    -- general + attached
    CREATE TEMPORARY TABLE IF NOT EXISTS tGroup AS (
        SELECT students_groups.GroupID
        FROM view_disciplines_recordbooks
        INNER JOIN students_groups ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID AND
                                      students_groups.SemesterID = vSemesterID AND
                                      students_groups.State <= 'outlet'
        WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID AND
              'attach' = COALESCE(view_disciplines_recordbooks.Type, 'attach')
        GROUP BY students_groups.GroupID
    );

    return query
    SELECT  view_groups.GroupID AS "ID",
            view_groups.GroupNum as "GroupNum",
            view_groups.GradeID	as "GradeID",
            view_groups.GradeNum as "GradeNum",
            view_groups."degree" as "Degree",
            view_groups.SpecID as "SpecID",
            view_groups.SpecName as "SpecName",
            view_groups.SpecAbbr as "SpecAbbr"
        FROM tGroup
        INNER JOIN view_groups ON tGroup.GroupID = view_groups.GroupID AND view_groups.Year = vYear
        ORDER BY view_groups.GradeID ASC, view_groups.GroupID ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getmilestone(pfacultyid integer, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vMilestone INT DEFAULT 0; vCounter INT DEFAULT 0;
BEGIN
    -- get most frequent milestone
    SELECT COUNT(*) AS cnt, disciplines.Milestone INTO vCounter, vMilestone
        FROM disciplines
        WHERE disciplines.SemesterID = pSemesterID AND disciplines.FacultyID = pFacultyID
        GROUP BY disciplines.Milestone
        ORDER BY cnt DESC
        LIMIT 1;

    RETURN vMilestone;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getrates(pstudentid integer, pdisciplineid integer)
 RETURNS TABLE("ModuleID" integer, "ModuleName" character varying, "SubmoduleID" integer, "SubmoduleName" character varying, "MaxRate" integer, "SubmoduleControl" current_landmark_control, "ModuleType" regular_exam_bonus_extra, "Rate" integer, "Date" date, "ExamPeriodOption" absence_pass)
 LANGUAGE plpgsql
AS $function$

    DECLARE vExamSubmoduleID INT; 
begin
	vExamSubmoduleID= GetExamRateID(pStudentID, pDisciplineID);
 return query
    SELECT  vr.ModuleID as "ModuleID",
            vr.ModuleName as "ModuleName",
            vr.SubmoduleID as "SubmoduleID",
            vr.SubmoduleName as "SubmoduleName",
            vr.SubmoduleRate AS "MaxRate",
            vr.SubmoduleType AS "SubmoduleControl",
            vr.ModuleType as "ModuleType",
            rt.Rate as "Rate",
            rt."date" as "Date" ,
            epo."type" AS "ExamPeriodOption"
        FROM view_roadmap AS vr
        LEFT JOIN rating_table AS rt ON  vr.SubmoduleID = rt.SubmoduleID AND rt.RecordBookID = pRecordBookID
        LEFT JOIN exam_period_options AS epo ON  epo.SubmoduleID = rt.SubmoduleID AND epo.RecordBookID = pRecordBookID
        WHERE   vr.DisciplineID = pDisciplineID AND ( vr.ModuleType != 'exam' OR vr.SubmoduleID = vExamSubmoduleID )
        ORDER BY vr.ModuleType, --^ 1 ASC, -- 1, 3, 2, 4 ASC
                vr.ModuleOrderNum ASC,
                vr.SubmoduleOrderNum ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesall(precordbookid integer, pdisciplineid integer)
 RETURNS TABLE("ModuleID" integer, "ModuleName" character varying, "SubmoduleID" integer, "SubmoduleName" character varying, "MaxRate" integer, "SubmoduleType" current_landmark_control, "SubmoduleOrderNum" integer, "Rate" integer, "Date" date, "ModuleType" regular_exam_bonus_extra, "ExamPeriodOption" absence_pass)
 LANGUAGE plpgsql
AS $function$
--select * from public.getratesall(
--	:pstudentid,	-- put the pstudentid parameter value instead of 'pstudentid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
begin
return query
    SELECT  view_roadmap.ModuleID as "ModuleID",
            view_roadmap.ModuleName as "ModuleName",
            view_roadmap.SubmoduleID as "SubmoduleID",
            view_roadmap.SubmoduleName as "SubmoduleName",
            view_roadmap.SubmoduleRate AS "MaxRate",
            view_roadmap.SubmoduleType as "SubmoduleType",
            view_roadmap.SubmoduleOrderNum as "SubmoduleOrderNum",
            rating_table.Rate as "Rate",
            rating_table."date" as "Date",
            view_roadmap.ModuleType as "ModuleType",
            exam_period_options.Type As "ExamPeriodOption"
    FROM view_roadmap
    LEFT JOIN rating_table ON rating_table.SubmoduleID = view_roadmap.SubmoduleID
      AND rating_table.RecordBookID = pRecordBookID
    LEFT JOIN exam_period_options ON exam_period_options.SubmoduleID = view_roadmap.SubmoduleID
      AND exam_period_options.RecordBookID = pRecordBookID
    WHERE view_roadmap.DisciplineID = pDisciplineID
    ORDER BY    view_roadmap.ModuleOrderNum ASC,
                view_roadmap.SubmoduleOrderNum ASC;
 end;
$function$;

CREATE OR REPLACE FUNCTION public.getratesfordiscipline(pdisciplineid integer)
 RETURNS TABLE("SubmoduleID" integer, "RecordBookID" integer, "Rate" integer)
 LANGUAGE plpgsql
AS $function$
    DECLARE vSemesterID INT DEFAULT -1;
begin
--	select * from  public.getratesfordiscipline(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

     return QUERY SELECT  view_roadmap.SubmoduleID as "SubmoduleID",
            rating_table.recordbookid as "RecordBookID",
            rating_table.Rate as "Rate"
        FROM view_roadmap
            LEFT JOIN rating_table ON rating_table.SubmoduleID = view_roadmap.SubmoduleID
            INNER JOIN students_groups ON students_groups.SemesterID = vSemesterID AND
                                            students_groups.recordbookid = rating_table.recordbookid
            WHERE view_roadmap.DisciplineID = pDisciplineID;
            --ORDER BY rating_table.recordbookid;
     return;
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesforgroup(pdisciplineid integer, pgroupid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, intermediate integer, bonus integer, exam integer)
 LANGUAGE plpgsql
AS $function$

    DECLARE vInGeneralGroup BOOLEAN DEFAULT FALSE;
    vSemesterID INT DEFAULT -1;
BEGIN
    vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');

    vInGeneralGroup = EXISTS(
        SELECT * FROM disciplines_groups
        WHERE disciplines_groups.DisciplineID = pDisciplineID AND
              disciplines_groups.GroupID = pGroupID
        LIMIT 1
    );


    DROP TABLE IF EXISTS tStudents;
        CREATE TEMPORARY TABLE tStudents AS (
        SELECT students_groups.RecordBookID
        FROM students_groups
        LEFT JOIN disciplines_students ON disciplines_students.DisciplineID = pDisciplineID AND
                                          disciplines_students.RecordBookID = students_groups.RecordBookID
        WHERE   students_groups.SemesterID = vSemesterID AND
                students_groups.State <= 'outlet' AND -- actual students
                students_groups.GroupID = pGroupID AND
                /*CASE WHEN vInGeneralGroup THEN
                    NOT disciplines_students.Type <=> 'detach' -- not detached
                ELSE
                    COALESCE(disciplines_students.Type, 'attach') = 'attach'-- is attached
                END*/
                (disciplines_students."type" is null or  disciplines_students."type" ='attach')
    );

    DROP TABLE IF EXISTS vRoadMap;
    CREATE TEMPORARY TABLE vRoadMap AS (
        SELECT  view_roadmap.SubmoduleID,
                view_roadmap.ModuleType AS Type
            FROM view_roadmap
            WHERE view_roadmap.DisciplineID = pDisciplineID
    );
    return query
    SELECT  students.ID			as "ID",
            accounts.LastName	as "LastName",
            accounts.FirstName	as "FirstName",
            accounts.SecondName	as "SecondName",
            vRates.RateRegular 	AS intermediate,
            vRates.RateBonus 	AS bonus,
            vRates.RateExam 	AS exam
        FROM (
            SELECT tStudents.StudentID,
                   SUM(CASE WHEN vRoadMap.Type = 'regular' THEN  rt.Rate ELSE  0 END) AS RateRegular,
                   SUM(CASE WHEN vRoadMap.Type = 'extra' THEN  rt.Rate ELSE  0 END) AS RateExtra,
                   SUM(CASE WHEN vRoadMap.Type = 'bonus' THEN  rt.Rate ELSE  0 END) AS RateBonus,
                   MAX(CASE WHEN vRoadMap.Type = 'exam' THEN  rt.Rate ELSE  0 END) AS RateExam
                FROM tStudents
                CROSS JOIN vRoadMap
                LEFT JOIN rating_table as rt  ON  rt.StudentID = tStudents.StudentID AND
                                                    rt.SubmoduleID = vRoadMap.SubmoduleID
                GROUP BY tStudents.StudentID
        ) vRates
        INNER JOIN students ON students.ID = vRates.StudentID
        LEFT JOIN accounts ON students.AccountID = accounts.ID
        ORDER BY CONCAT(accounts.LastName, accounts.FirstName,  coalesce (accounts.SecondName,'')) ASC, 
                 students.ID ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesforgroupall(pgroupid integer, psemesterid integer)
 RETURNS TABLE("StudentID" integer, "DisciplineID" integer, "CompoundDiscID" integer, intermediate integer, bonus integer, extra integer, exam integer, examcnt integer)
 LANGUAGE plpgsql
AS $function$
BEGIN
    DROP TABLE IF EXISTS tDisc;
    CREATE TEMPORARY TABLE tDisc AS (
        SELECT view_disciplines_recordbooks.DisciplineID
        FROM view_disciplines_recordbooks
            INNER JOIN students_groups ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID AND
                                          students_groups.SemesterID = pSemesterID AND
                                          students_groups.State <= 'outlet'
        WHERE students_groups.GroupID = pGroupID AND
            'attach' = COALESCE(view_disciplines_recordbooks."type" , 'attach')
        GROUP BY view_disciplines_recordbooks.DisciplineID
    );

    DROP TABLE IF EXISTS tStudents;
    CREATE TEMPORARY TABLE tStudents AS (
        SELECT students_groups.RecordBookID,
                record_books.StudentID
            FROM students_groups
            INNER JOIN record_books ON record_books.ID = students_groups.RecordBookID
            WHERE   students_groups.GroupID = pGroupID
                    and students_groups.SemesterID = pSemesterID
    );

    DROP TABLE IF EXISTS vRoadMap;
    CREATE TEMPORARY TABLE vRoadMap AS (
        SELECT  tDisc.DisciplineID as DisciplineID,
                view_roadmap.ModuleType as ModuleType,
                view_roadmap.SubmoduleID
        FROM tDisc
        LEFT JOIN view_roadmap ON view_roadmap.DisciplineID = tDisc.DisciplineID
    );
   return query
    SELECT  vRates.StudentID as "StudentID",
            vRates.DisciplineID as "DisciplineID",
            disciplines.CompoundDiscID as "CompoundDiscID",
            vRates.RateRegular AS intermediate,
            vRates.RateBonus AS bonus,
            vRates.RateExtra AS extra,
            vRates.RateExam AS exam,
            vRates.CntExam AS examCnt
    FROM (
        SELECT  tStudents.StudentID,
                vRoadMap.DisciplineID,
                SUM(CASE WHEN vRoadMap.ModuleType = 'regular' THEN  rt.Rate ELSE  0 END) AS RateRegular,
                SUM(CASE WHEN vRoadMap.ModuleType = 'extra' THEN  rt.Rate ELSE  0 END) AS RateExtra,
                SUM(CASE WHEN vRoadMap.ModuleType = 'bonus' THEN  rt.Rate ELSE  0 END) AS RateBonus,
                MAX(CASE WHEN vRoadMap.ModuleType = 'exam' THEN  rt.Rate ELSE  0 END) AS RateExam,
                SUM(CASE WHEN vRoadMap.ModuleType = 'exam' THEN  1 ELSE  0 END) AS CntExam
            FROM tStudents
            CROSS JOIN vRoadMap
            LEFT JOIN rating_table as rt  ON  rt.StudentID = tStudents.StudentID AND
                                                rt.SubmoduleID = vRoadMap.SubmoduleID
            WHERE rt.Rate IS NOT NULL
            GROUP BY tStudents.StudentID, vRoadMap.DisciplineID
    ) vRates
        INNER JOIN students ON students.ID = vRates.StudentID
        INNER JOIN accounts ON students.AccountID = accounts.ID
        INNER JOIN disciplines ON disciplines.id = vRates.DisciplineID
        ORDER BY    CONCAT(accounts.LastName, accounts.FirstName,  coalesce(accounts.SecondName,'')) ASC,
                    vRates.DisciplineID ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getratesforgroupbystage(pdisciplineid integer, pgroupid integer, pmilestone integer)
 RETURNS TABLE("StudentID" integer, "Semester" integer, "Bonus" integer, "Extra" integer, "PreviousExtra" integer, "Exam" integer, "PreviousExam" integer, "Option" absence_pass, "AutoPassed" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying)
 LANGUAGE plpgsql
AS $function$

    DECLARE vSemesterID INT DEFAULT -1; vGroupID INT DEFAULT -1;
    vInGeneralGroup BOOL DEFAULT FALSE;
BEGIN
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');
    /*DROP TABLE IF EXISTS tStudents;
    CREATE TEMPORARY TABLE tStudents (
        StudentID INT NOT NULL
    );*/

    -- check that group attached to discipline. Otherwise vGroupID = -1;
    vInGeneralGroup = EXISTS(
        SELECT * FROM disciplines_groups
        WHERE disciplines_groups.DisciplineID = pDisciplineID AND
              disciplines_groups.GroupID = pGroupID
        LIMIT 1
    );


    DROP TABLE IF EXISTS tStudents;
    CREATE TEMPORARY TABLE tStudents AS (
      SELECT record_books.ID, record_books.StudentID
      FROM students_groups
        LEFT JOIN record_books ON record_books.ID = students_groups.RecordBookID
        LEFT JOIN disciplines_students ON disciplines_students.DisciplineID = pDisciplineID AND
                                            disciplines_students.RecordBookID = record_books.ID
      WHERE   students_groups.SemesterID = vSemesterID AND
              students_groups.State <= 'outlet' AND --actual students
              students_groups.GroupID = pGroupID and
              COALESCE(disciplines_students.Type, 'attach') = 'attach'
              /*CASE WHEN vInGeneralGroup THEN
                NOT disciplines_students.Type <=> 'detach' # not detached
              ELSE
                COALESCE(disciplines_students.Type, 'attach') = 'attach' # is attached
              END*/
    );

  return query
    SELECT  tRes.*,
            accounts.LastName as "LastName",
            accounts.FirstName as "FirstName",
            accounts.SecondName as "SecondName"
        FROM (
            SELECT  tStudents.StudentID as "StudentID",
                    SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'regular',1,0)) AS "Semester",
                    SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'bonus',1,0)) AS "Bonus",
                    SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) AS "Extra",
                    SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'extra',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone - 1,1,0)) AS "PreviousExtra",
                    SUM(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum = pMilestone,1,0)) AS "Exam",
                    MAX(tRate.Rate*iif_sql(tMap.ModuleType = 'exam',1,0)*iif_sql(tMap.SubmoduleOrderNum < pMilestone,1,0)) AS "PreviousExam",
                    MAX(CASE WHEN tMap.SubmoduleOrderNum = pMilestone THEN  exam_period_options."type" ELSE  NULL END) As "Option",
                    MAX(CASE WHEN exam_period_options."type" = 'pass' THEN  1 ELSE  0 END) As "AutoPassed"
                FROM tStudents
                LEFT JOIN view_roadmap AS tMap   ON  tMap.DisciplineID = pDisciplineID
                LEFT JOIN rating_table AS tRate  ON  tRate.StudentID = tStudents.StudentID AND
                                                    tRate.SubmoduleID = tMap.SubmoduleID
                LEFT JOIN exam_period_options ON  exam_period_options.submoduleID = tMap.SubmoduleID AND
                                                    exam_period_options.StudentID = tStudents.StudentID
                GROUP BY tStudents.StudentID
            ) tRes
        INNER JOIN students ON students.ID = tRes.StudentID
        INNER JOIN accounts ON students.AccountID = accounts.ID
        ORDER BY    accounts.LastName ASC,
                    accounts.FirstName ASC,
                    accounts.SecondName ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getrequests(poffset integer, pcount integer, paccountid integer, pfilter opened_processed_closed_all)
 RETURNS void
 LANGUAGE plpgsql
AS $function$
BEGIN
    SELECT  requests.*, GetUserFullNameByAccountID(accounts.ID) as FullName, accounts.UserRoleID,
      GetUserStudentOrTeacherID(AccountID, accounts.UserRoleID) as PersonalID
    FROM requests join accounts on requests.AccountID = accounts.ID
    WHERE   if (pFilter = 'all', TRUE, requests.Status = pFilter) AND
            requests.Title != '' AND
            requests.Description != ''
    ORDER BY requests.Date DESC
    LIMIT pCount OFFSET pOffset;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getroadmap(pdisciplineid integer, ptype exam_rate_all)
 RETURNS TABLE("ModuleID" integer, "ModuleName" character varying, "ModuleType" regular_exam_bonus_extra, "SubmoduleID" integer, "SubmoduleName" character varying, "MaxRate" integer, "SubmoduleType" current_landmark_control)
 LANGUAGE plpgsql
AS $function$
begin
	return query
    SELECT  view_roadmap.ModuleID 	as "ModuleID",
            view_roadmap.ModuleName	as "ModuleName",
            view_roadmap.ModuleType	as "ModuleType",
            view_roadmap.SubmoduleID as "SubmoduleID",
            view_roadmap.SubmoduleName as "SubmoduleName",
            view_roadmap.SubmoduleRate AS "MaxRate",
            view_roadmap.SubmoduleType as "SubmoduleType"
        FROM view_roadmap
        WHERE   view_roadmap.DisciplineID = pDisciplineID AND
                CASE pType
                    WHEN 'exam' THEN view_roadmap.ModuleType = 'exam' OR view_roadmap.ModuleType = 'extra'
                    WHEN 'rate' THEN view_roadmap.ModuleType != 'exam' OR view_roadmap.SubmoduleOrderNum = 1
                    ELSE TRUE
                END
        ORDER by
                 view_roadmap.ModuleType,
                /*CASE pType
                    WHEN 'exam' THEN InternalOrderModuleTypesForSession(view_roadmap.ModuleType)
                    WHEN 'rate' THEN view_roadmap.ModuleType ^ 1 -- 1, 3, 2, 4 ASC
                    ELSE TRUE
                END ASC,*/
                view_roadmap.ModuleOrderNum ASC,
                view_roadmap.SubmoduleOrderNum ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getsigninhistory(plimit integer)
 RETURNS void
 LANGUAGE plpgsql
AS $function$
BEGIN
    SELECT  logs_signin.ID,
            logs_signin.Date,
            logs_signin.AccountID,
            teachers.LastName,
            teachers.FirstName,
            teachers.SecondName
        FROM logs_signin
        INNER JOIN teachers ON teachers.AccountID = logs_signin.AccountID
        ORDER BY  logs_signin.Date DESC LIMIT pLimit;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getspecializations(pfacultyid integer)
 RETURNS TABLE("ID" integer, "Name" character varying, "Abbr" character varying, "Code" character varying, "FaculyID" integer)
 LANGUAGE plpgsql
AS $function$

begin
	return query
SELECT  specializations.id as "ID", -- ID, Name, Abbr, FacultyID
		specializations."name" as "Name",
		specializations.abbr as "Abbr",
		specializations.code as "Code",
		specializations.facultyid as "FaculyID"
    FROM specializations
    WHERE specializations.FacultyID = pFacultyID
    ORDER BY subjects.Name ASC;
    end;
$function$;

CREATE OR REPLACE FUNCTION public.getstudentgroup(precordbookid integer, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
begin
--	select public.GetStudentGroup(
--	:pRecordBookID,	-- put the pRecordBookID parameter value instead of 'pRecordBookID' (INT)
--	:pSemesterID 	-- put the pSemesterID parameter value instead of 'pSemesterID' (INT)
--);

RETURN COALESCE((
    SELECT students_groups.GroupID FROM students_groups
        join record_books on record_books.id=students_groups.recordbookid
        WHERE   record_books.id=precordbookID
                AND students_groups.SemesterID = pSemesterID
                AND students_groups.State != 'expulsion'
        LIMIT 1
), -1);
END
$function$;

CREATE OR REPLACE FUNCTION public.getstudents(pfacultyid integer, pgradeid integer, pgroupid integer, psemesterid integer, pword1 character varying, pword2 character varying, pword3 character varying)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer)
 LANGUAGE plpgsql
AS $function$
  DECLARE FullName VARCHAR;
begin
	return query
  SELECT  view_students.StudentID AS "ID",
    view_students.LastName AS "LastName",
    view_students.FirstName AS "FirstName",
    view_students.SecondName AS "SecondName",
    view_students.AccountID AS "AccountID",
    view_students.GradeID AS "GradeID",
    view_students.GradeNum AS "GradeNum",
    view_students.Degree AS "Degree",
    view_students.GroupID AS "GroupID",
    view_students.GroupNum AS "GroupNum"
  FROM view_students
    WHERE
    CASE WHEN pSemesterID != 0 THEN view_students.SemesterID = pSemesterID ELSE
    TRUE
    END AND
      CASE WHEN pFacultyID != 0 THEN view_students.FacultyID = pFacultyID ELSE TRUE END AND
    CASE WHEN pGradeID != 0 THEN view_students.GradeID = pGradeID ELSE TRUE END AND
    CASE WHEN pGroupID != 0 THEN view_students.GroupID = pGroupID ELSE TRUE END AND
    CASE WHEN pWord1 != '' AND FullName != '' THEN
	  -- нужно переделать через курсор 
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord1, '%') AND 
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord2, '%') AND 
      CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) LIKE CONCAT('%', pWord3, '%')
      ELSE TRUE END;
END
$function$;

CREATE OR REPLACE FUNCTION public.getstudentsforrating(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "RecordBookID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "IsAttached" boolean)
 LANGUAGE plpgsql
AS $function$

    DECLARE vSemesterID INT DEFAULT -1; vRes INT DEFAULT -1;
BEGIN
    vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');

    DROP TABLE IF EXISTS tDisciplineStudents;
    vRes := Discipline_GetStudentsTemp(pDisciplineID);
   return query
    SELECT  view_students.StudentID 		AS "ID",
            view_students.RecordBookID		as "RecordBookID",
            view_students.LastName			as "LastName",
            view_students.FirstName			as "FirstName",
            view_students.SecondName		as "SecondName",
            view_students.GradeID			as "GradeID",
            view_students.GradeNum			as "GradeNum",
            view_students."degree"			as "Degree",
            view_students.GroupID			as "GroupID",
            view_students.GroupNum			as "GroupNum",
            ( tDisciplineStudents."Type" IS NOT NULL ) AS "IsAttached"
        FROM tDisciplineStudents
        INNER JOIN view_students ON   view_students.StudentID = tDisciplineStudents.StudentID AND
                                        view_students.SemesterID = vSemesterID
        WHERE  tDisciplineStudents."Type" is null or ( tDisciplineStudents."Type" !='detach')
        ORDER BY    view_students.GradeID ASC,
                    view_students.GroupNum ASC,
                    view_students.LastName ASC,
                    view_students.FirstName ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getsubjects(pfacultyid integer)
 RETURNS TABLE("ID" integer, "Title" character varying, "Abbr" character varying)
 LANGUAGE plpgsql
AS $function$
begin
	return query
    SELECT  subjects.ID as "ID",
            subjects."name"  AS "Title",
            subjects.Abbr as "Abbr"
        FROM subjects_faculties
        INNER JOIN subjects ON subjects_faculties.SubjectID = subjects.ID
        WHERE subjects_faculties.FacultyID = pFacultyID
        ORDER BY subjects."name" ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getteachers(pfacultyid integer, pdepartmentid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "JobPositionName" character varying, "DepID" integer, "DepName" character varying)
 LANGUAGE plpgsql
AS $function$
begin
	return query
    SELECT  view_teachers.TeacherID AS "ID",
            view_teachers.LastName as "LastName",
            view_teachers.FirstName as "FirstName",
            view_teachers.SecondName as "SecondName",
            view_teachers.AccountID as "AccountID",
            view_teachers.JobPositionName as "JobPositionName",
            view_teachers.DepID as "DepID",
            view_teachers.DepName as "DepName"
        FROM view_teachers
        WHERE
            CASE WHEN pFacultyID != 0 THEN view_teachers.FacultyID = pFacultyID ELSE TRUE END AND
            CASE WHEN pDepartmentID != 0 THEN view_teachers.DepID = pDepartmentID ELSE TRUE END
        ORDER BY    view_teachers.LastName ASC,
                    view_teachers.FirstName ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.getteachersfordiscipline(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "JobPositionID" integer, "JobPositionName" character varying, "DepID" integer, "DepName" character varying, "FacultyID" integer, "FacultyAbbr" character varying, "IsAuthor" boolean)
 LANGUAGE plpgsql
AS $function$

    DECLARE vAuthorID INT DEFAULT -1;
BEGIN
    vAuthorID := GetDisciplineProperty(pDisciplineID, 'author');
 return query
    SELECT  view_teachers.TeacherID AS "ID",
            view_teachers.LastName as "LastName",
            view_teachers.FirstName as "FirstName",
            view_teachers.SecondName as "SecondName",
            view_teachers.JobPositionID as "JobPositionID",
            view_teachers.JobPositionName as "JobPositionName",
            view_teachers.DepID as "DepID",
            view_teachers.DepName as "DepName",
            view_teachers.FacultyID as "FacultyID",
            view_teachers.FacultyAbbr as "FacultyAbbr",
            ( view_teachers.TeacherID = vAuthorID ) AS "IsAuthor"
        FROM disciplines_teachers
        INNER JOIN view_teachers ON view_teachers.TeacherID = disciplines_teachers.TeacherID
        WHERE disciplines_teachers.DisciplineID = pDisciplineID
        ORDER BY    view_teachers.TeacherID = vAuthorID DESC,
                    view_teachers.LastName ASC,
                    view_teachers.FirstName ASC;
END;
$function$;

CREATE OR REPLACE FUNCTION public.gettextmark(isafterexam character, examtype character varying, rate integer)
 RETURNS character varying
 LANGUAGE plpgsql
AS $function$

	declare markTextId int;
    markText varchar(100);
begin
--	select public.gettextmark(
--	:isafterexam,	-- put the isafterexam parameter value instead of 'isafterexam' (bpchar)
--	:examtype,	-- put the examtype parameter value instead of 'examtype' (varchar)
--	:rate 	-- put the rate parameter value instead of 'rate' (int4)
--);
    if examType = 'credit' then
        if rate < 60 then
			markTextId := 6;--незачет
		else 
			markTextId := 7;--зачет
		end if;
    elseif examType = 'gradingcredit' then
		if rate >= 85 then 
			markTextId := 5;--неуд.
		elseif rate >= 70 then 
			markTextId := 4;--хорошо
		elseif rate >= 60 then
			markTextId := 3;--удовл.
		else
			markTextId := 2;--неуд.
		end if;
    elseif isAfterExam = '0' then
		if rate < 38 then
			markTextId := 1;--недопуск
		else
			return ' ' ;
		end if;
	else
		if rate >= 85 then 
			markTextId := 5;--неуд.
		elseif rate >= 70 then 
			markTextId := 4;--хорошо
		elseif rate >= 60 then
			markTextId := 3;--удовл.
		else
			markTextId := 2;--неуд.
		end if;
	end if;
    select name from text_mark where id = markTextId into markText;
    return markText;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getuserfullnamebyaccountid(paccountid integer)
 RETURNS character varying
 LANGUAGE plpgsql
AS $function$

    DECLARE vUserFullName VARCHAR(255);
    vChecker INT DEFAULT null;
BEGIN
--select public.getuserfullnamebyaccountid(
--	:paccountid 	-- put the paccountid parameter value instead of 'paccountid' (int4)
--);
    -- try to find student with that account id
    SELECT students.ID, CONCAT(students.LastName,' ',students.FirstName,' ',students.SecondName)
        INTO vChecker, vUserFullName
        FROM students
        WHERE students.AccountID = pAccountID
        LIMIT 1;

    IF vChecker is null THEN -- try to find teacher with that account id
        SELECT teachers.ID, CONCAT(teachers.LastName,' ',teachers.FirstName,' ',teachers.SecondName)
            INTO vChecker, vUserFullName
            FROM teachers
            WHERE teachers.AccountID = pAccountID
            LIMIT 1;

        IF vChecker is null THEN
            RETURN '';
        END IF;
    END IF;

    RETURN vUserFullName;
END;
$function$;

CREATE OR REPLACE FUNCTION public.getuserrole(ptype student_teacher_admin_dean)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

BEGIN
--select public.getuserrole(
--	:ptype 	-- put the ptype parameter value instead of 'ptype' (student_teacher_admin_dean)
--);
RETURN (CASE pType
    WHEN 'dean' THEN 4
    WHEN 'teacher' THEN 2
    WHEN 'admin' THEN 3
    ELSE 1
END);
end
$function$;

CREATE OR REPLACE FUNCTION public.getuserstudentorteacherid(paccountid integer, puserroleid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vID INT DEFAULT null;
BEGIN

    IF pUserRoleID = GetUserRole('student') THEN
        SELECT students.ID
        INTO vID
        FROM students
        WHERE students.AccountID = pAccountID
        LIMIT 1;
    ELSE
        SELECT teachers.ID
        INTO vID
        FROM teachers
        WHERE teachers.AccountID = pAccountID
        LIMIT 1;
    END IF;

    RETURN coalesce(vID,-1);
END;
$function$;

CREATE OR REPLACE FUNCTION public.grade_getid(gradenum integer, pdegree bachelor_master_specialist)
 RETURNS TABLE("ID" integer)
 LANGUAGE plpgsql
AS $function$
begin
    return query
    SELECT  grades.ID as "ID"
        FROM grades
        WHERE grades.Num = gradeNum AND grades.Degree = pdegree;
end;
$function$;"GetStudents"

CREATE OR REPLACE FUNCTION public.internalismapcreated(pdisciplineid integer)
 RETURNS boolean
 LANGUAGE plpgsql
AS $function$
--select public.internalismapcreated(
--	:pdisciplineid-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
declare maxRate integer;
BEGIN
    maxRate=null;
    SELECT disciplines.MaxRate FROM disciplines into maxRate
    WHERE disciplines.ID = pDisciplineID ;
    RETURN (maxrate is not null and maxrate=100);
END
$function$;

CREATE OR REPLACE FUNCTION public.internalismaplocked(pdisciplineid integer)
 RETURNS boolean
 LANGUAGE plpgsql
AS $function$
--select public.internalismaplocked(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
declare isLocked integer;
BEGIN
	isLocked =null;
	SELECT disciplines.IsLocked FROM disciplines into isLocked
    WHERE disciplines.ID = pDisciplineID;
    return (isLocked is not null and isLocked =1);
END
$function$;

CREATE OR REPLACE FUNCTION public.internalisstudentattached(precordbookid integer, pdisciplineid integer)
 RETURNS boolean
 LANGUAGE plpgsql
AS $function$
    DECLARE vAttachType attach_detach DEFAULT 'detach';
BEGIN
--select public.internalisstudentattached(
--	:pRecordBookID,	-- put the pstudentid parameter value instead of 'pRecordBookID' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);

    SELECT view_disciplines_recordbooks.Type INTO vAttachType
    FROM view_disciplines_recordbooks
    WHERE view_disciplines_recordbooks.RecordBookID = pRecordBookID AND
          view_disciplines_recordbooks.DisciplineID = pDisciplineID
    LIMIT 1;
	RETURN 'attach' = COALESCE(vAttachType, 'attach');
END;
$function$;

CREATE OR REPLACE FUNCTION public.internalisteacherauthor(pteacherid integer, pdisciplineid integer)
 RETURNS boolean
 LANGUAGE plpgsql
AS $function$
--select public.internalisteacherauthor(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
declare author integer;
BEGIN
    author = NULL; 
    SELECT disciplines.AuthorID FROM disciplines into author
    WHERE disciplines.ID = pDisciplineID;
    return (author is not null and author = pTeacherID);
END
$function$;

CREATE OR REPLACE FUNCTION public.internalisteacherbound(pteacherid integer, pdisciplineid integer)
 RETURNS boolean
 LANGUAGE plpgsql
AS $function$
--select public.internalisteacherbound(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--  :pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
BEGIN
RETURN EXISTS (
    SELECT * FROM disciplines_teachers
    WHERE disciplines_teachers.TeacherID = pTeacherID AND disciplines_teachers.DisciplineID = pDisciplineID
    );
END
$function$;

CREATE OR REPLACE FUNCTION public.internalordermoduletypesforsession(pmoduletype integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
--select public.internalordermoduletypesforsession(
--	:pmoduletype 	-- put the pmoduletype parameter value instead of 'pmoduletype' (int4)
--);
BEGIN
RETURN (CASE pModuleType
    WHEN 4 THEN 1 -- extra
    WHEN 2 THEN 2 -- exam
    WHEN 3 THEN 3 -- bonus
    ELSE 4
END);
end
$function$;

CREATE OR REPLACE FUNCTION public.internalordermoduletypesforsession(pmoduletype regular_exam_bonus_extra)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
--select public.internalordermoduletypesforsession(
--	:pmoduletype 	-- put the pmoduletype parameter value instead of 'pmoduletype' (int4)
--);

BEGIN
RETURN (CASE pModuleType
    WHEN 'extra' THEN 1 -- extra
    WHEN 'exam'  THEN 2 -- exam
    WHEN 'bonus' THEN 3 -- bonus
    ELSE 4
END);
end
$function$;

CREATE OR REPLACE FUNCTION public.jobposition_create(pjobpositionname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vInsertID integer;
begin
--	select public.jobposition_create(
--	:pjobpositionname 	-- put the pjobpositionname parameter value instead of 'pjobpositionname' (varchar)
--);
  INSERT INTO job_positions (Name) VALUES (pJobPositionName) returning id into vInsertID;
  RETURN vInsertID;
 --exception
 -- when others then RETURN -1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.logbind(pdisciplineid integer, pteacherid integer, entityid integer, pattach attach_detach, ptype group_student)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
 declare countR int default 0;
BEGIN 
--	select public.logbind(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:entityid,	-- put the entityid parameter value instead of 'entityid' (int4)
--	:pattach,	-- put the pattach parameter value instead of 'pattach' (attach_detach)
--	:ptype 	-- put the ptype parameter value instead of 'ptype' (group_student)
--);

    IF pType = 'group' THEN
        INSERT INTO logs_binds_groups
            (DisciplineID, TeacherID, GroupID, Type)
            VALUES (pDisciplineID, pTeacherID, EntityID, pAttach);
    ELSEIF pType = 'student' THEN
        INSERT INTO logs_binds_students
            (DisciplineID, TeacherID, StudentID, Type)
            VALUES (pDisciplineID, pTeacherID, EntityID, pAttach);
    END IF;
    get diagnostics countR = ROW_COUNT;
    RETURN countR-1;
  -- EXCEPTION 
 --   when others then RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.plan_binddiscipline(pstudyplanid integer, pdisciplineid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
BEGIN
--    select public.Plan_BindDiscipline(
--	:pStudyPlanID,	-- put the pStudyPlanID parameter value instead of 'pStudyPlanID' (INT)
--	:pDisciplineID 	-- put the pDisciplineID parameter value instead of 'pDisciplineID' (INT)
--);   
        INSERT INTO disciplines_study_plans (StudyPlanID, DisciplineID) VALUES (pStudyPlanID, pDisciplineID)
         on conflict on constraint disciplines_study_plans_studyplanid_disciplineid_key
         do nothing;
        RETURN 0;
     --  EXCEPTION 
     --  when others then RETURN -1;
    END
$function$;

CREATE OR REPLACE FUNCTION public.plan_changeinfo(pid integer, pyear integer, pfacultyid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
begin
--	select public.Plan_ChangeInfo(
--	:pID,	-- put the pID parameter value instead of 'pID' (INT)
--	:pYear,	-- put the pYear parameter value instead of 'pYear' (INT)
--	:pFacultyID 	-- put the pFacultyID parameter value instead of 'pFacultyID' (INT)
--);
    UPDATE study_plans
        SET study_plans.Year = COALESCE(pYear, study_plans.Year),
            study_plans.FacultyID = COALESCE(pFacultyID, study_plans.FacultyID)
        WHERE study_plans.ID = pID;
    RETURN 0;
 --  EXCEPTION 
 --   when others then RETURN -1;
END
 $function$;

CREATE OR REPLACE FUNCTION public.plan_create(pplanexternalid integer, pyear integer, pfacultyid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
        DECLARE vID INT DEFAULT -1;
begin
--	select public.Plan_Create(
--	:pPlanExternalID,	-- put the pPlanExternalID parameter value instead of 'pPlanExternalID' (VARCHAR(9))
--	:pYear,	-- put the pYear parameter value instead of 'pYear' (INT)
--	:pFacultyID 	-- put the pFacultyID parameter value instead of 'pFacultyID' (INT)
--);
        INSERT INTO study_plans (ExternalID, Year, FacultyID) VALUES (pPlanExternalID, pYear, pFacultyID)
         on conflict on constraint study_plans_externalid_key
         do UPDATE set Year = pYear, FacultyID = pFacultyID
            returning id into vID;
        
        RETURN vID;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.plan_create(pplanexternalid integer, pyear integer, pfacultyid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
        DECLARE vID INT DEFAULT -1;
begin
--	select public.Plan_Create(
--	:pPlanExternalID,	-- put the pPlanExternalID parameter value instead of 'pPlanExternalID' (VARCHAR(9))
--	:pYear,	-- put the pYear parameter value instead of 'pYear' (INT)
--	:pFacultyID 	-- put the pFacultyID parameter value instead of 'pFacultyID' (INT)
--);
        INSERT INTO study_plans (ExternalID, Year, FacultyID) VALUES (pPlanExternalID, pYear, pFacultyID)
         on conflict on constraint study_plans_externalid_key
         do UPDATE set study_plans.Year = pYear, study_plans.FacultyID = pFacultyID
            returning id into vID;
        
        RETURN vID;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.plan_getidfromexternalid(pplanexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
        DECLARE vID INT DEFAULT null;
begin
--	select public.plan_getidfromexternalid(
--	:pplanexternalid 	-- put the pplanexternalid parameter value instead of 'pplanexternalid' (varchar)
--);
        SELECT study_plans.ID INTO vID
        FROM study_plans
        WHERE study_plans.ExternalID = cast(pPlanExternalID as integer)
        LIMIT 1;

        RETURN coalesce(vID,-1);
    END
 $function$;

CREATE OR REPLACE FUNCTION public.recordbook_bind(pexternalid character varying, pstudentid integer, pplanid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
        DECLARE vRecordBookID INT DEFAULT -1;
BEGIN
--select public.recordbook_bind(
--	:pexternalid,	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--	:pstudentid,	-- put the pstudentid parameter value instead of 'pstudentid' (int4)
--	:pplanid 	-- put the pplanid parameter value instead of 'pplanid' (int4)
--);
        INSERT INTO record_books (StudentID, ExternalID)
            VALUES  (pStudentID, pExternalID)
           on conflict on constraint record_books_externalid_key do
           update set StudentID = pStudentID
                      -- record_books.ID = LAST_INSERT_ID(record_books.ID)
                     returning ID into vRecordBookID;

        INSERT INTO record_books_plans (RecordBookID, StudyPlanID)
            VALUES (vRecordBookID, pPlanID)
            on conflict on constraint recordbookid_studyplanid_record_books_plans_key do
            nothing;

        RETURN vRecordBookID;

     --  EXCEPTION
    --     when others then RETURN -1;
    END
 $function$
;

CREATE OR REPLACE FUNCTION public.recordbook_changeinfo(pid integer, pexternalid character varying, pstudentid integer, pplanid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
 declare vRes int;
begin
--	select public.recordbook_changeinfo(
--	:pid,	-- put the pid parameter value instead of 'pid' (int4)
--	:pexternalid,	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--	:pstudentid,	-- put the pstudentid parameter value instead of 'pstudentid' (int4)
--	:pplanid 	-- put the pplanid parameter value instead of 'pplanid' (int4)
--);


        UPDATE record_books
            SET record_books.ExternalID = COALESCE(pExternalID, record_books.ExternalID)
            WHERE record_books.ID = pID;
        SELECT RecordBook_Bind(pExternalID, pStudentID, pPlanID) INTO vRes;
        RETURN 0;
      -- EXCEPTION 
      -- when others then RETURN -1;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.recordbook_getidfromexternalid(precordbookexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
DECLARE pID INT DEFAULT null;
begin
--	select public.recordbook_getidfromexternalid(
--	:precordbookexternalid 	-- put the precordbookexternalid parameter value instead of 'precordbookexternalid' (varchar)
--);

        SELECT record_books.ID INTO pID
        FROM record_books
        WHERE record_books.ExternalID = pRecordBookExternalID
        LIMIT 1;
		if pID is null then return -1;
        else  RETURN pID;
        end if; 
    END
 $function$;

CREATE OR REPLACE FUNCTION public.removefromgroupinsemester(precordbook integer, pgroupid integer, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vRowCount int;
BEGIN
-- select public.RemoveFromGroupInSemester(
--	:pRecordBook,	-- put the pRecordBook parameter value instead of 'pRecordBook' (INT)
--	:pGroupID,	-- put the pGroupID parameter value instead of 'pGroupID' (INT)
--	:pSemesterID 	-- put the pSemesterID parameter value instead of 'pSemesterID' (INT)
--);   
    DELETE FROM students_groups
        WHERE   students_groups.GroupID = pGroupID
                and students_groups.pRecordBook = pRecordBook
                and students_groups.SemesterID = pSemesterID
                and students_groups.State = 'common';
    get diagnostics vRowCount = ROW_COUNT;
   RETURN vRowCount-1;
   --EXCEPTION 
  -- when others then RETURN -1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.removestudentsattach(pdisciplineid integer, pgroupid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vSemesterID INT; 
begin
--	select public.removestudentsattach(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pgroupid 	-- put the pgroupid parameter value instead of 'pgroupid' (int4)
--);

    vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');
    DELETE FROM disciplines_students
        WHERE   disciplines_students.DisciplineID = pDisciplineID AND
                disciplines_students.StudentID IN (
                    SELECT students_groups.StudentID FROM students_groups
                    WHERE   students_groups.GroupID = pGroupID AND students_groups.SemesterID = vSemesterID
                );
        RETURN 0;
END;
$function$;

CREATE OR REPLACE FUNCTION public.requestsnum(pfilter opened_processed_closed_all)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vtype opened_processed_closed;
begin
--select public.requestsnum(
--	:pfilter 	-- put the pfilter parameter value instead of 'pfilter' (opened_processed_closed_all)
--);
    if (pFilter != 'all') then
     vtype = cast (pFilter as opened_processed_closed);
    end if;
    SELECT COUNT(*)     AS Num
    FROM requests
    WHERE iif_sql(pFilter = 'all', TRUE, requests.Status =  vtype) AND
            requests.Title != '' AND
            requests.Description != '';
end
$function$;

CREATE OR REPLACE FUNCTION public.restrictaftermilestone(pdisciplineid integer, pmilestone integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
begin
--	select public.restrictaftermilestone(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pmilestone 	-- put the pmilestone parameter value instead of 'pmilestone' (int4)
--);

    UPDATE disciplines
        SET disciplines.MilestoneDate = current_date,
            disciplines.Milestone = pMilestone
        WHERE disciplines.ID = pDisciplineID;
       -- LIMIT 1;
    RETURN 0;
END;
$function$;

CREATE OR REPLACE FUNCTION public.restrictaftermilestoneforcredits(pfacultyid integer, pmilestone integer, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
begin
--	select public.restrictaftermilestoneforcredits(
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:pmilestone,	-- put the pmilestone parameter value instead of 'pmilestone' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);

    UPDATE disciplines
        SET disciplines.MilestoneDate = current_date,
            disciplines.Milestone = pMilestone
        WHERE   disciplines.FacultyID= pFacultyID AND
                disciplines.SemesterID = pSemesterID AND
                ( disciplines.ExamType = 'credit' OR disciplines.ExamType = 'grading_credit');
    RETURN 0;
END;
$function$;

CREATE OR REPLACE FUNCTION public.searchstudents(pgradeid integer, pgroupid integer, pfacultyid integer, pfullname character varying, pdisciplineid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" integer, "GroupID" integer, "GroupNum" integer)
 LANGUAGE plpgsql
AS $function$
  DECLARE vSemesterID INTEGER DEFAULT -1;
  DECLARE vRes INTEGER DEFAULT -1;
BEGIN

  DROP TABLE IF EXISTS tDisciplineStudents;
  SELECT Discipline_GetStudentsTemp(pDisciplineID) INTO vRes;

  SELECT  view_students.StudentID AS "ID",
    view_students.LastName AS "LastName",
    view_students.FirstName AS "FirstName",
    view_students.SecondName AS "SecondName",
    view_students.GradeID AS "GradeID",
    view_students.GradeNum AS "GradeNum",
    view_students.Degree AS "Degree",
    view_students.GroupID AS "GroupID",
    view_students.GroupNum AS "GroupNum",
    CONCAT(view_students.LastName, ' ', view_students.FirstName, ' ', view_students.SecondName) AS "FullName"
  FROM view_students
    LEFT JOIN tDisciplineStudents ON tDisciplineStudents.StudentID = view_students.StudentID
  WHERE   view_students.SemesterID = vSemesterID AND
          view_students.FacultyID = pFacultyID AND
          view_students.GradeID = pGradeID AND
          (pGroupID = 0 OR view_students.GroupID = pGroupID) AND
          tDisciplineStudents.StudentID IS NULL AND
          CASE WHEN pFullName != '' THEN
            FullName LIKE CONCAT('%', pFullName, '%')
          ELSE TRUE END
  ORDER BY    view_students.GradeID ASC,
    view_students.GroupID ASC,
    view_students.LastName ASC,
    view_students.FirstName ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.searchteachernew(facultyid integer, departmentid integer, last character varying, first character varying, second character varying, disciplineid integer)
 RETURNS void
 LANGUAGE plpgsql
AS $function$
BEGIN
	SELECT 	teachers.ID AS TeacherID, 
	teachers.LastName AS TeacherLast, 
	teachers.FirstName AS TeacherFirst, 
	teachers.SecondName AS TeacherSecond, 
	teachers.AccountID AS TeacherAccID,
	job_positions.Name AS JobPositionName, 
	departments.ID AS DepID,
	departments.Name AS DepName
	FROM teachers
	INNER JOIN departments ON teachers.DepartmentID = departments.ID AND
									( DepartmentID = 0 OR DepartmentID = departments.ID ) AND
									( FacultyID = 0 OR FacultyID = departments.FacultyID ) 
	INNER JOIN job_positions 	ON 	teachers.JobPositionID = job_positions.ID
	WHERE	NOT InternalIsTeacherBinded(teachers.ID, DisciplineID) AND
			( teachers.LastName LIKE CONCAT(Last,"%") ) AND 
			( teachers.FirstName LIKE CONCAT(First,"%") ) AND
			( teachers.SecondName LIKE CONCAT(Second,"%") )
	ORDER BY 	departments.FacultyID ASC, 
				departments.Name ASC, 
				teachers.LastName ASC, 
				teachers.FirstName ASC, 
				teachers.SecondName ASC;
END
$function$;

CREATE OR REPLACE FUNCTION public.searchteachers(pfacultyid integer, pdepartmentid integer, pword1 character varying, pword2 character varying, pword3 character varying, pdisciplineid integer)
 RETURNS TABLE("AccountID" integer, "ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "JobPositionName" character varying, "DepID" integer, "DepName" character varying, "IsAuthor" boolean)
 LANGUAGE plpgsql
AS $function$ 
 DECLARE vWord1 varchar; vWord2 varchar; vWord3 varchar;
 DECLARE vAuthorID INT DEFAULT null;
 
begin
	vWord1 = CONCAT('%', pWord1, '%');
    vWord2 = CONCAT('%', pWord2, '%');
    vWord3 = CONCAT('%', pWord3, '%');

    vAuthorID = GetDisciplineProperty(pDisciplineID, 'author');
   DROP TABLE IF EXISTS tDiscTeachers;
    CREATE TEMPORARY TABLE tDiscTeachers AS (
        SELECT disciplines_teachers.TeacherID
            FROM disciplines_teachers
            WHERE disciplines_teachers.DisciplineID = pDisciplineID
    );
	return query
	SELECT  tTeachers.AccountID as "AccountID",
            tTeachers.TeacherID AS "ID",
            tTeachers.LastName as "LastName",
            tTeachers.FirstName as "FirstName",
            tTeachers.SecondName as "SecondName",
            tTeachers.JobPositionName as "JobPositionName",
            tTeachers.DepID as "DepID",
            tTeachers.DepName as "DepName",
            ( tTeachers.TeacherID = vAuthorID ) AS "IsAuthor"
        FROM view_teachers AS tTeachers
        WHERE   CASE WHEN pFacultyID != 0     THEN tTeachers.FacultyID = pFacultyID ELSE TRUE END AND
                CASE WHEN pDepartmentID != 0  THEN tTeachers.DepID = pDepartmentID  ELSE TRUE END AND
                NOT EXISTS (
                    SELECT * FROM tDiscTeachers
                    WHERE tDiscTeachers.TeacherID = tTeachers.TeacherID
                    LIMIT 1
                ) AND
                CASE WHEN pWord1 != '' and CONCAT(tTeachers.LastName, ' ', tTeachers.FirstName, ' ', coalesce(tTeachers.SecondName,'')) != ''
                   /*(@full := CONCAT(tTeachers.LastName, ' ', tTeachers.FirstName, ' ', tTeachers.SecondName)) != '' THEN
                         @full LIKE vWord1 AND
                         @full LIKE vWord2 AND
                         @full LIKE vWord3*/
                    then -- лучше пока не придумала
                       CONCAT(tTeachers.LastName, ' ', tTeachers.FirstName, ' ', coalesce(tTeachers.SecondName,'')) like vWord1 and
                       CONCAT(tTeachers.LastName, ' ', tTeachers.FirstName, ' ', coalesce(tTeachers.SecondName,'')) like vWord2 and
                       CONCAT(tTeachers.LastName, ' ', tTeachers.FirstName, ' ', coalesce(tTeachers.SecondName,'')) like vWord3
                    ELSE TRUE
                END
        ORDER BY    tTeachers.FacultyID ASC,
                    tTeachers.DepName ASC,
                    tTeachers.LastName ASC,
                    tTeachers.FirstName ASC;

end
 $function$;

CREATE OR REPLACE FUNCTION public.setrequeststatus(prequestid integer, pstatus opened_processed_closed)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
 declare vROWS integer;
BEGIN
--select public.setrequeststatus(
--	:prequestid,	-- put the prequestid parameter value instead of 'prequestid' (int4)
--	:pstatus 	-- put the pstatus parameter value instead of 'pstatus' (opened_processed_closed)
--);

    UPDATE requests
        SET requests.Status = pStatus
        WHERE requests.ID = pRequestID;
    get diagnostics vROWS = ROW_COUNT;
    RETURN vROWS-1;
   
  -- EXCEPTION 
  -- when others then RETURN -1;
END
$function$;

CREATE OR REPLACE FUNCTION public.setrole(pid integer, prole integer)
 RETURNS smallint
 LANGUAGE plpgsql
AS $function$
begin
--	select public.setrole(
--	:pid,	-- put the pid parameter value instead of 'pid' (int4)
--	:prole 	-- put the prole parameter value instead of 'prole' (int4)
--);

        UPDATE accounts
        SET accounts.UserRoleID = pRole
        WHERE accounts.ID = pID;
        RETURN 1;
       
      -- EXCEPTION 
     --  when others then
     --  RETURN 0;
    END;
$function$;

CREATE OR REPLACE FUNCTION public.signin(ploginormail character varying, ppassword character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vAccountID INT DEFAULT -1;
BEGIN
--select public.signin(
--	:ploginormail,	-- put the ploginormail parameter value instead of 'ploginormail' (varchar)
--	:ppassword 	-- put the ppassword parameter value instead of 'ppassword' (varchar)
--);
    --check account existence
    SELECT accounts.ID INTO vAccountID FROM accounts
        WHERE   accounts.Password = SHA1(pPassword) AND
                (accounts.Login = pLoginOrMail OR accounts.EMail = pLoginOrMail)
        LIMIT 1;
    IF vAccountID is null THEN
        RETURN -1;
    END IF;

    INSERT INTO logs_signin (AccountID) VALUES (vAccountID);
    RETURN vAccountID;
END;
$function$;

CREATE OR REPLACE FUNCTION public.signinbyopenid(pglobalkey character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
    DECLARE vAccountID INT DEFAULT null;
begin
--	select public.signinbyopenid(
--	:pglobalkey 	-- put the pglobalkey parameter value instead of 'pglobalkey' (varchar)
--);

    --check account existence
    SELECT accounts.ID INTO vAccountID FROM accounts
    WHERE concat('st-', accounts.ExternalID) = pGlobalKey
        OR  concat('ss-', accounts.INILA) = pGlobalKey
    LIMIT 1;
    IF vAccountID is null  THEN
        RETURN -1;
    END IF;

    INSERT INTO logs_signin (AccountID) VALUES (vAccountID);
    RETURN vAccountID;
END
 $function$;

CREATE OR REPLACE FUNCTION public.signinbytoken(ptoken character)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vAccountID INT DEFAULT -1;
BEGIN
--select public.signinbytoken(
--	:ptoken 	-- put the ptoken parameter value instead of 'ptoken' (bpchar)
--);
    SELECT auth_tokens.AccountID INTO vAccountID
        FROM auth_tokens
        WHERE auth_tokens.Token = pToken;
        --LIMIT 1;

    IF vAccountID is null THEN
        RETURN -1; -- token not found
    END IF;

    UPDATE auth_tokens
        SET Accessed = CURRENT_TIMESTAMP
        WHERE auth_tokens.Token = pToken;
        --LIMIT 1;

    INSERT INTO logs_signin(AccountID) VALUES (vAccountID);
    RETURN vAccountID;
END;
$function$;

CREATE OR REPLACE FUNCTION public.signinteacherbyopenid(pglobalkey character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
    DECLARE vAccountID INT DEFAULT null;
begin
--	select public.signinteacherbyopenid(
--	:pglobalkey 	-- put the pglobalkey parameter value instead of 'pglobalkey' (varchar)
--);

    --check account existence
    SELECT accounts.ID INTO vAccountID
    FROM accounts
    JOIN teachers ON accounts.ID = teachers.AccountID
    WHERE teachers.INILA = pGlobalKey
    LIMIT 1;
    IF vAccountID is null THEN
        RETURN -1;
    END IF;

    INSERT INTO logs_signin (AccountID) VALUES (vAccountID);
    RETURN vAccountID;
END
 $function$;

CREATE OR REPLACE FUNCTION public.student_changeinfo(pid integer, pexternalid character varying, plastname character varying, pfirstname character varying, psecondname character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
DECLARE vAccountID INT DEFAULT -1;
		vStudentID INT DEFAULT -1;
begin
--	select public.student_changeinfo(
--	:pid,	-- put the pid parameter value instead of 'pid' (int4)
--	:pexternalid,	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
--	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
--	:psecondname 	-- put the psecondname parameter value instead of 'psecondname' (varchar)
--);
        SELECT students.AccountID INTO vAccountID
            FROM students
            WHERE students.ID = pID;

        UPDATE accounts
            SET accounts.ExternalID = COALESCE(pExternalID, accounts.ExternalID),
                accounts.LastName = COALESCE(pLastName, accounts.LastName),
                accounts.FirstName = COALESCE(pFirstName, accounts.FirstName),
                accounts.SecondName = COALESCE(pSecondName, accounts.SecondName)
            WHERE accounts.ID = vAccountID;

        RETURN 0;
     -- EXCEPTION 
     -- when others then RETURN -1; 
    END
 $function$;

CREATE OR REPLACE FUNCTION public.student_getdisciplines(precordbookid integer, psemesterid integer)
 RETURNS TABLE("ID" integer, "SubjectID" integer, "SubjectName" character varying, "Type" exam_credit_grading_credit, "Subtype" scientific_disciplinary_coursework, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "Rate" bigint, "MaxCurrentRate" bigint)
 LANGUAGE plpgsql
AS $function$ 
DECLARE vRes INT DEFAULT -1;
begin
--	select public.student_getdisciplines(
--	:precordbookid,	-- put the precordbookid parameter value instead of 'precordbookid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);

	    DROP TABLE IF EXISTS tStudentDisciplines;
   		vRes = Student_GetDisciplinesTemp(pRecordBookID, pSemesterID);
   	return query
   	SELECT  view_disciplines.DisciplineID AS "ID",
            view_disciplines.SubjectID as "SubjectID",
            view_disciplines.SubjectName as "SubjectName",
            view_disciplines.ExamType AS "Type",
            view_disciplines.Subtype as "Subtype",
            accounts.LastName as "LastName",
            accounts.FirstName as "FirstName",
            accounts.SecondName as "SecondName",
            ( tDR.RateExam + tDR.RateMExam ) AS "Rate",
            ( tDR.MaxRegularRate + tDR.MaxExamRate ) AS "MaxCurrentRate"
        FROM (
            SELECT  tRating.DisciplineID,
                    SUM(
                        iif_sql( tRating.SubmoduleIsUsed AND 
                            tRating.ModuleType  is not null and 
                            tRating.ModuleType = 'regular',
                            tRating.SubmoduleRate, 0)
                    ) AS MaxRegularRate,
                    MAX(
                        iif_sql( tRating.SubmoduleIsUsed AND 
                        tRating.ModuleType  is not null and 
                        tRating.ModuleType = 'exam',
                            tRating.SubmoduleRate, 0)
                    ) AS MaxExamRate,
                    SUM(
                    	 iif_sql(tRating.ModuleType  is not null and 
                         tRating.ModuleType != 'exam', tRating.Rate, 0)
                         ) AS RateMExam,
                    MAX(
                    	iif_sql(tRating.ModuleType  is not null and 
                    	       tRating.ModuleType = 'exam' AND 
                    	       tRating.Rate IS NOT NULL, tRating.Rate, 0)
                    	       ) AS RateExam
                FROM (
                    SELECT  tStudentDisciplines.DisciplineID,
                            vr.SubmoduleRate,
                            vr.ModuleType,
                            rt.Rate,
                            vr.SubmoduleIsUsed
                        FROM tStudentDisciplines
                        LEFT JOIN view_roadmap AS vr ON   vr.DisciplineID = tStudentDisciplines.DisciplineID
                        LEFT JOIN rating_table AS rt  ON  rt.RecordBookID = pRecordBookID AND
                                                            rt.SubmoduleID = vr.SubmoduleID
                ) AS tRating
                GROUP BY tRating.DisciplineID
            ) AS tDR
        INNER JOIN view_disciplines ON view_disciplines.DisciplineID = tDR.DisciplineID
        INNER JOIN teachers ON teachers.ID = view_disciplines.AuthorID
        INNER JOIN accounts ON teachers.AccountID = accounts.ID
        ORDER BY view_disciplines.ExamType ASC, view_disciplines.SubjectName ASC;
end
 $function$;

CREATE OR REPLACE FUNCTION public.student_getdisciplinestemp(precordbookid integer, psemesterid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
begin
--вспомогательная 
--создает временную таблицу для другой функции	
-- изменила группировку !!!!!
    CREATE TEMPORARY TABLE tStudentDisciplines AS (
        SELECT disc_rb.DisciplineID
        FROM (
            SELECT  view_disciplines_recordbooks.DisciplineID,
                    view_disciplines_recordbooks.RecordBookID,
                    COALESCE(view_disciplines_recordbooks.Type) AS type
                    FROM view_disciplines_recordbooks
                WHERE view_disciplines_recordbooks.SemesterID = pSemesterID AND
                      view_disciplines_recordbooks.RecordBookID = pRecordBookID
                GROUP BY view_disciplines_recordbooks.DisciplineID, 
                		view_disciplines_recordbooks.RecordBookID,
                		view_disciplines_recordbooks.Type
            ) AS disc_rb
        where disc_rb.type is null or  disc_rb.Type != 'detach'
    );
    RETURN 0;
END
 $function$;

CREATE OR REPLACE FUNCTION public.student_getidfromexternalid(paccountexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
 DECLARE pID INT DEFAULT null;
BEGIN   
--	select public.student_getidfromexternalid(
--	:paccountexternalid 	-- put the paccountexternalid parameter value instead of 'paccountexternalid' (varchar)
--);
        SELECT students.ID INTO pID
        FROM accounts
        JOIN students ON accounts.ID = students.AccountID
        WHERE accounts.ExternalID = pAccountExternalID
        LIMIT 1;
        RETURN coalesce(pID, -1);
    END
 $function$;

CREATE OR REPLACE FUNCTION public.student_getteacherslist(precordbookid integer, psemesterid integer, ploadall integer)
 RETURNS TABLE("DisciplineID" integer, "TeacherID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying)
 LANGUAGE plpgsql
AS $function$ 
    DECLARE vRes integer; vStudentGroup INT DEFAULT -1;
BEGIN
--select * from public.Student_GetTeachersList(
--	:pRecordBookID,	-- put the pRecordBookID parameter value instead of 'pRecordBookID' (INT)
--	:pSemesterID,	-- put the pSemesterID parameter value instead of 'pSemesterID' (INT)
--	:pLoadAll 	-- put the pLoadAll parameter value instead of 'pLoadAll' (INT)
--);
-- разобраться с параметром pLoadAll - поставила для него проверку ==1
    DROP TABLE IF EXISTS tStudentDisciplines;
     vRes = Student_GetDisciplinesTemp(pRecordBookID, pSemesterID);
   return query
    SELECT  tStudentDisciplines.DisciplineID as "DisciplineID",
            teachers.ID AS "TeacherID",
            accounts.LastName as "LastName",
            accounts.FirstName as "FirstName",
            accounts.SecondName as "SecondName"
        FROM tStudentDisciplines
        INNER JOIN disciplines ON disciplines.ID = tStudentDisciplines.DisciplineID
        LEFT JOIN disciplines_teachers ON disciplines_teachers.DisciplineID = tStudentDisciplines.DisciplineID
        INNER JOIN teachers ON teachers.ID = disciplines_teachers.TeacherID
        INNER JOIN accounts ON teachers.AccountID = accounts.ID
        WHERE pLoadAll=1 OR disciplines.Subtype IS NULL
        ORDER BY    tStudentDisciplines.DisciplineID ASC,
                    accounts.LastName ASC,
                    accounts.FirstName ASC;
END
 $function$;

CREATE OR REPLACE FUNCTION public.subject_create(pfacultyid integer, psubjectname text, psubjectabbr character varying, pexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
    DECLARE vSubjectID INT DEFAULT -1;
begin
--	select public.subject_create(
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psubjectname,	-- put the psubjectname parameter value instead of 'psubjectname' (text)
--	:psubjectabbr,	-- put the psubjectabbr parameter value instead of 'psubjectabbr' (varchar)
--	:pexternalid 	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--);
    IF pSubjectName = '' THEN RETURN -1; END IF;
-- create/get subject (subject name is unique key)
    INSERT INTO subjects ("name", Abbr, ExternalID) VALUES (pSubjectName, pSubjectAbbr, pExternalID)
        ON conflict on constraint subjects_name_key
        do update set
          --  subjects.ID = LAST_INSERT_ID(subjects.ID),
            "name" = pSubjectName,
            Abbr = COALESCE(pSubjectAbbr, subjects.Abbr),
            ExternalID = COALESCE(pExternalID, subjects.ExternalID)
           returning id into vSubjectID; -- должно вернуть или новый ID или ID измененной записи
    BEGIN --handler block
      
        INSERT INTO subjects_faculties (SubjectID, FacultyID) VALUES (vSubjectID, pFacultyID)
            on conflict on constraint subjects_faculties_subjectid_facultyid_key --just stub
               do nothing; 
     --  EXCEPTION 
     --  when others then RETURN -1;        
    END;
    RETURN vSubjectID;
END
$function$;


CREATE OR REPLACE FUNCTION public.subject_getbyexternalid(psubjectexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vID INT DEFAULT -1;
begin
--	select public.subject_getbyexternalid(
--	:psubjectexternalid 	-- put the psubjectexternalid parameter value instead of 'psubjectexternalid' (varchar)
--);
    SELECT subjects.ID INTO vID
    FROM subjects
    WHERE subjects.ExternalID = pSubjectExternalID
    LIMIT 1;
  if vID is null then return -1;
    else  RETURN vID;
   end if;
END
 $function$;

CREATE OR REPLACE FUNCTION public.submodule_d()
 RETURNS trigger
 LANGUAGE plpgsql
AS $function$ 
    declare vDisciplineID int default 0;
    declare vAccumType boolean default false; 
   			vChangeMax boolean default false;
    declare vModuleType regular_exam_bonus_extra default 'extra';
begin
    -- todo: check repeated bonus
    select modules.Type into vModuleType
        from modules
        where modules.ID = old.ModuleID
        limit 1;

    if (vModuleType is not null)and (vModuleType = 'regular' or vModuleType = 'exam') then 
        vAccumType = ( vModuleType != 'exam');

        -- todo: extract foo
        select disciplines.ID into vDisciplineID
            from modules
                inner join disciplines on disciplines.ID = modules.DisciplineID
            where modules.ID = old.ModuleID
            limit 1;
       if old.OrderNum is null then
       	   vChangeMax = vAccumType;	
       else
           vChangeMax = vAccumType or old.OrderNum = 1;
       end if;

        update disciplines
        set     CurRate = CurRate - iif_sql(old.isUsed=1, old.MaxRate, 0),
                MaxRate = MaxRate - iif_sql(vChangeMax, old.MaxRate, 0)
        where disciplines.ID = vDisciplineID;
    end if;
   return old;
end
 $function$;

CREATE OR REPLACE FUNCTION public.submodule_i()
 RETURNS trigger
 LANGUAGE plpgsql
AS $function$ 
    declare vDisciplineID int default 0; addmax int default 0; addcur int default 0;
    declare vAccumType boolean default false; vChangeMax boolean default false;
    declare vModuleType regular_exam_bonus_extra default 'extra';
begin
	 -- todo: check repeated bonus
	    select modules.Type into vModuleType
        from modules
        where modules.ID = new.ModuleID
        limit 1;
 
    if (vModuleType is not null and(vModuleType = 'regular' or vModuleType = 'exam') )then 
    
	    vAccumType = (vModuleType != 'exam');
       
       select disciplines.ID into vDisciplineID
            from modules
                inner join disciplines on disciplines.ID = modules.DisciplineID
            where modules.ID = new.ModuleID
            limit 1;
           
         if (new.OrderNum is null)  then
         		vChangeMax = vAccumType;
        		else  vChangeMax = vAccumType or  new.OrderNum = 1;
        end if;	
       
        if (new.isused=1) then addcur=new.MaxRate; end if;
        if (vChangeMax) then addmax = new.maxrate; end if;
       
        update disciplines
        set     CurRate = CurRate + addcur,
                MaxRate = MaxRate + addmax
        where disciplines.ID = vDisciplineID;	    
    end if;
   return new;
end
 $function$;

CREATE OR REPLACE FUNCTION public.submodule_u()
 RETURNS trigger
 LANGUAGE plpgsql
AS $function$ 
    declare vDisciplineID int default 0;
    declare vAccumType    boolean default false; 
            vChangeMaxOld boolean default false;
            vChangeMaxNew boolean default false;
    declare vModuleType regular_exam_bonus_extra default 'extra';
begin
/*    -- странная ситуация
 * if old.ModuleID != new.ModuleID then
        signal sqlstate '45000';
    end if;*/

    -- todo: check repeated bonus
    select modules."type" into vModuleType
        from modules
        where modules.ID = new.ModuleID
        limit 1;


    if (vModuleType is not null and (vModuleType = 'regular' or vModuleType = 'exam') )then 
        vAccumType = (vModuleType != 'exam');
       
        -- todo: extract foo
       select disciplines.ID into vDisciplineID
            from modules
                inner join disciplines on disciplines.ID = modules.DisciplineID
            where modules.ID = old.ModuleID
            limit 1;
     if (new.OrderNum is null)  then
			vChangeMaxOld = vAccumType;
			vChangeMaxNew = vAccumType;
     else
     	 vChangeMaxOld = vAccumType or old.OrderNum = 1;
         vChangeMaxNew = vAccumType or new.OrderNum = 1;
     end if;
      
        update disciplines
        set     CurRate = CurRate
                    - iif_sql(old.isUsed=1, old.MaxRate, 0)
                    + iif_sql(new.isUsed=1, new.MaxRate, 0),
                MaxRate = MaxRate
                    - iif_sql(vChangeMaxOld, old.MaxRate, 0)
                    + iif_sql(vChangeMaxNew, new.MaxRate, 0)
        where disciplines.ID = vDisciplineID;
    return new;
   end if;
end
 $function$;

CREATE OR REPLACE FUNCTION public.swapmoduleorder(pteacherid integer, pmoduleid1 integer, pmoduleid2 integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vChecker INT DEFAULT -1; vOrder1 INT DEFAULT -1; vOrder2 INT DEFAULT -1;
            vDisciplineID1 INT DEFAULT Null; vDisciplineID2 INT DEFAULT Null;
BEGIN
--select public.swapmoduleorder(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pmoduleid1,	-- put the pmoduleid1 parameter value instead of 'pmoduleid1' (int4)
--	:pmoduleid2 	-- put the pmoduleid2 parameter value instead of 'pmoduleid2' (int4)
--);

    -- get disciplineID and orderNum for 1st module(pModuleID1)
    SELECT  modules.OrderNum,
            modules.DisciplineID
        INTO vOrder1, vDisciplineID1
        FROM modules
        INNER JOIN disciplines ON disciplines.ID = modules.DisciplineID
        WHERE   disciplines.AuthorID = pTeacherID AND
                modules.ID = pModuleID1 AND
                modules."type" = 'regular'
        LIMIT 1;

    -- get disciplineID and orderNum for 2st module(pModuleID2)
    SELECT  modules.OrderNum,
            modules.DisciplineID
        INTO vOrder2, vDisciplineID2
        FROM modules
        INNER JOIN disciplines ON disciplines.ID = modules.DisciplineID
        WHERE   disciplines.AuthorID = pTeacherID AND
                modules.ID = pModuleID2 AND
                modules."type" = 'regular'
        LIMIT 1;

    -- check that modules belong to one discipline, check rights
    IF  vDisciplineID1 is  null or vDisciplineID1 != vDisciplineID2 OR 
        InternalIsMapLocked(vDisciplineID1) THEN
        RETURN -1;
    END IF;

    -- swap
    UPDATE modules
        SET modules.OrderNum = 271828
        WHERE modules.ID = pModuleID1;
    UPDATE modules
        SET modules.OrderNum = vOrder1
        WHERE modules.ID = pModuleID2;
        --LIMIT 1;
    UPDATE modules
        SET modules.OrderNum = vOrder2
        WHERE modules.ID = pModuleID1;
        --LIMIT 1;

    RETURN 0;
END;

$function$;

CREATE OR REPLACE FUNCTION public.swapsubmoduleorder(pteacherid integer, psubmoduleid1 integer, psubmoduleid2 integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vDisciplineID INT DEFAULT -1; vOrder1 INT DEFAULT -1; vOrder2 INT DEFAULT -1;
            vModule1 INT default null; vModule2 INT DEFAULT null;
BEGIN
--select public.swapsubmoduleorder(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psubmoduleid1,	-- put the psubmoduleid1 parameter value instead of 'psubmoduleid1' (int4)
--	:psubmoduleid2 	-- put the psubmoduleid2 parameter value instead of 'psubmoduleid2' (int4)
--);

    SELECT  submodules.OrderNum,
            submodules.ModuleID,
            disciplines.ID
        INTO vOrder1, vModule1, vDisciplineID
        FROM submodules
        INNER JOIN modules        ON  submodules.ModuleID = modules.ID
        INNER JOIN disciplines    ON  disciplines.ID = modules.DisciplineID
        WHERE   disciplines.AuthorID = pTeacherID AND
                submodules.ID = pSubmoduleID1
        LIMIT 1;

    SELECT  submodules.OrderNum,
            submodules.ModuleID
        INTO vOrder2, vModule2
        FROM submodules
        INNER JOIN modules        ON  submodules.ModuleID = modules.ID
        INNER JOIN disciplines    ON  disciplines.ID = modules.DisciplineID
        WHERE   disciplines.AuthorID = pTeacherID AND
                submodules.ID = pSubmoduleID2
        LIMIT 1;

    -- check, that modules exists and belong to one discipline
    IF  vModule1  is null OR vModule1 != vModule2 OR
        InternalIsMapLocked(vDisciplineID)
    THEN
        RETURN -1;
    END IF;

    -- swap
    UPDATE submodules
        SET submodules.OrderNum     = 271828
        WHERE submodules.ID = pSubmoduleID1;
        --LIMIT 1;
    UPDATE submodules
        SET submodules.OrderNum     = vOrder1
        WHERE submodules.ID = pSubmoduleID2;
       -- LIMIT 1;
    UPDATE submodules
        SET submodules.OrderNum     = vOrder2
        WHERE submodules.ID = pSubmoduleID1;
        --LIMIT 1;
    RETURN 0;
END;

$function$;

CREATE OR REPLACE FUNCTION public.teacher_create(plastname character varying, pfirstname character varying, psecondname character varying, pjobpositionid integer, pdepartmentid integer, pactivationcode character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$

    DECLARE vAccountID INT DEFAULT -1;
   			vID integer;
begin
--	select public.teacher_create(
--	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
--	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
--	:psecondname,	-- put the psecondname parameter value instead of 'psecondname' (varchar)
--	:pjobpositionid,	-- put the pjobpositionid parameter value instead of 'pjobpositionid' (int4)
--	:pdepartmentid,	-- put the pdepartmentid parameter value instead of 'pdepartmentid' (int4)
--	:pactivationcode 	-- put the pactivationcode parameter value instead of 'pactivationcode' (varchar)
--);

      vAccountID := CreateAccount(pActivationCode, GetUserRole('teacher'));
   -- add new teacher
    INSERT INTO teachers (AccountID, LastName, FirstName, SecondName, JobPositionID, DepartmentID) VALUES
        (vAccountID, pLastName, pFirstName, pSecondName, pJobPositionID, pDepartmentID) returning vID;
    RETURN vID;
  --EXCEPTION 
 -- when others then
  RETURN -1;
   
END;
$function$;

CREATE OR REPLACE FUNCTION public.teacher_createactivated(plastname character varying, pfirstname character varying, psecondname character varying, pjobpositionid integer, pfacultyid integer, pdepartmentid integer, pexternalid character varying, pinila character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
        DECLARE vAccountID INT DEFAULT -1;
       			insID int;
BEGIN
--select public.teacher_createactivated(
--	:plastname,	-- put the plastname parameter value instead of 'plastname' (varchar)
--	:pfirstname,	-- put the pfirstname parameter value instead of 'pfirstname' (varchar)
--	:psecondname,	-- put the psecondname parameter value instead of 'psecondname' (varchar)
--	:pjobpositionid,	-- put the pjobpositionid parameter value instead of 'pjobpositionid' (int4)
--	:pdepartmentid,	-- put the pdepartmentid parameter value instead of 'pdepartmentid' (int4)
--	:pexternalid 	-- put the pexternalid parameter value instead of 'pexternalid' (varchar)
--);
        vAccountID = Account_CreateActivated(pLastName, pFirstName, pSecondName, pExternalID, pINILA, GetUserRole('teacher'));

        -- add new teacher
               INSERT INTO teachers (AccountID, JobPositionID, DepartmentID, FacultyID) VALUES
            (vAccountID, pJobPositionID, pDepartmentID, pFacultyID) returning ID into insID;
        RETURN insID;
      -- EXCEPTION 
      --  when others then RETURN -1;
    END;
$function$;

CREATE OR REPLACE FUNCTION public.teacher_getidfromexternalid(pteacherexternalid character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
        DECLARE pID INT DEFAULT -1;
begin
--	select public.teacher_getidfromexternalid(
--	:pteacherexternalid 	-- put the pteacherexternalid parameter value instead of 'pteacherexternalid' (varchar)
--);
        SELECT teachers.ID INTO pID
        FROM teachers
            INNER JOIN accounts ON teachers.AccountID = accounts.ID
        WHERE accounts.ExternalID = pTeacherExternalID
        LIMIT 1; -- скорее всего не требуется
        RETURN pID;
    END;
$function$;

CREATE OR REPLACE FUNCTION public.teacher_getidfrominila(pteacherinila character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
        DECLARE pID INT DEFAULT null;
begin
--	select public.Teacher_GetIDFromINILA(
--	:pTeacherINILA 	-- put the pTeacherINILA parameter value instead of 'pTeacherINILA' (VARCHAR(40))
--);
        SELECT teachers.ID INTO pID
        FROM teachers
            INNER JOIN accounts ON teachers.AccountID = accounts.ID
        WHERE accounts.INILA = pTeacherINILA
        LIMIT 1;
        if pID is null then pID = -1;
        end if;
        RETURN pID;
    END
 $function$;

CREATE OR REPLACE FUNCTION public.unbindgroup(pteacherid integer, pdisciplineid integer, pgroupid integer, vres integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
declare vRes int;
begin
--	select public.unbindgroup(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pgroupid,	-- put the pgroupid parameter value instead of 'pgroupid' (int4)
--	:vres 	-- put the vres parameter value instead of 'vres' (int4)
--);

    -- todo: move to php layer
    CASE WHEN NOT InternalIsTeacherBound(pTeacherID, pDisciplineID) OR InternalIsMapLocked(pDisciplineID) THEN
        RETURN -1;
		else null;
    END CASE;
    -- detach group from the discipline
    DELETE FROM disciplines_groups
        WHERE   disciplines_groups.DisciplineID = pDisciplineID AND disciplines_groups.GroupID = pGroupID;
        --LIMIT 1;

    SELECT LogBind(pDisciplineID, pTeacherID, pGroupID, 'detach', 'group') INTO vRes;
    RETURN RemoveStudentsAttach(pDisciplineID, pGroupID);
END
$function$;

CREATE OR REPLACE FUNCTION public.unbindstudent(pteacherid integer, pdisciplineid integer, precordbookid integer)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$ 
   DECLARE vInGroup int DEFAULT -1;  
  		   vStudentGroupID int DEFAULT -1; 
  		   vSemesterID INT DEFAULT -1;
BEGIN
--select public.unbindstudent(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:precordbookid 	-- put the precordbookid parameter value instead of 'precordbookid' (int4)
--);

-- todo: move to php layer
    IF pTeacherID IS NOT NULL AND NOT InternalIsTeacherBound(pTeacherID, pDisciplineID) THEN
        RETURN -1;
    END IF;

    vSemesterID = GetDisciplineProperty(pDisciplineID, 'semester');
    vStudentGroupID = GetStudentGroup(pRecordBookID, vSemesterID);

    --2. check if student's group is bound yet
     vInGroup = EXISTS(
        SELECT * FROM disciplines_groups
        WHERE disciplines_groups.DisciplineID = pDisciplineID AND
              disciplines_groups.GroupID = vStudentGroupID
        LIMIT 1
    );

    IF vInGroup > 0 THEN --student in general group
        INSERT INTO disciplines_students (DisciplineID, RecordBookID, Type)
            VALUES (pDisciplineID, pRecordBookID, 'detach')
            ON conflict on constraint disciplines_students_recordbookid_disciplineid_key do 
            UPDATE  set disciplines_students.Type = 'detach';
    ELSE
        DELETE FROM disciplines_students
            WHERE   disciplines_students.DisciplineID = pDisciplineID AND
                    disciplines_students.RecordBookID = pRecordBookID
            ;
    END IF;
    RETURN LogBind(pDisciplineID, pTeacherID, pRecordBookID, 'detach', 'student');
   --EXCEPTION 
  -- when others then RETURN -1;
END
 $function$;

CREATE OR REPLACE FUNCTION public.updaterequest(prequestid integer, ptitle character varying, pdescription text)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE retValue INT DEFAULT -1;
BEGIN
    --DECLARE EXIT HANDLER FOR SQLEXCEPTION RETURN -1; непонятно зачем обрабочик,ошибки быть не может
--  select public.UpdateRequest(
--	:pRequestID,	-- put the pRequestID parameter value instead of 'pRequestID' (INT)
--	:pTitle,	-- put the pTitle parameter value instead of 'pTitle' (VARCHAR(50))
--	:pDescription 	-- put the pDescription parameter value instead of 'pDescription' (CHAR(1))
--);
    UPDATE requests
        SET requests.Description = pDescription,
            requests.Title = pTitle,
            requests.Date = NOW()
        WHERE   requests.ID = pRequestID AND
                requests.Description IS NULL AND
                requests.Title IS NULL;
        --LIMIT 1; -- запрос через id не может вернуть больше 1 строки
    --RETURN ROW_COUNT()-1;
    GET diagnostics retValue=ROW_COUNT;
   return retValue-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public.userecoverytoken(ptoken character varying)
 RETURNS integer
 LANGUAGE plpgsql
AS $function$
    DECLARE vChecker INT DEFAULT -1;
BEGIN
    -- set token used
	-- select public.userecoverytoken(
	-- :ptoken 	-- put the ptoken parameter value instead of 'ptoken' (varchar)
    --);
    UPDATE recovery_tokens
        SET IsUsed = 1
        WHERE Token = pToken;
      -- LIMIT 1;  не нужен, т.к. token имеет unique key
      GET diagnostics vChecker=ROW_COUNT; --другого способа не нашла
   -- RETURN ROW_COUNT-1;
   return vchecker-1;
END;
$function$;

CREATE OR REPLACE FUNCTION public."GetRatesHistory"(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "Date" date, "Rate" integer, "StudentLast" character varying, "StudentFirst" character varying, "StudentSecond" character varying, "TeacherLast" character varying, "TeacherFirst" character varying, "TeacherSecond" character varying, "SubmoduleName" character varying, "ModuleName" character varying)
 LANGUAGE sql
AS $function$
  SELECT  logs_rating.ID AS "ID",
    DATE(logs_rating.Date) As "Date",
    logs_rating.Rate AS "Rate",
    studentAccount.LastName   AS "StudentLast",
    studentAccount.FirstName  AS "StudentFirst",
    studentAccount.SecondName AS "StudentSecond",
    teacherAccount.LastName   AS "TeacherLast",
    teacherAccount.FirstName  AS "TeacherFirst",
    teacherAccount.SecondName AS "TeacherSecond",
    submodules.Name     AS "SubmoduleName",
    modules.Name        AS "ModuleName"
  FROM logs_rating
    INNER JOIN record_books ON record_books.ID = logs_rating.RecordBookID
    INNER JOIN students ON students.ID = record_books.StudentID
    INNER JOIN accounts as studentAccount ON students.AccountID = studentAccount.ID
    INNER JOIN teachers ON teachers.ID = logs_rating.TeacherID
    INNER JOIN accounts as teacherAccount ON teachers.AccountID = teacherAccount.ID
    INNER JOIN submodules ON submodules.ID = logs_rating.SubmoduleID
    INNER JOIN modules ON modules.ID = submodules.ModuleID
  WHERE modules.DisciplineID = pDisciplineID
  ORDER BY logs_rating.Date DESC
;
$function$;

CREATE OR REPLACE FUNCTION public."Plan_GetTeachers"(pstudyplanid integer, psemesterid integer, ploadall integer)
 RETURNS TABLE("ID" integer)
 LANGUAGE sql
AS $function$
--select * from public.getgroups(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT teachers.ID AS "ID"
FROM teachers
  INNER JOIN disciplines_teachers ON disciplines_teachers.TeacherID = teachers.ID
  INNER JOIN disciplines ON disciplines.ID = disciplines_teachers.DisciplineID
  INNER JOIN disciplines_study_plans ON disciplines_study_plans.DisciplineID = disciplines.ID
WHERE disciplines_study_plans.StudyPlanID = pStudyPlanID AND
      disciplines.SemesterID = COALESCE(pSemesterID, disciplines.SemesterID) AND
      (pLoadAll =  1 OR disciplines.Subtype IS NULL);
$function$;

CREATE OR REPLACE FUNCTION public.department_getidbyexternalid(pdepartmentexternalid character varying)
 RETURNS integer
 LANGUAGE sql
AS $function$
--select public.department_getidbyexternalid(
--	:pdepartmentexternalid 	-- put the pdepartmentexternalid parameter value instead of 'pdepartmentexternalid' (varchar)
--);
  SELECT departments.ID as ID
  FROM departments
  WHERE departments.ExternalID = pDepartmentExternalID; --поле unique
$function$;

CREATE OR REPLACE FUNCTION public.department_getidbyname(pdepartmentname character varying)
 RETURNS SETOF integer
 LANGUAGE sql
AS $function$
--	select public.department_getidbyname(
--	:pdepartmentname 	-- put the pdepartmentname parameter value instead of 'pdepartmentname' (varchar)
--);
  SELECT departments.ID as ID
  FROM departments
  WHERE departments.Name = pDepartmentName;
$function$;

CREATE OR REPLACE FUNCTION public.discipline_countratings(pdisciplineid integer)
 RETURNS bigint
 LANGUAGE sql
AS $function$
--select public.discipline_countratings(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    SELECT COUNT(rating_table.recordbookid) FROM view_roadmap
        LEFT JOIN rating_table ON rating_table.SubmoduleID = view_roadmap.SubmoduleID
        WHERE view_roadmap.DisciplineID = pDisciplineID
$function$;

CREATE OR REPLACE FUNCTION public.discipline_find(pstudyplanid integer, psemesterid integer, psubjectid integer, pexamtype exam_credit_grading_credit)
 RETURNS SETOF integer
 LANGUAGE sql
AS $function$
--select public.discipline_find(
--	:pstudyplanid,	-- put the pstudyplanid parameter value instead of 'pstudyplanid' (int4)
--	:psemesterid,	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--	:psubjectid,	-- put the psubjectid parameter value instead of 'psubjectid' (int4)
--	:pexamtype 	-- put the pexamtype parameter value instead of 'pexamtype' (exam_credit_grading_credit)
--);
	SELECT disciplines.ID
    FROM disciplines
      INNER JOIN disciplines_study_plans ON disciplines.ID = disciplines_study_plans.DisciplineID
    WHERE disciplines_study_plans.StudyPlanID = pStudyPlanID AND
          disciplines.SemesterID = pSemesterID AND
          disciplines.SubjectID = pSubjectID AND
          disciplines.ExamType = pExamType;
 $function$;

CREATE OR REPLACE FUNCTION public.discipline_getwithfullness(ptype noukd_norate_full, psemesterid integer, pfacultyid integer)
 RETURNS SETOF integer
 LANGUAGE sql
AS $function$ 
--select public.discipline_getwithfullness(
--	:ptype,	-- put the ptype parameter value instead of 'ptype' (noukd_norate_full)
--	:psemesterid,	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--	:pfacultyid 	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--);

SELECT view_disciplines.DisciplineID AS ID
        FROM view_disciplines
        WHERE view_disciplines.SemesterID = pSemesterID AND
              view_disciplines.FacultyID = pFacultyID AND
              CASE WHEN pType = 'noukd' 
                   THEN (view_disciplines.MaxRate != 100 AND view_disciplines.IsLocked = 0) 
                   ELSE true
              END 
              AND
              CASE WHEN pType = 'norate' 
              	THEN (view_disciplines.MaxRate = 100 AND view_disciplines.IsLocked = 0) 
              	ELSE TRUE 
              END 
              AND
              CASE WHEN pType = 'full' 
             	   THEN (view_disciplines.MaxRate = 100 AND view_disciplines.IsLocked = 1) 
                   ELSE TRUE 
               END;
 $function$;

CREATE OR REPLACE FUNCTION public.getauthtokens(paccountid integer)
 RETURNS TABLE("AccountID" integer, "Created" timestamp without time zone, "Accessed" timestamp without time zone, "Mask" integer)
 LANGUAGE sql
AS $function$
-- select * from public.getauthtokens(:paccountid 	-- put the paccountid parameter value instead of 'paccountid' (int4));
-- если вызывать select  public.getauthtokens(...); выдает таблицу с одним солбцом - все данные склены в список
    SELECT accountID as "AccountID", 
           created as "Created", 
           accessed as "Accessed", 
           mask as "Mask"
    FROM auth_tokens
    INNER JOIN accounts ON accounts.ID = auth_tokens.AccountID
    WHERE
        auth_tokens.AccountID = pAccountID OR
        pAccountID = 0
    ORDER BY auth_tokens.Created DESC;
$function$;

CREATE OR REPLACE FUNCTION public.getbonusmodule(pdisciplineid integer)
 RETURNS integer
 LANGUAGE sql
AS $function$
-- select public.getbonusmodule(:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4));
-- limit 1 оставлено, хотя ситуация с возникновением в одной дисциплине нескольких бонусных модулей сомнительная,
-- может это нужно контролировать ограничением на таблицу
    SELECT modules.ID FROM modules
        WHERE modules.DisciplineID = pDisciplineID AND modules.Type = 'bonus'
        LIMIT 1;
$function$;



CREATE OR REPLACE FUNCTION public.getdepartmentidbyname(pdepartmentname character varying)
 RETURNS integer
 LANGUAGE sql
AS $function$
-- select public.getdepartmentidbyname(	:pdepartmentname 	-- put the pdepartmentname parameter value instead of 'pdepartmentname' (varchar));
-- видимо расчитано на уникальность имени кафедры, возможно не используется, или нужно добавить еще параметр - факультет
    SELECT departments.ID as ID
    FROM departments
    WHERE departments.Name = pDepartmentName;
$function$;

CREATE OR REPLACE FUNCTION public.getdisciplines(pfacultyid integer, psemesterid integer)
 RETURNS TABLE("ID" integer, "SubjectID" integer, "SubjectName" character varying, "Type" exam_credit_grading_credit, "isMapCreated" boolean)
 LANGUAGE sql
AS $function$
--select * from public.getdisciplines(
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
    SELECT  view_disciplines.DisciplineID AS "ID",
            view_disciplines.SubjectID as "SubjectID",
            view_disciplines.SubjectName as "SubjectName",
            view_disciplines.ExamType AS "Type",
            (view_disciplines.MaxRate = 100) AS "isMapCreated"
        FROM view_disciplines
        WHERE   view_disciplines.SemesterID = pSemesterID AND
                view_disciplines.FacultyID = pFacultyID
        ORDER BY view_disciplines.SubjectName ASC;
$function$;

CREATE OR REPLACE FUNCTION public.getdisciplinesforteacher(pteacherid integer, psemesterid integer)
 RETURNS TABLE("ID" integer, "Type" exam_credit_grading_credit, "SubType" scientific_disciplinary_coursework, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "GroupID" integer, "GroupNum" integer, "GroupName" character varying, "SubjectID" integer, "SubjectName" character varying, "AuthorID" integer, "IsLocked" integer, "IsMapCreated" boolean)
 LANGUAGE sql
AS $function$
--select public.getdisciplinesforteacher(
--	:pteacherid,	-- put the pteacherid parameter value instead of 'pteacherid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
    SELECT DISTINCT view_disciplines.DisciplineID AS "ID",
                view_disciplines.ExamType AS "Type",
                view_disciplines.Subtype as "SubType",
                view_disciplines.GradeID as "GradeID",
                view_disciplines.GradeNum as "GradeNum",
                view_disciplines."degree" as "Degree",
                view_groups.GroupID as "GroupID",
                view_groups.GroupNum as "GroupNum",
                view_groups.GroupName as "GroupName",
                view_disciplines.SubjectID as "SubjectID",
                view_disciplines.SubjectName as "SubjectName",
                view_disciplines.AuthorID as "AuthorID",
                view_disciplines.IsLocked AS "IsLocked", -- bodging, db schema remembered lowerCase
                (view_disciplines.MaxRate = 100)    AS "IsMapCreated"
    FROM disciplines_teachers
    INNER JOIN view_disciplines ON disciplines_teachers.DisciplineID = view_disciplines.DisciplineID
    INNER JOIN semesters ON semesters.ID = view_disciplines.SemesterID
    LEFT JOIN disciplines_groups ON disciplines_groups.DisciplineID = disciplines_teachers.DisciplineID
    LEFT JOIN view_groups ON view_groups.GroupID = disciplines_groups.GroupID AND view_groups.Year = semesters.Year
    WHERE disciplines_teachers.TeacherID = pTeacherID AND view_disciplines.SemesterID = pSemesterID
    ORDER BY    view_disciplines.GradeID ASC,
                view_disciplines.SubjectName ASC,
                view_disciplines.DisciplineID ASC,
                view_groups.GroupNum ASC;
$function$;

CREATE OR REPLACE FUNCTION public.getdisciplinetypeexam(pdisciplineid integer)
 RETURNS exam_credit_grading_credit
 LANGUAGE sql
AS $function$ 
--select public.getdisciplinetypeexam(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
       select examtype 
          from disciplines 
          where id=pdisciplineid;
 $function$;

CREATE OR REPLACE FUNCTION public.getfaculties()
 RETURNS TABLE("ID" integer, "Name" character varying, "Abbr" character varying, "ExternalID" character varying)
 LANGUAGE sql
AS $function$
--select public.getfaculties();
SELECT  --faculties.* -- ID, Abbr, Name
		faculties.ID as "ID",
		faculties."name" as "Name",
		faculties.abbr as "Abbr",
		faculties.externalID as "ExternalID"
    FROM faculties
    ORDER BY faculties.Name ASC;
$function$;

CREATE OR REPLACE FUNCTION public.getfinalforminfo(pdisciplineid integer, pgroupid integer)
 RETURNS TABLE(groupnum integer, groupname character varying, gradeid integer, gradenum integer, degree bachelor_master_specialist, specid integer, specname character varying, specabbr character varying, speccode character varying, facultyid integer, facultyname character varying, facultyabbr character varying, examtype exam_credit_grading_credit, subjectid integer, subjectname character varying, subjectabbr character varying, authorid integer, lastname character varying, firstname character varying, secondname character varying, jobposition character varying, depid integer, depname character varying, year integer, semesternum integer)
 LANGUAGE sql
AS $function$
--select * from public.getfinalforminfo(
--	:pdisciplineid,	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--	:pgroupid 	-- put the pgroupid parameter value instead of 'pgroupid' (int4)
--);
    SELECT  study_groups.GroupNum AS GroupNum,
            groups_years.Name AS GroupName,
            grades.ID AS GradeID,
            grades.Num AS GradeNum,
            grades.Degree AS Degree,
            specializations.ID AS SpecID,
            specializations.Name AS SpecName,
            specializations.Abbr AS SpecAbbr,
            specializations.Code AS SpecCode,
            faculties.ID AS FacultyID,
            faculties.Name AS FacultyName,
            faculties.Abbr AS FacultyAbbr,
            disciplines.ExamType AS ExamType,
            subjects.ID AS SubjectID,
            subjects.Name AS SubjectName,
            subjects.Abbr AS SubjectAbbr,
            teachers.ID AS AuthorID,
            accounts.LastName AS LastName,
            accounts.FirstName AS FirstName,
            accounts.SecondName AS SecondName,
            job_positions.Name AS JobPosition,
            departments.ID AS DepID,
            departments.Name AS DepName,
            semesters.Year AS Year,
            semesters.Num AS SemesterNum
        FROM study_groups
        INNER JOIN grades ON study_groups.GradeID = grades.ID
        INNER JOIN disciplines ON disciplines.ID = pDisciplineID
        INNER JOIN subjects ON disciplines.SubjectID = subjects.ID
        INNER JOIN teachers ON teachers.ID = disciplines.AuthorID
        INNER JOIN accounts ON teachers.AccountID = accounts.ID
        INNER JOIN departments ON departments.ID = teachers.DepartmentID
        INNER JOIN job_positions ON job_positions.ID = teachers.JobPositionID
        INNER JOIN semesters ON disciplines.SemesterID = semesters.ID
        INNER JOIN groups_years ON groups_years.GroupID = study_groups.ID AND groups_years.Year = semesters.Year
        INNER JOIN specializations ON groups_years.SpecializationID = specializations.ID
        INNER JOIN faculties ON faculties.ID = specializations.FacultyID
        WHERE study_groups.ID = pGroupID
        LIMIT 1;
$function$;

CREATE OR REPLACE FUNCTION public.getgrades()
 RETURNS SETOF grades
 LANGUAGE sql
AS $function$
--select public.getgrades();
SELECT  grades.* -- ID, Num, Degree
    FROM grades
    ORDER BY grades.ID;   
$function$;

CREATE OR REPLACE FUNCTION public.getgroups(pgradeid integer, pfacultyid integer, psemesterid integer)
 RETURNS TABLE(id integer, groupnum integer, specid integer, specname character varying, specabbr character varying)
 LANGUAGE sql
AS $function$
--select * from public.getgroups(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT  view_groups.GroupID AS ID,
  view_groups.GroupNum,
  view_groups.SpecID,
  view_groups.SpecName,
  view_groups.SpecAbbr
FROM view_groups
  INNER JOIN semesters ON semesters.ID = pSemesterID
WHERE   view_groups.GradeID = pGradeID AND
        view_groups.FacultyID = pFacultyID AND
        view_groups.Year = semesters.Year
ORDER BY view_groups.GroupNum ASC;
$function$;

CREATE OR REPLACE FUNCTION public.getgroupsfordiscipline(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "GroupNum" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying)
 LANGUAGE sql
AS $function$
--select * from public.getgroupsfordiscipline(
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    SELECT  view_groups.GroupID AS "ID",
            view_groups.GroupNum as "GroupNum",
            view_groups.GradeID as "GradeID",
            view_groups.GradeNum as "GradeNum",
            view_groups."degree" as "Degree",
            view_groups.SpecID as "SpecID",
            view_groups.SpecName as "SpecName",
            view_groups.SpecAbbr as "SpecAbbr"
        FROM disciplines_groups
        INNER JOIN disciplines ON disciplines_groups.DisciplineID = disciplines.ID
        INNER JOIN semesters ON disciplines.SemesterID = semesters.ID
        INNER JOIN view_groups ON disciplines_groups.GroupID = view_groups.GroupID
        WHERE   disciplines_groups.DisciplineID = pDisciplineID
                AND view_groups."year" = semesters."year"
        ORDER BY view_groups.GradeID ASC, view_groups.GroupID ASC;

$function$;

CREATE OR REPLACE FUNCTION public.getjobpositionidbyname(pjobpositionname character varying)
 RETURNS integer
 LANGUAGE sql
AS $function$
--select public.getjobpositionidbyname(
--	:pjobpositionname 	-- put the pjobpositionname parameter value instead of 'pjobpositionname' (varchar)
--);

    SELECT job_positions.ID as ID
    FROM job_positions
    WHERE job_positions.Name = pJobPositionName;
$function$;

CREATE OR REPLACE FUNCTION public.getjobpositions()
 RETURNS SETOF job_positions
 LANGUAGE sql
AS $function$
--select * from public.getjobpositions();
SELECT  job_positions.* -- ID, Name
    FROM    job_positions
    ORDER BY job_positions.Name;
$function$;

CREATE OR REPLACE FUNCTION public.getratesexam(pstudentid integer, pdisciplineid integer)
 RETURNS TABLE("ModuleID" integer, "ModuleName" character varying, "SubmoduleID" integer, "SubmoduleName" character varying, "MaxRate" integer, "SubmoduleType" current_landmark_control, "Rate" integer, "Date" date, "ModuleType" regular_exam_bonus_extra, "ExamPeriodOption" absence_pass)
 LANGUAGE sql
AS $function$
--select * from public.getratesexam(
--	:pstudentid,	-- put the pstudentid parameter value instead of 'pstudentid' (int4)
--	:pdisciplineid 	-- put the pdisciplineid parameter value instead of 'pdisciplineid' (int4)
--);
    SELECT  view_roadmap.ModuleID  as "ModuleID",
            view_roadmap.ModuleName as "ModuleName",
            view_roadmap.SubmoduleID as "SubmoduleID",
            view_roadmap.SubmoduleName as "SubmoduleName",
            view_roadmap.SubmoduleRate AS "MaxRate",
            view_roadmap.SubmoduleType as "SubmoduleType",
            rating_table.Rate as "Rate",
            rating_table."date"  as "Date",
            view_roadmap.ModuleType "ModuleType",
            exam_period_options."type" As "ExamPeriodOption"
    FROM view_roadmap
    LEFT JOIN rating_table    ON  rating_table.SubmoduleID = view_roadmap.SubmoduleID AND
                                    rating_table.recordbookid = pStudentID
    LEFT JOIN exam_period_options ON  exam_period_options.recordbookid = pStudentID AND
                                        exam_period_options.SubmoduleID = view_roadmap.SubmoduleID
    WHERE view_roadmap.DisciplineID = pDisciplineID
    ORDER BY    view_roadmap.ModuleOrderNum ASC,
                view_roadmap.SubmoduleOrderNum ASC;
$function$;

CREATE OR REPLACE FUNCTION public.getrateshistory(pdisciplineid integer)
 RETURNS TABLE("ID" integer, "Date" date, "Rate" integer, "StudentLast" character varying, "StudentFirst" character varying, "StudentSecond" character varying, "TeacherLast" character varying, "TeacherFirst" character varying, "TeacherSecond" character varying, "SubmoduleName" character varying, "ModuleName" character varying)
 LANGUAGE sql
AS $function$
  SELECT  logs_rating.ID AS "ID",
    DATE(logs_rating.Date) As "Date",
    logs_rating.Rate AS "Rate",
    studentAccount.LastName   AS "StudentLast",
    studentAccount.FirstName  AS "StudentFirst",
    studentAccount.SecondName AS "StudentSecond",
    teacherAccount.LastName   AS "TeacherLast",
    teacherAccount.FirstName  AS "TeacherFirst",
    teacherAccount.SecondName AS "TeacherSecond",
    submodules.Name     AS "SubmoduleName",
    modules.Name        AS "ModuleName"
  FROM logs_rating
    INNER JOIN record_books ON record_books.ID = logs_rating.RecordBookID
    INNER JOIN students ON students.ID = record_books.StudentID
    INNER JOIN accounts as studentAccount ON students.AccountID = studentAccount.ID
    INNER JOIN teachers ON teachers.ID = logs_rating.TeacherID
    INNER JOIN accounts as teacherAccount ON teachers.AccountID = teacherAccount.ID
    INNER JOIN submodules ON submodules.ID = logs_rating.SubmoduleID
    INNER JOIN modules ON modules.ID = submodules.ModuleID
  WHERE modules.DisciplineID = pDisciplineID
  ORDER BY logs_rating.Date DESC
;
$function$;

CREATE OR REPLACE FUNCTION public.getrecoveryinfobyemail(pemail character varying)
 RETURNS TABLE("ID" integer, "AccountID" integer, "Date" timestamp without time zone, "Token" character varying, "IsUsed" smallint)
 LANGUAGE sql
AS $function$
--select * from public.getrecoveryinfobyemail(
--	:pemail 	-- put the pemail parameter value instead of 'pemail' (varchar)
--);

    SELECT  recovery_tokens.ID as "ID",
            recovery_tokens.AccountID as "AccountID",
            recovery_tokens."date" as "Date",
            recovery_tokens.Token as "Token",
            recovery_tokens.IsUsed as "IsUsed"
        FROM accounts
        INNER JOIN recovery_tokens ON recovery_tokens.AccountID = accounts.ID
        WHERE   accounts.EMail = pEMail AND
                recovery_tokens.IsUsed = 0
        LIMIT 1;
$function$;

CREATE OR REPLACE FUNCTION public.getrecoveryinfobytoken(ptoken character varying)
 RETURNS TABLE("ID" integer, "AccountID" integer, "Date" timestamp without time zone, "Token" character varying, "IsUsed" smallint)
 LANGUAGE sql
AS $function$
--select * from public.getrecoveryinfobytoken(
--	:ptoken 	-- put the ptoken parameter value instead of 'ptoken' (varchar)
--);
    SELECT  recovery_tokens.ID as "ID",
            recovery_tokens.AccountID as "AccountID",
            recovery_tokens."date" as "Date",
            recovery_tokens.Token as "Token",
            recovery_tokens.IsUsed as "IsUsed"
        FROM recovery_tokens
        WHERE recovery_tokens.Token = pToken
        LIMIT 1;
$function$;

CREATE OR REPLACE FUNCTION public.getroles()
 RETURNS TABLE("ID" integer, "RoleMark" integer, "RoleName" character varying)
 LANGUAGE sql
AS $function$

    SELECT
      user_roles.ID       AS "ID",
      user_roles.Mark     AS "RoleMark",
      user_roles.RoleName AS "RoleName"
    FROM user_roles
    WHERE user_roles."type"='teacher';

$function$;

CREATE OR REPLACE FUNCTION public.getsemestersinfo(psemesterid integer)
 RETURNS TABLE("ID" integer, "Year" integer, "Num" integer)
 LANGUAGE sql
AS $function$
-- select public.getsemestersinfo(:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4));
SELECT  
     semesters.id as "ID",
     semesters."year" as "Year",
     semesters.num as "Num"
    FROM semesters
    WHERE  case when psemesterid <> 0 then semesters.ID = psemesterid else true end
    ORDER BY semesters.ID DESC;
$function$;

CREATE OR REPLACE FUNCTION public.iif_sql(boolean, character varying, character varying)
 RETURNS character varying
 LANGUAGE sql
 IMMUTABLE
AS $function$ 
SELECT CASE $1 WHEN TRUE THEN $2 ELSE $3 END 
$function$;

CREATE OR REPLACE FUNCTION public.iif_sql(boolean, integer, integer)
 RETURNS integer
 LANGUAGE sql
 IMMUTABLE
AS $function$ 
SELECT CASE $1 WHEN TRUE THEN $2 ELSE $3 END 
$function$;

CREATE OR REPLACE FUNCTION public.jobposition_getidbyname(pjobpositionname character varying)
 RETURNS integer
 LANGUAGE sql
AS $function$
--select public.jobposition_getidbyname(
--	:pjobpositionname 	-- put the pjobpositionname parameter value instead of 'pjobpositionname' (varchar)
--);

  SELECT job_positions.ID as ID
  FROM job_positions
  WHERE job_positions.Name = pJobPositionName;

$function$;

CREATE OR REPLACE FUNCTION public.jobpositions_get()
 RETURNS SETOF job_positions
 LANGUAGE sql
AS $function$
--select public.jobpositions_get();
SELECT  job_positions.* -- ID, Name
  FROM    job_positions
  ORDER BY job_positions.Name;
$function$;

CREATE OR REPLACE FUNCTION public.plan_getdisciplines(pstudyplanid integer, psemesterid integer)
 RETURNS SETOF integer
 LANGUAGE sql
AS $function$ 
--select public.plan_getdisciplines(
--	:pstudyplanid,	-- put the pstudyplanid parameter value instead of 'pstudyplanid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);

SELECT disciplines.ID AS ID
      FROM disciplines
      INNER JOIN disciplines_study_plans ON disciplines_study_plans.DisciplineID = disciplines.ID
    WHERE disciplines_study_plans.StudyPlanID = pStudyPlanID AND
          disciplines.SemesterID = COALESCE(pSemesterID, disciplines.SemesterID); --???????
 $function$;

CREATE OR REPLACE FUNCTION public.plan_getinfo(pid integer)
 RETURNS SETOF study_plans
 LANGUAGE sql
AS $function$ 
--select public.plan_getinfo(
--	:pid 	-- put the pid parameter value instead of 'pid' (int4)
--);

SELECT study_plans.*
    FROM study_plans
    WHERE study_plans.ID = pID
    LIMIT 1;
 $function$;

CREATE OR REPLACE FUNCTION public.plan_getteachers(pstudyplanid integer, psemesterid integer, ploadall integer)
 RETURNS TABLE("ID" integer)
 LANGUAGE sql
AS $function$
--select * from public.getgroups(
--	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
--	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
--	:psemesterid 	-- put the psemesterid parameter value instead of 'psemesterid' (int4)
--);
SELECT teachers.ID AS "ID"
FROM teachers
  INNER JOIN disciplines_teachers ON disciplines_teachers.TeacherID = teachers.ID
  INNER JOIN disciplines ON disciplines.ID = disciplines_teachers.DisciplineID
  INNER JOIN disciplines_study_plans ON disciplines_study_plans.DisciplineID = disciplines.ID
WHERE disciplines_study_plans.StudyPlanID = pStudyPlanID AND
      disciplines.SemesterID = COALESCE(pSemesterID, disciplines.SemesterID) AND
      (pLoadAll =  1 OR disciplines.Subtype IS NULL);
$function$;

CREATE OR REPLACE FUNCTION public.recordbook_getinfo(precordbook integer)
 RETURNS TABLE("ID" integer, "ExternalID" character varying, "StudentID" integer)
 LANGUAGE sql
AS $function$ 
--select * from public.recordbook_getinfo(
--	:precordbook 	-- put the precordbook parameter value instead of 'precordbook' (int4)
--);

SELECT
      record_books.ID as "ID",
      record_books.ExternalID as "ExternalID",
      record_books.StudentID as "StudentID"
    FROM record_books
    WHERE record_books.ID = pRecordBook;
 $function$;

CREATE OR REPLACE FUNCTION public.semester_find(pyear integer, psemesternum integer)
 RETURNS TABLE("ID" integer, "Year" integer, "Num" integer)
 LANGUAGE sql
AS $function$ 
select
semesters.id as "ID",
semesters."year" as "Year",
semesters.num as "Num"
  FROM semesters
  WHERE semesters.Year = pYear AND semesters.Num = pSemesterNum
 $function$;

CREATE OR REPLACE FUNCTION public.sha1(password character varying)
 RETURNS character varying
 LANGUAGE sql
AS $function$ 
 SELECT ENCODE(DIGEST(password,'sha1'),'hex');
$function$;

CREATE OR REPLACE FUNCTION public.stats_getsupportrequestfrequency(dlimit date)
 RETURNS TABLE(counted_leads bigint, count_date date)
 LANGUAGE sql
AS $function$

--	select public.stats_getsupportrequestfrequency(
--	:dlimit 	-- put the dlimit parameter value instead of 'dlimit' (date)
--);
SELECT
  count(DATE(requests.Date)) as counted_leads, DATE(requests.Date) as count_date
FROM requests WHERE requests.Date > dLimit
GROUP BY DATE(requests.Date);
 
$function$;

CREATE OR REPLACE FUNCTION public.student_getinfo(pid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "SemesterID" integer, "GroupID" integer, "GroupNum" integer, "GroupName" character varying, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "SpecCode" character varying, "FacultyID" integer, "FacultyName" character varying, "FacultyAbbr" character varying)
 LANGUAGE sql
AS $function$
--select * from public.student_getinfo(
--	:pid 	-- put the pid parameter value instead of 'pid' (int4)
--);

    SELECT
        view_students.StudentID   AS "ID",
        view_students.LastName as "LastName",
        view_students.FirstName as "FirstName",
        view_students.SecondName as "SecondName",
        view_students.AccountID as "AccountID",
        view_students.SemesterID as "SemesterID",
        view_students.GroupID as "GroupID",
        view_students.GroupNum as "GroupNum",
        view_students.GroupName as "GroupName",
        view_students.GradeID as "GradeID",
        view_students.GradeNum as "GradeNum",
        view_students."degree" as "Degree" ,
        view_students.SpecID as "SpecID",
        view_students.SpecName as "SpecName",
        view_students.SpecAbbr as "SpecAbbr",
        view_students.SpecCode as "SpecCode",
        view_students.FacultyID as "FacultyID",
        view_students.FacultyName as "FacultyName",
        view_students.FacultyAbbr as "FacultyAbbr"
    FROM view_students WHERE pID = view_students.StudentID;
    --LIMIT 1;

$function$;

CREATE OR REPLACE FUNCTION public.student_getrecordbookandgroup(pstudentid integer, psemesterid integer)
 RETURNS TABLE("RecordBookID" integer, "GroupID" integer)
 LANGUAGE sql
AS $function$ 
-- select * from Student_GetRecordBookAndGroup(
--	:pStudentID,	-- put the pStudentID parameter value instead of 'pStudentID' (INT)
--	:pSemesterID 	-- put the pSemesterID parameter value instead of 'pSemesterID' (INT)
--);
    SELECT  record_books.ID AS "RecordBookID",
            students_groups.GroupID AS "GroupID"
        FROM students_groups
        JOIN record_books ON record_books.ID = students_groups.RecordBookID
        WHERE record_books.StudentID = pStudentID AND
          students_groups.SemesterID = pSemesterID AND
          students_groups.State != 'expulsion';
 $function$;

CREATE OR REPLACE FUNCTION public.student_getrecordbooks(pstudentid integer)
 RETURNS TABLE("ID" integer, "ExternalID" character varying, "Degree" bachelor_master_specialist)
 LANGUAGE sql
AS $function$ 
--	select  * from Student_GetRecordBooks(
--	:pStudentID 	-- put the pStudentID parameter value instead of 'pStudentID' (INT)
--);

    SELECT DISTINCT
      view_students.RecordBookID       AS "ID",
      view_students.RecordBookNumber       AS "ExternalID",
      view_students."degree" as "Degree"
    FROM view_students
    WHERE view_students.StudentID = pStudentID;
 $function$;

CREATE OR REPLACE FUNCTION public.teacher_getinfo(pid integer)
 RETURNS TABLE("ID" integer, "LastName" character varying, "FirstName" character varying, "SecondName" character varying, "AccountID" integer, "JobPositionID" integer, "JobPositionName" character varying, "DepID" integer, "DepName" character varying, "FacultyID" integer, "FacultyName" character varying, "FacultyAbbr" character varying)
 LANGUAGE sql
AS $function$
--select * from public.teacher_getinfo(
--	:pid 	-- put the pid parameter value instead of 'pid' (int4)
--);

SELECT  view_teachers.TeacherID 	AS "ID",
		view_teachers.lastname 		as "LastName",
		view_teachers.firstname 	as "FirstName",
		view_teachers.secondname 	as "SecondName",
		view_teachers.accountid 	as "AccountID",
		view_teachers.jobpositionid as "JobPositionID",
		view_teachers.jobpositionname as "JobPositionName",
		view_teachers.depid 		as "DepID",
		view_teachers.depname 		as "DepName",
		view_teachers.facultyid 	as "FacultyID",
		view_teachers.facultyname 	as "FacultyName",
		view_teachers.facultyabbr 	as "FacultyAbbr"

    FROM view_teachers WHERE pID = view_teachers.TeacherID;
   -- LIMIT 1;

$function$;

CREATE OR REPLACE FUNCTION public.teacher_getsimilardisciplines(pid integer, psubjectid integer)
 RETURNS TABLE("ID" integer, "SemesterID" integer, "ExamType" exam_credit_grading_credit)
 LANGUAGE sql
AS $function$ 
--select * from public.teacher_getsimilardisciplines(
--	:pid,	-- put the pid parameter value instead of 'pid' (int4)
--	:psubjectid 	-- put the psubjectid parameter value instead of 'psubjectid' (int4)
--);

SELECT  disciplines.ID			as "ID",
        disciplines.SemesterID  as "SemesterID",
        disciplines.ExamType	as "ExamType"
  FROM disciplines_teachers
      JOIN disciplines ON disciplines_teachers.DisciplineID = disciplines.ID
      WHERE disciplines_teachers.TeacherID = pID AND
            disciplines.SubjectID = pSubjectID AND
            disciplines.MaxRate = 100
 $function$;

 -------------------------------
 

  CREATE TRIGGER tr_i_submodule BEFORE INSERT on submodules for each row
    EXECUTE PROCEDURE submodule_i();

  CREATE TRIGGER tr_u_submodule BEFORE INSERT on submodules for each row
    EXECUTE PROCEDURE submodule_u();

  CREATE TRIGGER tr_d_submodule BEFORE INSERT on submodules for each row
    EXECUTE PROCEDURE submodule_d();